;/------------------\
;| Game Fixer       |
;| Core             |
;|                  |
;| Version 0.01     |
;| Date: 29.04.2012 |
;| PureBasic 4.60   |
;|                  |
;| by Thorium       |
;\------------------/

EnableExplicit

#AppTitle = "Game Fixer"

#CoreFileName = "GameFixer Core.dll"

#SharedData_Name = "Game Fixer Shared Data"
#SharedData_Size = 1024

#SystemFolder = "System\"
#PluginFolder = "Plugins\"

Enumeration 1
  #Error_PluginMissing
  #Error_PluginWrongFormat
  #Error_PluginWrongConfig
EndEnumeration

Structure SharedData
  Ready.a
  Error.l
  PlugInsCnt.u
  GfPathLd.u
EndStructure

Structure SharedDataPlugin
  FileLd.a
  ConfigLd.u  
EndStructure

Structure StdProcHook
  ProcAddr.i
  HookProcAddr.i
  SavedCodeBuffer.i
  SavedCodeSize.i
EndStructure

XIncludeFile "BeaEngine32.pb"

Import "BeaEngineCheetah.lib"
  Disasm.i(*Instruction.DisasmStruc)
EndImport

Prototype.i Proto_Plugin_Config(Config.s)

Global *SharedData.SharedData
Global GameFixerPath.s
Global CoreDllHandle.i

Global NewList StdProcHooks.StdProcHook()

Procedure AllocSharedMemory(MemoryName.s, SizeOfSharedData.i)
  
  Define hMapping.i
  
  hMapping = CreateFileMapping_(#INVALID_HANDLE_VALUE, 0, #PAGE_READWRITE, 0, SizeOfSharedData, MemoryName)
  hMapping = MapViewOfFile_(hMapping, #FILE_MAP_WRITE, 0, 0, 0)
  
  ProcedureReturn hMapping

EndProcedure

Procedure FreeSharedMemory(hMapping.i)
  
  UnmapViewOfFile_(hMapping)
  
EndProcedure

Procedure InitCore(hAttachProcessThread.i)
  
  Protected i.i
  Protected *SharedDataPlugin.SharedDataPlugin
  Protected PluginName.s
  Protected PluginConfig.s
  Protected Plugin.i
  Protected PluginConfigProc.Proto_Plugin_Config

  ;wait for AttachProcess thread to finish
  WaitForSingleObject_(hAttachProcessThread, #INFINITE)
  
  ;get handle of core DLL (this DLL)
  CoreDllHandle = GetModuleHandle_(#CoreFileName)
  
  ;set up shared memory to get data from the main program
  *SharedData = AllocSharedMemory(#SharedData_Name, #SharedData_Size)
  
  ;get the path of game fixer from the shared data
  GameFixerPath = PeekS(*SharedData + SizeOf(SharedData), *SharedData\GfPathLd)
  
  ;load plugins and configure them
  
  *SharedDataPlugin = *SharedData + SizeOf(SharedData) + *SharedData\GfPathLd
  
  For i = 1 To *SharedData\PlugInsCnt
    
    PluginName   = PeekS(*SharedDataPlugIn + SizeOf(SharedDataPlugIn), *SharedDataPlugin\FileLd)
    PluginConfig = PeekS(*SharedDataPlugIn + SizeOf(SharedDataPlugIn) + *SharedDataPlugin\FileLd, *SharedDataPlugin\ConfigLd)
    
    Plugin = LoadLibrary_(GameFixerPath + #PluginFolder + PluginName)
    If Plugin = 0
      *SharedData\Error = #Error_PluginMissing
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf
    
    PluginConfigProc = GetProcAddress_(Plugin, "Config")
    If PluginConfigProc = 0
      *SharedData\Error = #Error_PluginWrongFormat
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf
    
    If PluginConfigProc(PluginConfig) = #False
      *SharedData\Error = #Error_PluginWrongConfig
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf

    *SharedDataPlugin = *SharedDataPlugin + SizeOf(SharedDataPlugIn) + *SharedDataPlugin\FileLd + *SharedDataPlugin\ConfigLd
    
  Next
  
  *SharedData\Ready = #True
  
  FreeSharedMemory(*SharedData)
  
EndProcedure

ProcedureDLL AttachProcess(Instance.i)

  Protected hAttachProcessThread.i
  Protected hProcess.i
  
  ;get the handle to the thread to enable the init procedure to wait for it's termination
  
  hProcess = GetCurrentProcess_()
  DuplicateHandle_(hProcess, GetCurrentThread_(), hProcess, @hAttachProcessThread, 0, #False, #DUPLICATE_SAME_ACCESS)

  CreateThread(@InitCore(), hAttachProcessThread)
  
EndProcedure

ProcedureDLL DetachProcess(Instance.i)


EndProcedure

;-------------------------------------------------------------
;- disassembling procedures
;-------------------------------------------------------------

;disassembles a instruction using BeaEngine Cheetah
;does not construct the instruction text
;returns the length of the instruction
ProcedureDLL.i Dasm(*Instruction.DisasmStruc)
  
  ProcedureReturn Disasm(*Instruction)
  
EndProcedure

;-------------------------------------------------------------
;- generic hook procedures
;-------------------------------------------------------------

;installs a hook on a StdCall procedure
;returns a hook ID for calling the original and uninstalling the hook
;returns #False if it fails
ProcedureDLL.i InstallStdProcHook(ProcAddr.i, HookProcAddr.i)
  
  Protected Length.i
  Protected OldProtect.i
  Protected Dummy.i
  Protected HookId.i
  Protected Instruction.DisasmStruc
  
  ;add new element to the hooks list
  HookId = AddElement(StdProcHooks())
  If HookId = 0
    ProcedureReturn #False
  EndIf
  
  ;get length of code to save
  
  Repeat
    Instruction\EIP = ProcAddr + Length
    Length = Length + Disasm(Instruction)
  Until Length >= 5
  
  ;allocate memory to save code that gets overwritten
  
  StdProcHooks()\SavedCodeBuffer = AllocateMemory(Length + 5)
  If StdProcHooks()\SavedCodeBuffer = 0
    DeleteElement(StdProcHooks())
    ProcedureReturn #False
  EndIf

  VirtualProtect_(StdProcHooks()\SavedCodeBuffer, Length + 5, #PAGE_EXECUTE_READWRITE, @Dummy)
  
  ;copy code that gets overwritten
  CopyMemory(ProcAddr, StdProcHooks()\SavedCodeBuffer, Length)
  
  ;write jump to end of buffer that jumps to the original procedure behind the patched code
  PokeA(StdProcHooks()\SavedCodeBuffer + Length, $E9)
  PokeL(StdProcHooks()\SavedCodeBuffer + Length + 1, (ProcAddr + Length) - (StdProcHooks()\SavedCodeBuffer + Length) - 5)
  
  ;set write privilegs of target procedure
  VirtualProtect_(ProcAddr, 5, #PAGE_EXECUTE_READWRITE, @OldProtect)
  
  ;write jump to target procedure that jumps to the hook procedure
  PokeB(ProcAddr,$E9)
  PokeL(ProcAddr + 1, HookProcAddr - ProcAddr - 5)
  
  ;restore access privilegs of target procedure
  VirtualProtect_(ProcAddr, 5, OldProtect, @Dummy)
  
  ;set all values of hook structure
  StdProcHooks()\ProcAddr      = ProcAddr
  StdProcHooks()\HookProcAddr  = HookProcAddr
  StdProcHooks()\SavedCodeSize = Length

  ProcedureReturn HookId
  
EndProcedure

;uninstalls a StdCall hook previously installed with InstallStdProcHook
ProcedureDLL UninstallStdProcHook(HookId.i)
  
  Protected OldProtect.i
  Protected Dummy.i
  
  ChangeCurrentElement(StdProcHooks(), HookId)
  
  ;set write privilegs of target procedure
  VirtualProtect_(StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize, #PAGE_EXECUTE_READWRITE, @OldProtect)
  
  ;copy back saved code
  CopyMemory(StdProcHooks()\SavedCodeBuffer, StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize)
  
  ;restore access privilegs of target procedure
  VirtualProtect_(StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize, OldProtect, @Dummy)
  
  ;free the memory of the saved code
  FreeMemory(StdProcHooks()\SavedCodeBuffer)
  
  DeleteElement(StdProcHooks())
  
EndProcedure

;returns the address of the procedure to call to call the original procedure
ProcedureDLL.i GetOrigStdProcAddr(HookId.i)
  
  ChangeCurrentElement(StdProcHooks(), HookId)
  
  ProcedureReturn StdProcHooks()\SavedCodeBuffer
  
EndProcedure

;-------------------------------------------------------------
;- Win32 API hook
;-------------------------------------------------------------

;installs a hook on a Win32 API function
;returns a hook ID for calling the original and uninstalling the hook
;returns #False if it fails
;use UninstallStdProcHook and GetOrigStdProcAddr with Win32 API hooks
ProcedureDLL.i InstallWin32ApiHook(DllName.s, ProcName.s, HookProcAddr.i)
  
  Protected Dll.i
  Protected ProcAddr.i
  
  Dll = LoadLibrary_(DllName)
  If Dll = 0
    ProcedureReturn #False
  EndIf
  
  ProcAddr = GetProcAddress_(Dll, ProcName)
  If ProcAddr = 0
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn InstallStdProcHook(ProcAddr, HookProcAddr)
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 78
; FirstLine = 52
; Folding = Aw
; EnableXP
; Executable = B:\GameFixer\Core\GameFixer Core.dll
; IncludeVersionInfo
; VersionField0 = 0,1,0,0
; VersionField1 = 0,1,0,0
; VersionField3 = Game Fixer
; VersionField4 = 0.01
; VersionField5 = 0.01
; VersionField9 = Janos David Ommert
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_DLL