;/------------------\
;| Game Fixer       |
;| DirectDraw Hook  |
;|                  |
;| Version 0.01     |
;| Date: 28.06.2012 |
;| PureBasic 4.60   |
;|                  |
;| by Thorium       |
;\------------------/

EnableExplicit

XIncludeFile "ddraw.pbi"
XIncludeFile "GameFixer DirectDraw Hook.pbi"

Structure MethodeHook
  HookId.i
  NewAddr.i
  OldAddr.i
  HookCount.i
  List Objects.i()
EndStructure

Structure Object
  Handle.i
  Type.i
EndStructure

;Holds the procedure hook ID's of the DirectDraw procedure hooks.
Global DirectDrawCreateHookId.i
Global DirectDrawCreateExHookId.i

;Prototype for the original DirectDrawCreate procedure.
Prototype Proto_DirectDrawCreate(*guid.GUID, *ddi.INTEGER, *unknown)
Global DirectDrawCreate.Proto_DirectDrawCreate

;Prototype for the original DirectDrawCreateEx procedure.
Prototype Proto_DirectDrawCreateEx(*guid.GUID, *ddi.INTEGER, *riid.GUID, *unkown)
Global DirectDrawCreateEx.Proto_DirectDrawCreateEx

Global DDrawVersion.i

Global NewList DDrawHookCallbacks.i()
Global DDrawHookCount.i

Global NewList DDrawObjects.Object()

Global NewList DDrawQueryInterfaceHooks.MethodeHook()
Global NewList DDrawReleaseHooks.MethodeHook()
Global NewList DDrawCreateClipperHooks.MethodeHook()
Global NewList DDrawCreatePaletteHooks.MethodeHook()
Global NewList DDrawCreateSurfaceHooks.MethodeHook()
Global NewList DDrawDuplicateSurfaceHooks.MethodeHook()

Prototype Proto_DDrawHookCallback(*Object, Action.i, ObjectType.i)
Prototype Proto_QueryInterface(*this, *riid.GUID, *Object.INTEGER)
Prototype Proto_Release(*this)
Prototype Proto_CreateClipper(*this, dwFlags.i, *lplpDDClipper.INTEGER, *pUnkOuter)
Prototype Proto_CreatePalette(*this, dwFlags.i, *lpColorTable, *lplpDDPalette.INTEGER, *pUnkOuter)
Prototype Proto_CreateSurface(*this, *lpDDSurfaceDesc, *lplpDDSurface.INTEGER, *pUnkOuter)
Prototype Proto_DuplicateSurface(*this, *lpDDSurface, *lplpDupDDSurface.INTEGER)

Declare.i DDraw_QueryInterface_Hook(*this, *riid.GUID, *Object.INTEGER)
Declare.i DDraw_Release_Hook(*this)
Declare.i DDraw_CreateClipper_Hook(*this, dwFlags.i, *lplpDDClipper.INTEGER, *pUnkOuter)
Declare.i DDraw_CreatePalette_Hook(*this, dwFlags.i, *lpColorTable, *lplpDDPalette.INTEGER, *pUnkOuter)
Declare.i DDraw_CreateSurface_Hook(*this, *lpDDSurfaceDesc, *lplpDDSurface.INTEGER, *pUnkOuter)
Declare.i DDraw_DuplicateSurface_Hook(*this, *lpDDSurface, *lplpDupDDSurface.INTEGER)

;Get the object tpye by checking *riid.
Procedure.i GetDDrawObjectType(*riid)
  
  Protected ObjectType.i
  
  If CompareMemory(*riid, ?IID_IDirectDraw, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDraw
  ElseIf CompareMemory(*riid, ?IID_IDirectDraw2, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDraw2
  ElseIf CompareMemory(*riid, ?IID_IDirectDraw4, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDraw4
  ElseIf CompareMemory(*riid, ?IID_IDirectDraw7, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDraw7
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawSurface, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawSurface
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawSurface2, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawSurface2
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawSurface3, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawSurface3
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawSurface4, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawSurface4
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawSurface7, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawSurface7
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawPalette, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawPalette
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawClipper, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawClipper
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawColorControl, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawColorControl
  ElseIf CompareMemory(*riid, ?IID_IDirectDrawGammaControl, SizeOf(GUID)) <> 0
    ObjectType = #GF_DD_IDirectDrawGammaControl
  EndIf
  
  ProcedureReturn ObjectType
  
EndProcedure

;Searches a methode hook list for the list element maching the procedure address.
;Returns #True if found, otherwhise returns #False.
Procedure.i GetMethodeListElement(OldAddr.i, List MethodeHooks.MethodeHook())
  
  ForEach MethodeHooks()
    
    If MethodeHooks()\OldAddr = OldAddr
      ProcedureReturn #True
    EndIf
    
  Next
  
  ProcedureReturn #False
  
EndProcedure

;Searches a methode hook list for the list element maching the object handle.
;Returns #True if found, otherwhise returns #False.
Procedure.i GetMethodeListElementByObject(Object.i, List MethodeHooks.MethodeHook())
  
  ForEach MethodeHooks()
    ForEach MethodeHooks()\Objects()
      
      If MethodeHooks()\Objects() = Object
        ProcedureReturn #True
      EndIf
      
    Next
  Next
  
EndProcedure

;Searches a object list for the list element maching the object handle.
;Returns #True if found, otherwhise returns #False.
Procedure.i GetObjectListElement(*Object, List Objects.Object())
  
  ForEach Objects()
    
    If Objects()\Handle = *Object
      ProcedureReturn #True
    EndIf
    
  Next
  
  ProcedureReturn #False
    
EndProcedure

;Installs a hook on a DirectDraw procedure.
;Returns a HookId, which is valid for GetOrigStdProcAddr and UninstallStdProcHook.
;Returns #False if it fails.
ProcedureDLL.i InstallDDrawProcHook(*Object, MethodeIndex.i, HookProcAddr.i)
  
  ProcedureReturn InstallStdProcHook(PeekI(PeekI(*Object) + (MethodeIndex * SizeOf(INTEGER))), HookProcAddr)

EndProcedure

;Calls back all the plugins that installed a DirectDraw hook.
Procedure CallBackDDrawPlugins(*Object, Action.i, ObjectType.i)
  
  Protected DDrawHookCallback.Proto_DDrawHookCallback
  
  If ObjectType = 0 And Action = #GF_DD_ObjectCreate
    ProcedureReturn
  EndIf
  
  ForEach DDrawHookCallbacks()
    
    DDrawHookCallback = DDrawHookCallbacks()
    DDrawHookCallback(*Object, Action, ObjectType)
    
  Next

EndProcedure

;Installs a hook on a QueryInferface methode.
Procedure DDraw_InstallQueryInterfaceHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_QueryInterface * SizeOf(INTEGER)))
  
  If GetMethodeListElement(OldAddr, DDrawQueryInterfaceHooks()) = #True
    DDrawQueryInterfaceHooks()\HookCount + 1
    AddElement(DDrawQueryInterfaceHooks()\Objects())
    DDrawQueryInterfaceHooks()\Objects() = *Object
  Else
    AddElement(DDrawQueryInterfaceHooks())
    DDrawQueryInterfaceHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_QueryInterface_Hook())
    DDrawQueryInterfaceHooks()\NewAddr   = GetOrigStdProcAddr(DDrawQueryInterfaceHooks()\HookId)
    DDrawQueryInterfaceHooks()\OldAddr   = OldAddr
    DDrawQueryInterfaceHooks()\HookCount = 1
    AddElement(DDrawQueryInterfaceHooks()\Objects())
    DDrawQueryInterfaceHooks()\Objects() = *Object
  EndIf

EndProcedure

;Installs a hook on a Release methode.
Procedure DDraw_InstallReleaseHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_Release * SizeOf(INTEGER)))
  If GetMethodeListElement(OldAddr, DDrawReleaseHooks()) = #True
    DDrawReleaseHooks()\HookCount + 1
    AddElement(DDrawReleaseHooks()\Objects())
    DDrawReleaseHooks()\Objects() = *Object
  Else
    AddElement(DDrawReleaseHooks())
    DDrawReleaseHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_Release_Hook())
    DDrawReleaseHooks()\NewAddr   = GetOrigStdProcAddr(DDrawReleaseHooks()\HookId)
    DDrawReleaseHooks()\OldAddr   = OldAddr
    DDrawReleaseHooks()\HookCount = 1
    AddElement(DDrawReleaseHooks()\Objects())
    DDrawReleaseHooks()\Objects() = *Object
  EndIf

EndProcedure

;Installs a hook on a CreateClipper methode.
Procedure DDraw_InstallCreateClipperHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_CreateClipper * SizeOf(INTEGER)))
  If GetMethodeListElement(OldAddr, DDrawCreateClipperHooks()) = #True
    DDrawCreateClipperHooks()\HookCount + 1
    AddElement(DDrawCreateClipperHooks()\Objects())
    DDrawCreateClipperHooks()\Objects() = *Object
  Else
    AddElement(DDrawCreateClipperHooks())
    DDrawCreateClipperHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_CreateClipper_Hook())
    DDrawCreateClipperHooks()\NewAddr   = GetOrigStdProcAddr(DDrawCreateClipperHooks()\HookId)
    DDrawCreateClipperHooks()\OldAddr   = OldAddr
    DDrawCreateClipperHooks()\HookCount = 1
    AddElement(DDrawCreateClipperHooks()\Objects())
    DDrawCreateClipperHooks()\Objects() = *Object
  EndIf

EndProcedure

;Installs a hook on a CreatePalette methode.
Procedure DDraw_InstallCreatePaletteHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_CreatePalette * SizeOf(INTEGER)))
  If GetMethodeListElement(OldAddr, DDrawCreatePaletteHooks()) = #True
    DDrawCreatePaletteHooks()\HookCount + 1
    AddElement(DDrawCreatePaletteHooks()\Objects())
    DDrawCreatePaletteHooks()\Objects() = *Object
  Else
    AddElement(DDrawCreatePaletteHooks())
    DDrawCreatePaletteHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_CreatePalette_Hook())
    DDrawCreatePaletteHooks()\NewAddr   = GetOrigStdProcAddr(DDrawCreatePaletteHooks()\HookId)
    DDrawCreatePaletteHooks()\OldAddr   = OldAddr
    DDrawCreatePaletteHooks()\HookCount = 1
    AddElement(DDrawCreatePaletteHooks()\Objects())
    DDrawCreatePaletteHooks()\Objects() = *Object
  EndIf

EndProcedure

;Installs a hook on a CreateSurface methode.
Procedure DDraw_InstallCreateSurfaceHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_CreateSurface * SizeOf(INTEGER)))
  If GetMethodeListElement(OldAddr, DDrawCreateSurfaceHooks()) = #True
    DDrawCreateSurfaceHooks()\HookCount + 1
    AddElement(DDrawCreateSurfaceHooks()\Objects())
    DDrawCreateSurfaceHooks()\Objects() = *Object
  Else
    AddElement(DDrawCreateSurfaceHooks())
    DDrawCreateSurfaceHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_CreateSurface_Hook())
    DDrawCreateSurfaceHooks()\NewAddr   = GetOrigStdProcAddr(DDrawCreateSurfaceHooks()\HookId)
    DDrawCreateSurfaceHooks()\OldAddr   = OldAddr
    DDrawCreateSurfaceHooks()\HookCount = 1
    AddElement(DDrawCreateSurfaceHooks()\Objects())
    DDrawCreateSurfaceHooks()\Objects() = *Object
  EndIf

EndProcedure

;Installs a hook on a DuplicateSurface methode.
Procedure DDraw_InstallDuplicateSurfaceHook(*Object)
  
  Protected OldAddr.i
  
  OldAddr = PeekI(PeekI(*Object) + (#GF_DD_IDD_DuplicateSurface * SizeOf(INTEGER)))
  If GetMethodeListElement(OldAddr, DDrawDuplicateSurfaceHooks()) = #True
    DDrawDuplicateSurfaceHooks()\HookCount + 1
    AddElement(DDrawDuplicateSurfaceHooks()\Objects())
    DDrawDuplicateSurfaceHooks()\Objects() = *Object
  Else
    AddElement(DDrawDuplicateSurfaceHooks())
    DDrawDuplicateSurfaceHooks()\HookId    = InstallStdProcHook(OldAddr, @DDraw_DuplicateSurface_Hook())
    DDrawDuplicateSurfaceHooks()\NewAddr   = GetOrigStdProcAddr(DDrawDuplicateSurfaceHooks()\HookId)
    DDrawDuplicateSurfaceHooks()\OldAddr   = OldAddr
    DDrawDuplicateSurfaceHooks()\HookCount = 1
    AddElement(DDrawDuplicateSurfaceHooks()\Objects())
    DDrawDuplicateSurfaceHooks()\Objects() = *Object
  EndIf

EndProcedure

;Hook on the CreateClipper procedure.
Procedure.i DDraw_CreateClipper_Hook(*this, dwFlags.i, *lplpDDClipper.INTEGER, *pUnkOuter)
  
  Protected Ret.i
  Protected CreateClipper.Proto_CreateClipper
  
  ;Call the original CreateClipper methode
  GetMethodeListElementByObject(*this, DDrawCreateClipperHooks())
  CreateClipper = DDrawCreateClipperHooks()\NewAddr
  
  Ret = CreateClipper(*this, dwFlags, *lplpDDClipper, *pUnkOuter)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *lplpDDClipper\i
  DDrawObjects()\Type = #GF_DD_IDirectDrawClipper  

  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*lplpDDClipper\i)
  DDraw_InstallReleaseHook(*lplpDDClipper\i)
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*lplpDDClipper\i, #GF_DD_ObjectCreate, #GF_DD_IDirectDrawClipper)
  
  ProcedureReturn Ret

EndProcedure

;Hook on the CreatePalette procedure.
Procedure.i DDraw_CreatePalette_Hook(*this, dwFlags.i, *lpColorTable, *lplpDDPalette.INTEGER, *pUnkOuter)
  
  Protected Ret.i
  Protected CreatePalette.Proto_CreatePalette
  
  ;Call the original CreatePalette methode
  GetMethodeListElementByObject(*this, DDrawCreatePaletteHooks())
  CreatePalette = DDrawCreatePaletteHooks()\NewAddr
  
  Ret = CreatePalette(*this, dwFlags, *lpColorTable, *lplpDDPalette, *pUnkOuter)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *lplpDDPalette\i
  DDrawObjects()\Type = #GF_DD_IDirectDrawPalette

  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*lplpDDPalette\i)
  DDraw_InstallReleaseHook(*lplpDDPalette\i)
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*lplpDDPalette\i, #GF_DD_ObjectCreate, #GF_DD_IDirectDrawPalette)
  
  ProcedureReturn Ret

EndProcedure

;Hook on the CreateSurface procedure.
Procedure.i DDraw_CreateSurface_Hook(*this, *lpDDSurfaceDesc, *lplpDDSurface.INTEGER, *pUnkOuter)
  
  Protected Ret.i
  Protected ObjectType.i
  Protected CreateSurface.Proto_CreateSurface
  
  ;Call the original CreateSurface methode
  GetMethodeListElementByObject(*this, DDrawCreateSurfaceHooks())
  CreateSurface = DDrawCreateSurfaceHooks()\NewAddr
  
  Ret = CreateSurface(*this, *lpDDSurfaceDesc, *lplpDDSurface, *pUnkOuter)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  Select DDrawVersion
    Case 0, 1, 2, 3
      ObjectType = #GF_DD_IDirectDrawSurface
    Case 4
      ObjectType = #GF_DD_IDirectDrawSurface4
    Case 7
      ObjectType = #GF_DD_IDirectDrawSurface7
  EndSelect
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *lplpDDSurface\i
  DDrawObjects()\Type = ObjectType  

  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*lplpDDSurface\i)
  DDraw_InstallReleaseHook(*lplpDDSurface\i)
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*lplpDDSurface\i, #GF_DD_ObjectCreate, ObjectType)
  
  ProcedureReturn Ret

EndProcedure

;Hook on the DuplicateSurface.
Procedure.i DDraw_DuplicateSurface_Hook(*this, *lpDDSurface, *lplpDupDDSurface.INTEGER)
  
  Protected Ret.i
  Protected ObjectType.i
  Protected DuplicateSurface.Proto_DuplicateSurface
  
  ;Call the original CreateSurface methode
  GetMethodeListElementByObject(*this, DDrawDuplicateSurfaceHooks())
  DuplicateSurface = DDrawDuplicateSurfaceHooks()\NewAddr
  
  Ret = DuplicateSurface(*this, *lpDDSurface, *lplpDupDDSurface)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  Select DDrawVersion
    Case 0, 1, 2, 3
      ObjectType = #GF_DD_IDirectDrawSurface
    Case 4
      ObjectType = #GF_DD_IDirectDrawSurface4
    Case 7
      ObjectType = #GF_DD_IDirectDraw7
  EndSelect
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *lplpDupDDSurface\i
  DDrawObjects()\Type = ObjectType  

  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*lplpDupDDSurface\i)
  DDraw_InstallReleaseHook(*lplpDupDDSurface\i)
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*lplpDupDDSurface\i, #GF_DD_ObjectCreate, ObjectType)

  ProcedureReturn Ret

EndProcedure

;Hook on the QueryInterface procedure.
Procedure.i DDraw_QueryInterface_Hook(*this, *riid.GUID, *Object.INTEGER)
  
  Protected Ret.i
  Protected ObjectType.i
  Protected QueryInterface.Proto_QueryInterface
  
  ;Call the original QueryInterface methode
  GetMethodeListElementByObject(*this, DDrawQueryInterfaceHooks())
  QueryInterface = DDrawQueryInterfaceHooks()\NewAddr
  
  Ret = QueryInterface(*this, *riid, *Object)
  If Ret <> #S_OK
    ProcedureReturn Ret
  EndIf
  
  ObjectType = GetDDrawObjectType(*riid)
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *Object\i
  DDrawObjects()\Type = ObjectType  

  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*Object\i)
  DDraw_InstallReleaseHook(*Object\i)

  ;Install object specific hooks on the new object.
  Select ObjectType
      
    Case #GF_DD_IDirectDraw, #GF_DD_IDirectDraw2, #GF_DD_IDirectDraw4, #GF_DD_IDirectDraw7
      DDraw_InstallCreateClipperHook(*Object\i)
      DDraw_InstallCreatePaletteHook(*Object\i)
      DDraw_InstallCreateSurfaceHook(*Object\i)
      DDraw_InstallDuplicateSurfaceHook(*Object\i)
      
  EndSelect
  
  ;Check used DirectDraw version.
  Select ObjectType
      
    Case #GF_DD_IDirectDraw
      If DDrawVersion < 1
        DDrawVersion = 1
      EndIf
      
    Case #GF_DD_IDirectDraw2
      If DDrawVersion < 2
        DDrawVersion = 2
      EndIf
  
    Case #GF_DD_IDirectDraw4
      If DDrawVersion < 4
        DDrawVersion = 4
      EndIf
      
    Case #GF_DD_IDirectDraw7
      If DDrawVersion < 7
        DDrawVersion = 7
      EndIf
      
    Case #GF_DD_IDirectDrawSurface3
      If DDrawVersion < 3
        DDrawVersion = 3
      EndIf
      
  EndSelect
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*Object\i, #GF_DD_ObjectCreate, ObjectType)
  
  ProcedureReturn Ret
  
EndProcedure

;Hook on the Release procedure.
Procedure.i DDraw_Release_Hook(*this)
  
  Protected Ret.i
  Protected ObjectType.i
  Protected Release.Proto_Release
  
  GetObjectListElement(*this, DDrawObjects())
  ObjectType = DDrawObjects()\Type
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*this, #GF_DD_ObjectRelease, ObjectType)
  
  ;Call the original Release methode.
  GetMethodeListElementByObject(*this, DDrawReleaseHooks())
  Release = DDrawReleaseHooks()\NewAddr
  Ret = Release(*this)
  
  ;Remove Release hook.
  DeleteElement(DDrawReleaseHooks()\Objects())
  DDrawReleaseHooks()\HookCount - 1
  If DDrawReleaseHooks()\HookCount = 0
    UnInstallStdProcHook(DDrawReleaseHooks()\HookId)
    DeleteElement(DDrawReleaseHooks())    
  EndIf
  
  ;Remove QueryInterface hook.
  GetMethodeListElementByObject(*this, DDrawQueryInterfaceHooks())
  DDrawQueryInterfaceHooks()\HookCount - 1
  If DDrawQueryInterfaceHooks()\HookCount = 0
    UnInstallStdProcHook(DDrawQueryInterfaceHooks()\HookId)
    DeleteElement(DDrawQueryInterfaceHooks())    
  EndIf
  
  ;Remove object specific methode hooks.
  Select ObjectType
      
    Case #GF_DD_IDirectDraw, #GF_DD_IDirectDraw2, #GF_DD_IDirectDraw4, #GF_DD_IDirectDraw7
  
      ;Remove CreateClipper hook.
      GetMethodeListElementByObject(*this, DDrawCreateClipperHooks())
      DDrawCreateClipperHooks()\HookCount - 1
      If DDrawCreateClipperHooks()\HookCount = 0
        UnInstallStdProcHook(DDrawCreateClipperHooks()\HookId)
        DeleteElement(DDrawCreateClipperHooks())    
      EndIf
      
      ;Remove CreatePalette hook.
      GetMethodeListElementByObject(*this, DDrawCreatePaletteHooks())
      DDrawCreatePaletteHooks()\HookCount - 1
      If DDrawCreatePaletteHooks()\HookCount = 0
        UnInstallStdProcHook(DDrawCreatePaletteHooks()\HookId)
        DeleteElement(DDrawCreatePaletteHooks())    
      EndIf
      
      ;Remove CreateSurface hook.
      GetMethodeListElementByObject(*this, DDrawCreateSurfaceHooks())
      DDrawCreateSurfaceHooks()\HookCount - 1
      If DDrawCreateSurfaceHooks()\HookCount = 0
        UnInstallStdProcHook(DDrawCreateSurfaceHooks()\HookId)
        DeleteElement(DDrawCreateSurfaceHooks())    
      EndIf
      
      ;Remove DuplicateSurface hook.
      GetMethodeListElementByObject(*this, DDrawDuplicateSurfaceHooks())
      DDrawDuplicateSurfaceHooks()\HookCount - 1
      If DDrawDuplicateSurfaceHooks()\HookCount = 0
        UnInstallStdProcHook(DDrawDuplicateSurfaceHooks()\HookId)
        DeleteElement(DDrawDuplicateSurfaceHooks())    
      EndIf

  EndSelect
  
  ;Remove object from list
  DeleteElement(DDrawObjects())
  
  ProcedureReturn Ret
  
EndProcedure

;Hook on DirectDrawCreate.
Procedure.i DirectDrawCreate_Hook(*guid.GUID, *ddi.INTEGER, *unknown)
  
  Protected Ret.i
  Protected OldAddr.i
  
  ;Call the original DirectDrawCreate procedure.
  Ret = DirectDrawCreate(*guid, *ddi, *unknown)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *ddi\i
  DDrawObjects()\Type = #GF_DD_IDirectDraw
  
  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*ddi\i)
  DDraw_InstallReleaseHook(*ddi\i)
  
  ;Install object specific hooks on the new object.
  DDraw_InstallCreateClipperHook(*ddi\i)
  DDraw_InstallCreatePaletteHook(*ddi\i)
  DDraw_InstallCreateSurfaceHook(*ddi\i)
  DDraw_InstallDuplicateSurfaceHook(*ddi\i)
  
  ;Check used DirectDraw version.
  If DDrawVersion < 1
    DDrawVersion = 1
  EndIf
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*ddi\i, #GF_DD_ObjectCreate, #GF_DD_IDirectDraw)
  
  ProcedureReturn Ret

EndProcedure

;Hook on DirectDrawCreateEx.
Procedure.i DirectDrawCreateEx_Hook(*guid.GUID, *ddi.INTEGER, *riid.GUID, *unkown)
  
  Protected Ret.i
  Protected ObjectType.i
  
  ;call the original DirectDrawCreateEx
  Ret = DirectDrawCreateEx(*guid, *ddi, *riid, *unkown)
  If Ret <> #DD_OK
    ProcedureReturn Ret
  EndIf
  
  ObjectType = GetDDrawObjectType(*riid)
  
  AddElement(DDrawObjects())
  DDrawObjects()\Handle = *ddi\i
  DDrawObjects()\Type = ObjectType
  
  ;Install standard hooks on the new object.
  DDraw_InstallQueryInterfaceHook(*ddi\i)
  DDraw_InstallReleaseHook(*ddi\i)
  
  ;Install object specific hooks on the new object.
  DDraw_InstallCreateClipperHook(*ddi\i)
  DDraw_InstallCreatePaletteHook(*ddi\i)
  DDraw_InstallCreateSurfaceHook(*ddi\i)
  DDraw_InstallDuplicateSurfaceHook(*ddi\i)
  
  ;Check used DirectDraw version.
  Select ObjectType
      
    Case #GF_DD_IDirectDraw
      If DDrawVersion < 1
        DDrawVersion = 1
      EndIf
      
    Case #GF_DD_IDirectDraw2
      If DDrawVersion < 2
        DDrawVersion = 2
      EndIf
  
    Case #GF_DD_IDirectDraw4
      If DDrawVersion < 4
        DDrawVersion = 4
      EndIf
      
    Case #GF_DD_IDirectDraw7
      If DDrawVersion < 7
        DDrawVersion = 7
      EndIf
      
  EndSelect
  
  ;Call back all plugins.
  CallBackDDrawPlugins(*ddi\i, #GF_DD_ObjectCreate, ObjectType)

  ProcedureReturn Ret
  
EndProcedure

;Installs the DirectDraw hook.
;Use this if you want to be notified on the creation of DirectDraw related objects.
;Use ObjectsToHook to specify which type of object creations you want to hook.
;Returns a DirectDraw hook ID if it was successfull.
;Returns #False if it failed.
ProcedureDLL.i InstallDDrawHook(*Callback)
  
  Protected DDrawHookId.i
  
  DDrawHookId = AddElement(DDrawHookCallbacks())
  If DDrawHookId = 0
    ProcedureReturn #False
  EndIf
  DDrawHookCallbacks() = *Callback
  
  ;If it isnt allready hooked, hook it.
  If DDrawHookCount = 0

    ;Install procedure hook on DirectDrawCreate for DirectDraw 1-7.
    
    DirectDrawCreateHookId = InstallWin32ApiHook("ddraw.dll", "DirectDrawCreate", @DirectDrawCreate_Hook())
    If DirectDrawCreateHookId = #False
      DeleteElement(DDrawHookCallbacks())
      ProcedureReturn #False
    EndIf
    
    DirectDrawCreate = GetOrigStdProcAddr(DirectDrawCreateHookId)
    
    ;Install procedure hook on DirectDrawCreateEx for DirectDraw 7 + 3D.
    
    DirectDrawCreateExHookId = InstallWin32ApiHook("ddraw.dll", "DirectDrawCreateEx", @DirectDrawCreateEx_Hook())
    If DirectDrawCreateExHookId = #False
      DeleteElement(DDrawHookCallbacks())
      UninstallStdProcHook(DirectDrawCreateHookId)
      DirectDrawCreateHookId = 0      
      ProcedureReturn #False
    EndIf
    
    DirectDrawCreateEx = GetOrigStdProcAddr(DirectDrawCreateExHookId)
    
  EndIf
  
  DDrawHookCount + 1

  ProcedureReturn DDrawHookId
  
EndProcedure

;Uninstalls the DirectDraw hook.
ProcedureDLL UninstallDDrawHook(DDrawHookId.i)
  
  ChangeCurrentElement(DDrawHookCallbacks(), DDrawHookId)
  DeleteElement(DDrawHookCallbacks())
  DDrawHookCount - 1
  
  If DDrawHookCount = 0
    UninstallStdProcHook(DirectDrawCreateHookId)
    DirectDrawCreateHookId = 0
    UninstallStdProcHook(DirectDrawCreateExHookId)
    DirectDrawCreateExHookId = 0
  EndIf
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; CursorPosition = 154
; FirstLine = 68
; Folding = AAAw
; EnableXP