;/-------------------\
;| Game Fixer        |
;| Procedure Hooking |
;|                   |
;| Version 0.01      |
;| Date: 01.05.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

Structure StdProcHook
  ProcAddr.i
  HookProcAddr.i
  SavedCodeBuffer.i
  SavedCodeSize.i
EndStructure

Global NewList StdProcHooks.StdProcHook()

;-------------------------------------------------------------
;- generic hook procedures
;-------------------------------------------------------------

;installs a hook on a StdCall procedure
;returns a hook ID for calling the original and uninstalling the hook
;returns #False if it fails
ProcedureDLL.i InstallStdProcHook(ProcAddr.i, HookProcAddr.i)
  
  Protected Length.i
  Protected OldProtect.i
  Protected Dummy.i
  Protected HookId.i
  Protected Instruction.DisasmStruc
  
  ;add new element to the hooks list
  HookId = AddElement(StdProcHooks())
  If HookId = 0
    ProcedureReturn #False
  EndIf
  
  ;get length of code to save
  
  Repeat
    Instruction\EIP = ProcAddr + Length
    Length = Length + Disasm(Instruction)
  Until Length >= 5
  
  ;allocate memory to save code that gets overwritten
  
  StdProcHooks()\SavedCodeBuffer = AllocateMemory(Length + 5)
  If StdProcHooks()\SavedCodeBuffer = 0
    DeleteElement(StdProcHooks())
    ProcedureReturn #False
  EndIf

  VirtualProtect_(StdProcHooks()\SavedCodeBuffer, Length + 5, #PAGE_EXECUTE_READWRITE, @Dummy)
  
  ;copy code that gets overwritten
  CopyMemory(ProcAddr, StdProcHooks()\SavedCodeBuffer, Length)
  
  ;write jump to end of buffer that jumps to the original procedure behind the patched code
  PokeA(StdProcHooks()\SavedCodeBuffer + Length, $E9)
  PokeL(StdProcHooks()\SavedCodeBuffer + Length + 1, (ProcAddr + Length) - (StdProcHooks()\SavedCodeBuffer + Length) - 5)
  
  ;set write privilegs of target procedure
  VirtualProtect_(ProcAddr, 5, #PAGE_EXECUTE_READWRITE, @OldProtect)
  
  ;write jump to target procedure that jumps to the hook procedure
  PokeB(ProcAddr,$E9)
  PokeL(ProcAddr + 1, HookProcAddr - ProcAddr - 5)
  
  ;restore access privilegs of target procedure
  VirtualProtect_(ProcAddr, 5, OldProtect, @Dummy)
  
  ;set all values of hook structure
  StdProcHooks()\ProcAddr      = ProcAddr
  StdProcHooks()\HookProcAddr  = HookProcAddr
  StdProcHooks()\SavedCodeSize = Length

  ProcedureReturn HookId
  
EndProcedure

;uninstalls a StdCall hook previously installed with InstallStdProcHook
ProcedureDLL UninstallStdProcHook(HookId.i)
  
  Protected OldProtect.i
  Protected Dummy.i
  
  ChangeCurrentElement(StdProcHooks(), HookId)
  
  ;set write privilegs of target procedure
  VirtualProtect_(StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize, #PAGE_EXECUTE_READWRITE, @OldProtect)
  
  ;copy back saved code
  CopyMemory(StdProcHooks()\SavedCodeBuffer, StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize)
  
  ;restore access privilegs of target procedure
  VirtualProtect_(StdProcHooks()\ProcAddr, StdProcHooks()\SavedCodeSize, OldProtect, @Dummy)
  
  ;free the memory of the saved code
  FreeMemory(StdProcHooks()\SavedCodeBuffer)
  
  DeleteElement(StdProcHooks())
  
EndProcedure

;returns the address of the procedure to call to call the original procedure
ProcedureDLL.i GetOrigStdProcAddr(HookId.i)
  
  ChangeCurrentElement(StdProcHooks(), HookId)
  
  ProcedureReturn StdProcHooks()\SavedCodeBuffer
  
EndProcedure

;-------------------------------------------------------------
;- Win32 API hook
;-------------------------------------------------------------

;installs a hook on a Win32 API function
;returns a hook ID for calling the original and uninstalling the hook
;returns #False if it fails
;use UninstallStdProcHook and GetOrigStdProcAddr with Win32 API hooks
ProcedureDLL.i InstallWin32ApiHook(DllName.s, ProcName.s, HookProcAddr.i)
  
  Protected Dll.i
  Protected ProcAddr.i
  
  Dll = LoadLibrary_(DllName)
  If Dll = 0
    ProcedureReturn #False
  EndIf
  
  ProcAddr = GetProcAddress_(Dll, ProcName)
  If ProcAddr = 0
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn InstallStdProcHook(ProcAddr, HookProcAddr)
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; CursorPosition = 145
; FirstLine = 22
; Folding = 5
; EnableXP