;/------------------\
;| Game Fixer       |
;| Core             |
;|                  |
;| Version 0.03     |
;| Date: 29.06.2012 |
;| PureBasic 4.60   |
;|                  |
;| by Thorium       |
;\------------------/

EnableExplicit

XIncludeFile "Disassembler.pbi"

Declare.i InstallStdProcHook(ProcAddr.i, HookProcAddr.i)
Declare UninstallStdProcHook(HookId.i)
Declare.i GetOrigStdProcAddr(HookId.i)
Declare.i InstallWin32ApiHook(DllName.s, ProcName.s, HookProcAddr.i)

XIncludeFile "Procedure Hook.pbi"
XIncludeFile "DirectDraw Hook.pbi"

#AppTitle = "Game Fixer"

#CoreFileName = "GameFixer Core.dll"

#SharedData_Name = "Game Fixer Shared Data"
#SharedData_Size = 1024

#SystemFolder = "System\"
#PluginFolder = "Plugins\"

Enumeration 1
  #Error_PluginMissing
  #Error_PluginWrongFormat
  #Error_PluginWrongConfig
EndEnumeration

Structure SharedData
  Ready.a
  Error.l
  PlugInsCnt.u
  GfPathLd.u
EndStructure

Structure SharedDataPlugin
  FileLd.a
  ConfigLd.u  
EndStructure

Prototype.i Proto_Plugin_Config(Config.s)

Global *SharedData.SharedData
Global GameFixerPath.s
Global CoreDllHandle.i

Procedure AllocSharedMemory(MemoryName.s, SizeOfSharedData.i)
  
  Define hMapping.i
  
  hMapping = CreateFileMapping_(#INVALID_HANDLE_VALUE, 0, #PAGE_READWRITE, 0, SizeOfSharedData, MemoryName)
  hMapping = MapViewOfFile_(hMapping, #FILE_MAP_WRITE, 0, 0, 0)
  
  ProcedureReturn hMapping

EndProcedure

Procedure FreeSharedMemory(hMapping.i)
  
  UnmapViewOfFile_(hMapping)
  
EndProcedure

Procedure InitCore(hAttachProcessThread.i)
  
  Protected i.i
  Protected *SharedDataPlugin.SharedDataPlugin
  Protected PluginName.s
  Protected PluginConfig.s
  Protected Plugin.i
  Protected PluginConfigProc.Proto_Plugin_Config

  ;wait for AttachProcess thread to finish
  WaitForSingleObject_(hAttachProcessThread, #INFINITE)
  
  ;get handle of core DLL (this DLL)
  CoreDllHandle = GetModuleHandle_(#CoreFileName)
  
  ;set up shared memory to get data from the main program
  *SharedData = AllocSharedMemory(#SharedData_Name, #SharedData_Size)
  
  ;get the path of game fixer from the shared data
  GameFixerPath = PeekS(*SharedData + SizeOf(SharedData), *SharedData\GfPathLd)
  
  ;load plugins and configure them
  
  *SharedDataPlugin = *SharedData + SizeOf(SharedData) + *SharedData\GfPathLd
  
  For i = 1 To *SharedData\PlugInsCnt
    
    PluginName   = PeekS(*SharedDataPlugIn + SizeOf(SharedDataPlugIn), *SharedDataPlugin\FileLd)
    PluginConfig = PeekS(*SharedDataPlugIn + SizeOf(SharedDataPlugIn) + *SharedDataPlugin\FileLd, *SharedDataPlugin\ConfigLd)
    
    Plugin = LoadLibrary_(GameFixerPath + #PluginFolder + PluginName)
    If Plugin = 0
      *SharedData\Error = #Error_PluginMissing
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf
    
    PluginConfigProc = GetProcAddress_(Plugin, "Config")
    If PluginConfigProc = 0
      *SharedData\Error = #Error_PluginWrongFormat
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf
    
    If PluginConfigProc(PluginConfig) = #False
      *SharedData\Error = #Error_PluginWrongConfig
      FreeLibraryAndExitThread_(CoreDllHandle, 0)
    EndIf

    *SharedDataPlugin = *SharedDataPlugin + SizeOf(SharedDataPlugIn) + *SharedDataPlugin\FileLd + *SharedDataPlugin\ConfigLd
    
  Next
  
  *SharedData\Ready = #True
  
  FreeSharedMemory(*SharedData)
  
EndProcedure

ProcedureDLL AttachProcess(Instance.i)

  Protected hAttachProcessThread.i
  Protected hProcess.i
  
  ;get the handle to the thread to enable the init procedure to wait for it's termination
  
  hProcess = GetCurrentProcess_()
  DuplicateHandle_(hProcess, GetCurrentThread_(), hProcess, @hAttachProcessThread, 0, #False, #DUPLICATE_SAME_ACCESS)

  CreateThread(@InitCore(), hAttachProcessThread)
  
EndProcedure

ProcedureDLL DetachProcess(Instance.i)


EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 149
; FirstLine = 26
; Folding = g
; EnableXP
; Executable = B:\GameFixer\System\GameFixer Core.dll
; IncludeVersionInfo
; VersionField0 = 0,3,0,0
; VersionField1 = 0,3,0,0
; VersionField3 = GameFixer
; VersionField9 = Janos David Ommert
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_DLL