;/-------------------\
;| GameFixer Plugin  |
;| DriveFix          |
;|                   |
;| Version 0.01      |
;| Date: 20.08.2013  |
;| PureBasic 5.11    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

#DQuoteMask = "/#DQ/"
#SpaceMask  = "/#SP/"

Import "GameFixer Core.lib"
  UninstallStdProcHook(HookId.i)
  GetOrigStdProcAddr.i(HookId.i)
  InstallWin32ApiHook.i(DllName.s, ProcName.s, HookProcAddr.i)
EndImport

Prototype.i Proto_CreateFileA(*lpFileName, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
Global CreateFileAApi.Proto_CreateFileA
Global CreateFileAHookId.i

Prototype.i Proto_CreateFileW(*lpFileName, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
Global CreateFileWApi.Proto_CreateFileW
Global CreateFileWHookId.i

Prototype.i Proto_mciSendStringA(*lpszCommand, *lpszReturnString, cchReturn.i, hwndCallback.i)
Global mciSendStringAApi.Proto_mciSendStringA
Global mciSendStringAHookId.i

Prototype.i Proto_mciSendStringW(*lpszCommand, *lpszReturnString, cchReturn.i, hwndCallback.i)
Global mciSendStringWApi.Proto_mciSendStringW
Global mciSendStringWHookId.i

Prototype.i Proto_mciExecuteA(*pszCommand)
Global Proto_mciExecuteAApi.Proto_mciExecuteA
Global mciExecuteAHookId.i

Prototype.i Proto_mciExecuteW(*pszCommand)
Global Proto_mciExecuteWApi.Proto_mciExecuteW
Global mciExecuteWHookId.i

Global SrcPath.s
Global DstPath.s
Global AutoMode.i

ProcedureDLL AttachProcess(Instance.i)

EndProcedure

ProcedureDLL DetachProcess(Instance.i)
  
  UninstallStdProcHook(CreateFileAHookId)
  UninstallStdProcHook(CreateFileWHookId)
  UninstallStdProcHook(mciSendStringAHookId)
  UninstallStdProcHook(mciSendStringWHookId)
  UninstallStdProcHook(mciExecuteAHookId)
  UninstallStdProcHook(mciExecuteWHookId)
  
EndProcedure

Procedure ParseConfig(Config.s)
  
  Protected i.i
  Protected Setting.s
  Protected Value.s
  
  For i = 1 To 100
    
    Setting = StringField(Config, i, " ")
    If Setting = ""
      Break
    EndIf
    
    Value = StringField(Setting, 2, "=")
    Value = ReplaceString(Value, #DQuoteMask, #DQUOTE$)
    Value = ReplaceString(Value, #SpaceMask, " ")

    Select LCase(StringField(Setting, 1, "="))
      
      Case "SrcPath", "sp"
        SrcPath = Value
        
      Case "DstPath", "dp"
        DstPath = Value
        
      Case "Auto", "a"
        If Value = "1"
          AutoMode = #True
        EndIf

    EndSelect

  Next
  
EndProcedure

Procedure.s CreateFileGetNewFileName(FileName.s)
  
  Protected Path.s
  
  Path = Left(GetPathPart(FileName), Len(SrcPath))
  If LCase(SrcPath) = LCase(DstPath)
    FileName = ReplaceString(FileName, SrcPath, DstPath, #PB_String_NoCase, 1, 1)
  EndIf
  
  ProcedureReturn FileName
    
EndProcedure

Procedure CreateFileAHook(*lpFileName, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
  
  Protected FileName.s
  Protected *NewFileNameBuffer
  Protected RetVal.i
  
  FileName = PeekS(*lpFileName, -1, #PB_Ascii)
  FileName = CreateFileGetNewFileName(FileName)
  
  *NewFileNameBuffer = AllocateMemory(Len(FileName) + 1)
  PokeS(*NewFileNameBuffer, FileName, -1, #PB_Ascii)
  
  RetVal = CreateFileAApi(*NewFileNameBuffer, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
  
  FreeMemory(*NewFileNameBuffer)
  
  ProcedureReturn RetVal

EndProcedure

Procedure CreateFileWHook(*lpFileName, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
  
  Protected FileName.s
  Protected NewFileName.s
  Protected *NewFileNameBuffer
  Protected RetVal.i
  
  FileName = PeekS(*lpFileName, -1, #PB_Unicode)
  NewFileName = CreateFileGetNewFileName(FileName)
  
  If NewFileName = FileName
    *NewFileNameBuffer = *lpFileName
  Else
    *NewFileNameBuffer = AllocateMemory(Len(NewFileName) + 2)
    PokeS(*NewFileNameBuffer, NewFileName, -1, #PB_Unicode)
  EndIf
  
  RetVal = CreateFileAApi(*NewFileNameBuffer, dwDesiredAccess.i, dwShareMode.i, *lpSecurityAttributes, dwCreationDisposition.i, dwFlagsAndAttributes.i, hTemplateFile.i)
  
  FreeMemory(*NewFileNameBuffer)
  
  ProcedureReturn RetVal
  
EndProcedure

Procedure.s ProcessMciString(MciString.s)
  
  Protected MciString.s
  
  MciString = ReplaceString(MciString, SrcPath, DstPath, #PB_String_NoCase)
  
  ProcedureReturn MciString
    
EndProcedure

Procedure.i mciSendStringAHook(*lpszCommand, *lpszReturnString, cchReturn.i, hwndCallback.i)
  
  Protected MciString.s
  Protected *NewMciStringBuffer
  Protected RetVal.i
  
  MciString = PeekS(*lpszCommand, -1, #PB_Ascii)
  MciString = ProcessMciString(MciString)
  
  *NewMciStringBuffer = AllocateMemory(Len(MciString) + 1)
  PokeS(*NewMciStringBuffer, MciString, -1, #PB_Ascii)
  
  RetVal = mciSendStringAApi(*NewMciStringBuffer, *lpszReturnString, cchReturn.i, hwndCallback.i)
  
  FreeMemory(*NewMciStringBuffer)
  
  ProcedureReturn RetVal
  
EndProcedure

Procedure.i mciSendStringWHook(*lpszCommand, *lpszReturnString, cchReturn.i, hwndCallback.i)
  
  Protected MciString.s
  Protected *NewMciStringBuffer
  Protected RetVal.i
  
  MciString = PeekS(*lpszCommand, -1, #PB_Unicode)
  MciString = ProcessMciString(MciString)
  
  *NewMciStringBuffer = AllocateMemory(Len(MciString) + 2)
  PokeS(*NewMciStringBuffer, MciString, -1, #PB_Unicode)
  
  RetVal = mciSendStringWApi(*NewMciStringBuffer, *lpszReturnString, cchReturn.i, hwndCallback.i)
  
  FreeMemory(*NewMciStringBuffer)
  
  ProcedureReturn RetVal
  
EndProcedure

Procedure.i mciExecuteAHook(*pszCommand)
  
  Protected MciString.s
  Protected *NewMciStringBuffer
  Protected RetVal.i
  
  MciString = PeekS(*lpszCommand, -1, #PB_Ascii)
  MciString = ProcessMciString(MciString)
  
  *NewMciStringBuffer = AllocateMemory(Len(MciString) + 1)
  PokeS(*NewMciStringBuffer, MciString, -1, #PB_Ascii)
  
  RetVal = mciExecuteAApi(*NewMciStringBuffer)
  
  FreeMemory(*NewMciStringBuffer)
  
  ProcedureReturn RetVal

EndProcedure

Procedure.i mciExecuteWHook(*pszCommand)
  
  Protected MciString.s
  Protected *NewMciStringBuffer
  Protected RetVal.i
  
  MciString = PeekS(*lpszCommand, -1, #PB_Unicode)
  MciString = ProcessMciString(MciString)
  
  *NewMciStringBuffer = AllocateMemory(Len(MciString) + 2)
  PokeS(*NewMciStringBuffer, MciString, -1, #PB_Unicode)
  
  RetVal = mciExecuteWApi(*NewMciStringBuffer)
  
  FreeMemory(*NewMciStringBuffer)
  
  ProcedureReturn RetVal
  
EndProcedure

Procedure.i InstallHooks()
  
  CreateFileAHookId = InstallWin32ApiHook("Kernel32.dll", "CreateFileA", @CreateFileAHook())
  If CreateFileAHookId = #False
    ProcedureReturn #False
  EndIf
  CreateFileAApi = GetOrigStdProcAddr(CreateFileAHookId)
  
  CreateFileWHookId = InstallWin32ApiHook("Kernel32.dll", "CreateFileW", @CreateFileWHook())
  If CreateFileWHookId = #False
    UninstallStdProcHook(CreateFileAHookId)
    ProcedureReturn #False
  EndIf
  CreateFileWApi = GetOrigStdProcAddr(CreateFileWHookId)
  
  mciSendStringAHookId = InstallWin32ApiHook("Winmm.dll", "mciSendStringA", @mciSendStringAHook())
  If mciSendStringAHookId = #False
    UninstallStdProcHook(CreateFileAHookId)
    UninstallStdProcHook(CreateFileWHookId)
    ProcedureReturn #False
  EndIf
  mciSendStringAApi = GetOrigStdProcAddr(mciSendStringAHookId)
  
  mciSendStringWHookId = InstallWin32ApiHook("Winmm.dll", "mciSendStringW", @mciSendStringWHook())
  If mciSendStringWHookId = #False
    UninstallStdProcHook(CreateFileAHookId)
    UninstallStdProcHook(CreateFileWHookId)
    UninstallStdProcHook(mciSendStringAHookId)
    ProcedureReturn #False
  EndIf
  mciSendStringWApi = GetOrigStdProcAddr(mciSendStringWHookId)
  
  mciExecuteAHookId = InstallWin32ApiHook("Winmm.dll", "mciExecuteA", @mciExecuteAHook())
  If mciExecuteAHookId = #False
    UninstallStdProcHook(CreateFileAHookId)
    UninstallStdProcHook(CreateFileWHookId)
    UninstallStdProcHook(mciSendStringAHookId)
    UninstallStdProcHook(mciSendStringWHookId)
    ProcedureReturn #False
  EndIf
  mciExecuteAApi = GetOrigStdProcAddr(mciExecuteAHookId)
  
  mciExecuteWHookId = InstallWin32ApiHook("Winmm.dll", "mciExecuteW", @mciExecuteWHook())
  If mciExecuteWHookId = #False
    UninstallStdProcHook(CreateFileAHookId)
    UninstallStdProcHook(CreateFileWHookId)
    UninstallStdProcHook(mciSendStringAHookId)
    UninstallStdProcHook(mciSendStringWHookId)
    UninstallStdProcHook(mciExecuteAHookId)
    ProcedureReturn #False
  EndIf
  mciExecuteWApi = GetOrigStdProcAddr(mciExecuteWHookId)

  ProcedureReturn #True
  
EndProcedure

ProcedureDLL.i Config(Config.s)

  ParseConfig(Config.s)
  
  If InstallHooks() = #False
    ProcedureReturn #False
  EndIf
    
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 5.11 (Windows - x64)
; ExecutableFormat = Shared Dll
; CursorPosition = 317
; FirstLine = 37
; Folding = AA+
; EnableXP
; Executable = B:\GameFixer\Plugins\Test.dll
; Warnings = Display