EnableExplicit

;****************************************************************************************
; HTTPGetFromWeb() 2.03 by Luis
; First release November 4, 2007 
; Free to be used in freeware and commercial programs.
;****************************************************************************************

; 2.03 (Dicember 31, 2009)
; ! Fixed for PB 4.40
;   Code would need an overhaul but I'll leave that to you :)

; 2.02 (Dicember 3, 2007)
; ! Changed the way to track the download progression internally, to correctly work 
;   with sites returning data in variable chunk sizes.
;   The problem was reported by Tranquil. Thank you !

; 2.01 (November 7, 2007)

; ! Solved the double authentication problem. Now a connection to a HTTP site requiring
;   authentication through a proxy server also requiring authentication is possible.

; + If the file to download is bigger than 10 MB the routine will automatically switch
;   to chunk mode using the default chunk size (65535) even if tHTTP\lChunkSize is not 
;   specified or it is equal to 0.

; + Added the param hRequest to the HTTPGetFromWeb_Start() callback, to access the opened
;   HTTP request handle to use custom HttpQueryInfo() calls (see example in sample1.pb).


; Some features:

; - Based on the WinInet API

; - HTTP download to MEMORY and to FILE

; - Download in a single block or in chunk mode

; - Full callback communication in chunk mode, partial in single block mode

; - Supports three different callback procedures for start download, end/abort download 
;   and processing (new in 2.0)

; - In chunk mode download can be aborted at any time

; - Able to connect through a proxy server with/without authentication

; - Able to connect to a web server using basic authentication in the form 
;   http://username:password@www.somedomain.com (new in 2.0)

; - Can be easily used for multithreaded download (new in 2.0)



; Included: 

; sample1.pb - basic single threaded usage example
; sample2.pb - multi threaded usage with very simple gui framework for reference



; Some usage examples:

; The simplest form: download bar.zip in memory.
; The pointer to the allocated memory area will be returned in tHTTP\*DestBuffer        
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://www.somedomain.com/foo/bar.zip"    
; tHTTP\lDestination = #PBL_WRITE_TO_MEMORY
; HTTPGetFromWeb (@tHTTP) 

 
; Same as above, loaded entirely in memory and then copied to the specified file.
; The memory in this case will be automatically freed by the procedure after
; saving the file to disk.
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://www.somedomain.com/foo/bar.zip"    
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; HTTPGetFromWeb (@tHTTP) 


; Again we download bar.zip, but this time we don't load it entirely in memory.
; We try to load it in "chunks" of 16384 bytes, the proc will automatically save a chunk to 
; file and then repeat the process until the download is completed.
; In this call we use a callback procedure as well.
; The callback procedure (see the prototype definition) will receive 
; - a pointer to the tHTTP structure
; - the number of bytes downloaded up to this point
; - the total size of the download (if available, else 0)
; - the time passed in seconds from the start of the download
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://www.somedomain.com/foo/bar.zip"    
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; tHTTP\lChunkSize = 16384
; tHTTP\fpCallBack = @MyCallBack()
; HTTPGetFromWeb (@tHTTP) 


; Same as above but using a proxy without authentication required.
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://www.somedomain.com/foo/bar.zip"    
; tHTTP\lAccess = #INTERNET_OPEN_TYPE_PROXY
; tHTTP\sProxyAndPort = "192.168.1.1:8080"
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; tHTTP\lChunkSize = 16384
; tHTTP\fpCallBack = @MyCallBack()
; HTTPGetFromWeb (@tHTTP) 


; Same as above but using a proxy requiring authentication
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://www.somedomain.com/foo/bar.zip"    
; tHTTP\lAccess = #INTERNET_OPEN_TYPE_PROXY
; tHTTP\sProxyAndPort = "192.168.1.1:8080"
; tHTTP\sProxyUsername = "username"
; tHTTP\sProxyPassword = "password"
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; tHTTP\lChunkSize = 16384
; tHTTP\fpCallBack = @MyCallBack()
; HTTPGetFromWeb (@tHTTP) 
 

; Accessing an HTTP resource requiring basic authentication
; 
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://username:password@www.somedomain.com/foo/bar.zip"    
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; tHTTP\lChunkSize = 16384
; tHTTP\fpCallBack = @MyCallBack();
; HTTPGetFromWeb (@tHTTP) 


; Accessing an HTTP resource requiring basic authentication through a 
; proxy requiring authentication
;
; tHTTP\lThreadID = 0 
; tHTTP\sURL = "http://username:password@www.somedomain.com/foo/bar.zip"    
; tHTTP\lAccess = #INTERNET_OPEN_TYPE_PROXY
; tHTTP\sProxyAndPort = "192.168.1.1:8080"
; tHTTP\sProxyUsername = "username"
; tHTTP\sProxyPassword = "password"
; tHTTP\lDestination = #PBL_WRITE_TO_FILE
; tHTTP\sFullFileName = "c:\download\bar.zip" 
; tHTTP\lChunkSize = 16384
; tHTTP\fpCallBack = @MyCallBack()
; HTTPGetFromWeb (@tHTTP) 


;**********************************************************
; Converted from WININET.H
;**********************************************************

#HTTP_STATUS_OK                         = 200

#HTTP_QUERY_FLAG_NUMBER                 = $20000000

#INTERNET_FLAG_RELOAD                   = $80000000  
#INTERNET_FLAG_RAW_DATA                 = $40000000  
#INTERNET_FLAG_EXISTING_CONNECT         = $20000000  
#INTERNET_FLAG_ASYNC                    = $10000000  
#INTERNET_FLAG_PASSIVE                  = $08000000  
#INTERNET_FLAG_NO_CACHE_WRITE           = $04000000  
#INTERNET_FLAG_MAKE_PERSISTENT          = $02000000  
#INTERNET_FLAG_FROM_CACHE               = $01000000  
#INTERNET_FLAG_SECURE                   = $00800000  
#INTERNET_FLAG_KEEP_CONNECTION          = $00400000  
#INTERNET_FLAG_NO_AUTO_REDIRECT         = $00200000  
#INTERNET_FLAG_READ_PREFETCH            = $00100000  
#INTERNET_FLAG_NO_COOKIES               = $00080000  
#INTERNET_FLAG_NO_AUTH                  = $00040000  
#INTERNET_FLAG_CACHE_IF_NET_FAIL        = $00010000  
#INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP  = $00008000 
#INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS = $00004000 
#INTERNET_FLAG_IGNORE_CERT_DATE_INVALID = $00002000 
#INTERNET_FLAG_IGNORE_CERT_CN_INVALID   = $00001000 
#INTERNET_FLAG_RESYNCHRONIZE            = $00000800  
#INTERNET_FLAG_HYPERLINK                = $00000400  
#INTERNET_FLAG_NO_UI                    = $00000200  
#INTERNET_FLAG_PRAGMA_NOCACHE           = $00000100  
#INTERNET_FLAG_CACHE_ASYNC              = $00000080  
#INTERNET_FLAG_FORMS_SUBMIT             = $00000040  
#INTERNET_FLAG_NEED_FILE                = $00000010  

#INTERNET_CONNECTION_MODEM              = $1
#INTERNET_CONNECTION_LAN                = $2
#INTERNET_CONNECTION_PROXY              = $4
#INTERNET_CONNECTION_MODEM_BUSY         = $8
#INTERNET_CONNECTION_OFFLINE            = $20
#INTERNET_CONNECTION_CONFIGURED         = $40
#INTERNET_RAS_INSTALLED                 = $10

#HTTP_QUERY_CONTENT_TYPE                = 1
#HTTP_QUERY_CONTENT_LENGTH              = 5
#HTTP_QUERY_STATUS_CODE                 = 19
#HTTP_QUERY_STATUS_TEXT                 = 20
#HTTP_QUERY_RAW_HEADERS                 = 21
#HTTP_QUERY_RAW_HEADERS_CRLF            = 22

#INTERNET_OPTION_USERNAME               = 28
#INTERNET_OPTION_PASSWORD               = 29
#INTERNET_OPTION_PROXY_USERNAME         = 43
#INTERNET_OPTION_PROXY_PASSWORD         = 44


#INTERNET_INVALID_PORT_NUMBER           = 0          
#INTERNET_DEFAULT_FTP_PORT              = 21         
#INTERNET_DEFAULT_GOPHER_PORT           = 70         
#INTERNET_DEFAULT_HTTP_PORT             = 80         
#INTERNET_DEFAULT_HTTPS_PORT            = 443        
#INTERNET_DEFAULT_SOCKS_PORT            = 1080       

#INTERNET_SERVICE_URL                   = 0
#INTERNET_SERVICE_FTP                   = 1
#INTERNET_SERVICE_GOPHER                = 2
#INTERNET_SERVICE_HTTP                  = 3

#INTERNET_OPEN_TYPE_PRECONFIG           = 0
#INTERNET_OPEN_TYPE_DIRECT              = 1
#INTERNET_OPEN_TYPE_PROXY               = 3
#INTERNET_OPEN_TYPE_PRECONFIG_WITH_NO_AUTOPROXY = 4   



Enumeration ; PBL constants
 #PBL_OK                        = $0001

 #PBL_WRITE_TO_FILE 
 #PBL_WRITE_TO_MEMORY
 
;******************************************************************************
;*** ERROR CONSTANTS **********************************************************
;******************************************************************************

 #PBL_ERR_CALL_FAILED           = $8000
 #PBL_ERR_INVALID_PARAMETERS            
 #PBL_ERR_OUT_OF_MEMORY          
 #PBL_ERR_ABORT_REQUESTED
 #PBL_ERR_NOT_FOUND             
 
 #PBL_ERR_FILE_IO
 #PBL_ERR_FILE_CREATION
 
 #PBL_ERR_HTTP_STATUS

 #PBL_ERR_INTERNET_OPEN
 #PBL_ERR_INTERNET_CONNECT
 #PBL_ERR_INTERNET_SET_PROXY
 #PBL_ERR_INTERNET_BASIC_AUTH
 #PBL_ERR_INTERNET_HTTP_OPEN
 #PBL_ERR_INTERNET_HTTP_SEND
 #PBL_ERR_INTERNET_HTTP_QUERY_STATUS
 #PBL_ERR_INTERNET_READ
 #PBL_ERR_INTERNET_CLOSE

EndEnumeration



Prototype   HTTPGetFromWeb_Start    (*tHTTP, hRequest.l)
Prototype   HTTPGetFromWeb_End      (*tHTTP, lRetVal.l, lBytesReceived.l, lSize.l, lElapsedTime.l)
Prototype.l HTTPGetFromWeb_CallBack (*tHTTP, lBytesReceived.l, lSize.l, lElapsedTime.l)


Structure T_PBL_HTTP_GET_FROM_WEB ; used by HTTPGetFromWeb()

 lThreadID.l        ; *must* contain a number >= 0 to identify the calling thread in the callback function
                    ; if you are not calling the proc in multithreading you can safely set this to 0
 
 sURL.s             ; *must* contain the full URL (eg: http://www.domain.com/foo/bar/file.zip)
 lDestination.l     ; *must* contain #PBL_WRITE_TO_FILE or #PBL_WRITE_TO_MEMORY 
 lChunkSize.l       ; *must* contain the size of the data chunk we want to read for any iteration, or 0 if we want all in one big block
 
 sUserAgent.s       ; defaults to "PureBasic for Windows (HTTPGetFromWeb)" if empty
 
 lPortNumber.l      ; defaults to 80 if not specified (if = 0)
 
 lAccess.l          ; defaults to #INTERNET_OPEN_TYPE_DIRECT if 0 (see the procedure body for other values)
 
 lFlags.l           ; defaults to #INTERNET_FLAG_NO_UI | #INTERNET_FLAG_RELOAD | #INTERNET_FLAG_NO_AUTO_REDIRECT if 0 
                    ; (see the procedure body for more values and description)
 
 sFullFileName.s    ; if lDestination = #PBL_WRITE_TO_FILE *must* contain the full pathname of the destination file
 
 sProxyAndPort.s    ; if lAccess = #INTERNET_OPEN_TYPE_PROXY you *must* specify a proxy here (eg: "192.168.1.1:80")
 sProxyUsername.s   ; if lAccess = #INTERNET_OPEN_TYPE_PROXY and authentication is required *must* contain the username 
 sProxyPassword.s   ; if lAccess = #INTERNET_OPEN_TYPE_PROXY and authentication is required *must* contain the password     
 
 lTotBytesRead.l    ; if return code = #PBL_OK will contain the total number of bytes read                     
                    ; else it will contain 0
                    
 lErrorCodeEx.l     ; if return code = #PBL_ERR_HTTP_STATUS will contain the actual HTTP status code
                    ; else it will contain 0
                    
                    ; some common status code you can encounter are:
                    
                    ; 301 moved permanently
                    ; 302 moved temporarily
                    ; 307 temporary redirect 
                    ; 400 bad request
                    ; 401 unauthorized
                    ; 403 forbidden
                    ; 404 not found
                    ; 407 proxy authorization required
                    ; 408 request timeout
                    ; 410 resource is gone
                    ; 500 internal server erro
                    ; 502 bad gateway
                    ; 503 service unavailable
                    ; 504 gateway timeout
                     
 *DestBuffer        ; if lDestination = #PBL_WRITE_TO_MEMORY will contain the address of the memory buffer allocated by the proc
                    ; else it will contain 0
                    
 lElapsedTime.l      

 fpCallBack.HTTPGetFromWeb_CallBack ; if not #Null, the callback procedure will be called for any iteration 
                                    ; or one time only if lChunkSize = 0

 fpCallStart.HTTPGetFromWeb_Start   ; if not #Null, the callback procedure will be called after params validation
                                    ; and when download is about to start                                     
                                    
 fpCallEnd.HTTPGetFromWeb_End       ; if not #Null, the callback procedure will be called on exiting
                                    ; in every case (success or failure)                                    
EndStructure 


Declare.l   HTTPGetFromWeb (*tHTTP.T_PBL_HTTP_GET_FROM_WEB)


Procedure.l URLGetPath (sURL.s, *sPath.STRING)
; [in] sURL
; The full url to process

; [out] *sPath
; Pointer to a STRING that will contain the portion of the url at the right of the domain

; [return]
; #PBL_OK 
; #PBL_ERR_NOT_FOUND

 Protected lPos.l, lPosEnd.l

 lPos = FindString(sURL, ":", 1)
 If lPos 
    If Mid(sURL, lPos + 1, 2) = "//"
        lPosEnd = FindString(sURL, "/", lPos + 3)

        If lPosEnd             
            *sPath\s = Right(sURL, Len(sURL) - lPosEnd + 1)
            ProcedureReturn #PBL_OK
        EndIf                        
    EndIf
 EndIf
 
 *sPath\s = ""
 
 ProcedureReturn #PBL_ERR_NOT_FOUND
EndProcedure

Procedure.l URLGetDomain (sURL.s, *sDomain.STRING)
; [in] sURL
; The full url to process

; [out] *sDomain
; Pointer to a STRING that will contain the domain extracted from the full url right after the protocol 

; [return]
; #PBL_OK
; #PBL_ERR_NOT_FOUND

 Protected lPos.l, lPosEnd.l

 lPos = FindString(sURL, "@", 1)
 
 If lPos
    lPosEnd = FindString(sURL, "/", lPos + 1)

        If lPosEnd 
            *sDomain\s = Mid(sURL, lPos + 1, lPosEnd - lPos - 1) 
            ProcedureReturn #PBL_OK
        EndIf
 EndIf
 
 lPos = FindString(sURL, ":", 1)
 
 If lPos    
    If Mid(sURL, lPos + 1, 2) = "//"
        lPosEnd = FindString(sURL, "/", lPos + 3)
        
        If lPosEnd
            lPosEnd - (LPos + 3)            
        Else
            lPosEnd = Len(sURL) - (lPos + 2)
        EndIf        
        
        If lPosEnd 
            *sDomain\s = Mid(sURL, lPos + 3, lPosEnd)    
            ProcedureReturn #PBL_OK
        EndIf
    EndIf
 EndIf
 
 *sDomain\s = ""
 
 ProcedureReturn #PBL_ERR_NOT_FOUND
EndProcedure

Procedure.l URLGetBasicAuth (sURL.s, *sUser.STRING, *sPassword.STRING)
; [in] sURL
; The full url to process (for example http://user:pass@www.somedomain.com/index.html)

; [out] *sUser
; Pointer to a STRING that will contain the user name extracted from the full url 

; [out] *sPassword
; Pointer to a STRING that will contain the password extracted from the full url 

; [return]
; #PBL_OK
; #PBL_ERR_NOT_FOUND

 Protected lPos.l, lPosStart.l, lPosEnd.l

 lPos = FindString(sURL, "@", 1)
 
 If lPos
    lPosEnd = lPos
    
    lPosStart = FindString(sURL, "/", 1)
    
    If Mid(sURL, lPosStart, 2) = "//"

        lPos = FindString(sURL, ":", lPosStart + 2)
        *sPassword\s = Mid(sURL, lPos + 1, lPosEnd - lPos - 1) 
        *sUser\s = Mid(sURL, lPosStart + 2, lPos - lPosStart - 2) 
        
        If Len(*sPassword\s) And Len(*sUser\s)
            ProcedureReturn #PBL_OK
        EndIf
    EndIf
 EndIf
 
 *sUser\s = ""
 *sPassword\s = ""
 
 ProcedureReturn #PBL_ERR_NOT_FOUND
EndProcedure

Procedure.l HTTPGetFromWeb (*tHTTP.T_PBL_HTTP_GET_FROM_WEB)
; [in / out] *tHTTP
; 
; See the definition above of structure T_PBL_HTTP_GET_FROM_WEB for the usage of the input / output fields

; [return]

; #PBL_OK                               if successful
; #PBL_ERR_INVALID_PARAMETERS           if parameters are missing or invalid
; #PBL_ERR_OUT_OF_MEMORY                if there is an error when allocating memory for the data buffers
; #PBL_ERR_FILE_CREATION                if there is an error while creating the file to disk
; #PBL_ERR_FILE_IO                      if there is an error while writing the file to disk
; #PBL_ERR_ABORT_REQUESTED              if the callback return #False it's because the download is aborted by request

; #PBL_ERR_HTTP_STATUS                  if different from HTTP OK (200), the value will be returned in *tHTTP\lErrorCodeEx

; #PBL_ERR_INTERNET_OPEN                
; #PBL_ERR_INTERNET_CONNECT
; #PBL_ERR_INTERNET_SET_PROXY
; #PBL_ERR_INTERNET_BASIC_AUTH
; #PBL_ERR_INTERNET_HTTP_OPEN
; #PBL_ERR_INTERNET_HTTP_SEND
; #PBL_ERR_INTERNET_HTTP_QUERY_STATUS
; #PBL_ERR_INTERNET_READ
; #PBL_ERR_INTERNET_CLOSE

 
; [notes]

; The procedure will fall back to use the chunk method even if the "all in one block" is requested if the server don't
; return the total size of the file we are about to download.
; This for example happen when downloading stock quotes from the Yahoo site.

; The callback procedure can optionally abort a download (if in chunk mode) when a user defined custom condition arise, 
; returning #False instead of #True.

; *tHTTP\lAccess values             
; 
; #INTERNET_OPEN_TYPE_DIRECT        = direct connection to the Internet.
; #INTERNET_OPEN_TYPE_PROXY         = passes requests to the proxy specified in sProxyAndPort
;
; *tHTTP\lFlags values
;
; #INTERNET_FLAG_NO_UI              = disables the cookie dialog box if cookies received
; #INTERNET_FLAG_RELOAD             = forces a download of the requested file, object, or directory listing from the server, not from the cache
; #INTERNET_FLAG_NO_COOKIES         = does not automatically add cookie headers to requests, and does not automatically add returned cookies to the cookie database
; #INTERNET_FLAG_NO_AUTO_REDIRECT   = does not automatically handle redirection in HttpSendRequest
 
 Protected sDomain.STRING, sPath.STRING
 Protected hInet.l, hInetCon.l, hReq.l 
 Protected lStatusCode.l, lContentLen.l, lBytesRead.l, lReadUntilNow.l, lBufSize.l, lLongSize.l = SizeOf(Long), lStartTime.l 
 Protected nFileNum.l, flgEOF.l = #False
 Protected flgBasicAuthTried.l, sBasicUser.STRING, sBasicPass.STRING
 Protected lDefaultChunkSize.l = 65536 ; 64 KBytes - used when chunk size is not specified (0) and a fallback to chunk mode is needed 
 Protected lRetVal.l
 
 ; clean up
 
 *tHTTP\lTotBytesRead = 0
 *tHTTP\lErrorCodeEx = 0
 flgBasicAuthTried = #False
 
 lStartTime =  ElapsedMilliseconds()
 
 ; sanity checks   
 
 If URLGetDomain (*tHTTP\sURL, @sDomain) <> #PBL_OK ; domain malformed
    lRetVal = #PBL_ERR_INVALID_PARAMETERS 
    Goto lbl_HTTPGetFromWeb_Exit    
 EndIf

 If URLGetPath (*tHTTP\sURL, @sPath) <> #PBL_OK ; url path malformed
    lRetVal = #PBL_ERR_INVALID_PARAMETERS      
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf

 URLGetBasicAuth (*tHTTP\sURL, @sBasicUser, @sBasicPass) 
 
 If *tHTTP\lDestination = #PBL_WRITE_TO_FILE
    If Len(*tHTTP\sFullFileName) = 0 ; no filename specified
        lRetVal = #PBL_ERR_INVALID_PARAMETERS         
        Goto lbl_HTTPGetFromWeb_Exit
    EndIf
 ElseIf  *tHTTP\lDestination = #PBL_WRITE_TO_MEMORY            
    *tHTTP\DestBuffer = #Null
 Else 
     lRetVal = #PBL_ERR_INVALID_PARAMETERS                     
     Goto lbl_HTTPGetFromWeb_Exit
 EndIf
  
 If *tHTTP\lAccess = #INTERNET_OPEN_TYPE_PROXY
    If Len(*tHTTP\sProxyAndPort) = 0
        lRetVal = #PBL_ERR_INVALID_PARAMETERS            
        Goto lbl_HTTPGetFromWeb_Exit
    EndIf    
 EndIf
 
 If *tHTTP\lChunkSize < 0 ; paranoid
    *tHTTP\lChunkSize = 0
 EndIf

 If *tHTTP\lThreadID < 0 ; paranoid
    *tHTTP\lThreadID = 0
 EndIf

 ; defaults
 
 If *tHTTP\lAccess = 0 ; if access type not specified    
    *tHTTP\lAccess = #INTERNET_OPEN_TYPE_DIRECT
 EndIf

 If Len(*tHTTP\sUserAgent) = 0 ; if user agent not specified  
    *tHTTP\sUserAgent = "PureBasic for Windows (HTTPGetFromWeb)"
 EndIf

 If *tHTTP\lFlags = 0 ; if flags not specified    
    ; no popup for cookies + load from Internet (not cache) 
    *tHTTP\lFlags = #INTERNET_FLAG_NO_UI | #INTERNET_FLAG_RELOAD 
 EndIf 
 
 If *tHTTP\lPortNumber = 0 ; if HTTP port not specified
    *tHTTP\lPortNumber = 80
 EndIf
 

 
 ; *** Internet Open ***
 
 hInet = InternetOpen_(*tHTTP\sUserAgent, *tHTTP\lAccess, *tHTTP\sProxyAndPort, #Null, 0)

 If hInet = #Null
    lRetVal = #PBL_ERR_INTERNET_OPEN
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf

 
 ; *** Internet Connect ***     
 
 hInetCon = InternetConnect_(hInet, sDomain\s, *tHTTP\lPortNumber, #Null, #Null, #INTERNET_SERVICE_HTTP, 0, 0)

 If hInetCon = #Null 
    lRetVal = #PBL_ERR_INTERNET_CONNECT
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf


 ; *** proxy authentication required ? *** 
 
 If *tHTTP\lAccess = #INTERNET_OPEN_TYPE_PROXY
    If Len(*tHTTP\sProxyUsername)
        If InternetSetOption_(hInetCon, #INTERNET_OPTION_PROXY_USERNAME, *tHTTP\sProxyUsername, Len(*tHTTP\sProxyUsername)) = #False
            lRetVal = #PBL_ERR_INTERNET_SET_PROXY
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf
    EndIf    
    
   If Len(*tHTTP\sProxyPassword)
        If InternetSetOption_(hInetCon, #INTERNET_OPTION_PROXY_PASSWORD, *tHTTP\sProxyPassword, Len(*tHTTP\sProxyPassword)) = #False
            lRetVal = #PBL_ERR_INTERNET_SET_PROXY
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf
    EndIf        
 EndIf
  


; *** Open Request ***     

 hReq = HttpOpenRequest_(hInetCon, "GET", sPath\s, #Null, #Null, #Null, *tHTTP\lFlags, 0)

 If hReq = #Null
    lRetVal = #PBL_ERR_INTERNET_HTTP_OPEN
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf        

 
 Repeat  
    ; *** Send Request ***     
        
     If (Not HttpSendRequest_(hReq, #Null, 0, 0, 0))   
        lRetVal = #PBL_ERR_INTERNET_HTTP_SEND
        Goto lbl_HTTPGetFromWeb_Exit
     EndIf
        
    ; *** Query Info (status code) ***     
    
     If (Not HttpQueryInfo_(hReq, #HTTP_QUERY_FLAG_NUMBER | #HTTP_QUERY_STATUS_CODE, @lStatusCode, @lLongSize, #Null))
        lRetVal = #PBL_ERR_INTERNET_HTTP_QUERY_STATUS
        Goto lbl_HTTPGetFromWeb_Exit
     EndIf
    
     ; *** basic authentication required ? ***   
     
     If lStatusCode = 401 And flgBasicAuthTried = #False ; we try one time only
        If Len(sBasicUser\s)
            If InternetSetOption_(hInetCon, #INTERNET_OPTION_USERNAME , @sBasicUser\s, Len(sBasicUser\s)) = #False
                lRetVal = #PBL_ERR_INTERNET_BASIC_AUTH
                Goto lbl_HTTPGetFromWeb_Exit
            EndIf
        EndIf    
        
       If Len(sBasicPass\s)
            If InternetSetOption_(hInetCon, #INTERNET_OPTION_PASSWORD, @sBasicPass\s, Len(sBasicPass\s)) = #False
                lRetVal = #PBL_ERR_INTERNET_BASIC_AUTH
                Goto lbl_HTTPGetFromWeb_Exit
            EndIf            
       EndIf     
       
       flgBasicAuthTried = #True     
     Else
        Break ; exit this loop
     EndIf     
 ForEver
 
 
; *** check status code ***

 If lStatusCode <> #HTTP_STATUS_OK
     *tHTTP\lErrorCodeEx =  lStatusCode
    lRetVal = #PBL_ERR_HTTP_STATUS
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf

 
 ; *** Query Info (content length) ***     
 
 If (Not HttpQueryInfo_(hReq, #HTTP_QUERY_FLAG_NUMBER | #HTTP_QUERY_CONTENT_LENGTH, @lContentLen, @lLongSize, #Null))
    lContentLen = 0 ; if failed, we fall back to set content length = 0
 EndIf

 
 If *tHTTP\lChunkSize = 0
    If lContentLen > 0 
        ; we want to read it all at once AND we know the total size         
        If lContentLen > 10*1024*1024 ; if > 10 MB
            lBufSize = lDefaultChunkSize ; fall back to chunk mode with default size
        Else
            lBufSize = lContentLen ; ok, it is acceptable ... one big chunk
        EndIf
    Else
        ; we want to read it all at once BUT we don't know the total size         
        ; fall back to chunk mode with default size
        lBufSize = lDefaultChunkSize
    EndIf
 Else
    ; we want to read it in chunk mode
    lBufSize = *tHTTP\lChunkSize 
 EndIf


 ; allocate memory buffer
 *tHTTP\DestBuffer = AllocateMemory (lBufSize)
 If (*tHTTP\DestBuffer = 0)
    lRetVal = #PBL_ERR_OUT_OF_MEMORY
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf

 ; create destination file if needed
 If *tHTTP\lDestination = #PBL_WRITE_TO_FILE  
    nFileNum = CreateFile (#PB_Any, *tHTTP\sFullFileName)
    If nFileNum = 0
        lRetVal = #PBL_ERR_FILE_CREATION
        Goto lbl_HTTPGetFromWeb_Exit
    EndIf
 EndIf

 lReadUntilNow = 0

 ; *** download is about to start ***
 
 If *tHTTP\fpCallStart
    *tHTTP\fpCallStart (*tHTTP, hReq) 
 EndIf
                    
 
 Repeat                    
    
    If *tHTTP\lDestination = #PBL_WRITE_TO_MEMORY
            
        If (InternetReadFile_(hReq, *tHTTP\DestBuffer + lReadUntilNow, lBufSize, @lBytesRead)) = #False
            lRetVal = #PBL_ERR_INTERNET_READ
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf
    ElseIf *tHTTP\lDestination = #PBL_WRITE_TO_FILE  
        If (InternetReadFile_(hReq, *tHTTP\DestBuffer, lBufSize, @lBytesRead)) = #False
            lRetVal = #PBL_ERR_INTERNET_READ            
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf        
    
        If (lBytesRead) : WriteData (nFileNum, *tHTTP\DestBuffer, lBytesRead) : EndIf

        ; check if it's going well
        If Lof(nFileNum) <> lReadUntilNow + lBytesRead
            lRetVal = #PBL_ERR_FILE_IO                
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf
    EndIf                
      
    lReadUntilNow + lBytesRead

    If *tHTTP\fpCallBack And lBytesRead
        If *tHTTP\fpCallBack (*tHTTP, lReadUntilNow, lContentLen, (ElapsedMilliseconds() - lStartTime) / 1000) = #False
            
            ; abort requested by the callback                                                            
            lRetVal = #PBL_ERR_ABORT_REQUESTED                    
            Goto lbl_HTTPGetFromWeb_Exit
        EndIf
    EndIf
                          
    If (lBytesRead); probably more to come        
            
        If *tHTTP\lDestination = #PBL_WRITE_TO_MEMORY
            *tHTTP\DestBuffer = ReAllocateMemory (*tHTTP\DestBuffer, lReadUntilNow + lBufSize)
        EndIf
                    
    Else ; last chunk
        
        If (*tHTTP\lDestination = #PBL_WRITE_TO_MEMORY)     
           *tHTTP\DestBuffer = ReAllocateMemory (*tHTTP\DestBuffer, lReadUntilNow)          
        EndIf
        
        flgEOF = #True
    EndIf

    If (*tHTTP\DestBuffer = 0)
        lRetVal = #PBL_ERR_OUT_OF_MEMORY
        Goto lbl_HTTPGetFromWeb_Exit
    EndIf
            
 Until flgEOF       

           
 If *tHTTP\lDestination = #PBL_WRITE_TO_FILE                  
    FlushFileBuffers(nFileNum)
    
    If Lof(nFileNum) <> lReadUntilNow
        lRetVal = #PBL_ERR_FILE_IO
        Goto lbl_HTTPGetFromWeb_Exit
    EndIf
                          
    CloseFile(nFileNum)                        
 EndIf

 *tHTTP\lTotBytesRead = lReadUntilNow
 
 ; *** close connection ***
 
 If (Not InternetCloseHandle_(hInetCon))
    lRetVal = #PBL_ERR_INTERNET_CLOSE
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf

 ; *** close Internet ***
 
 If (Not InternetCloseHandle_(hInet))
    lRetVal = #PBL_ERR_INTERNET_CLOSE
    Goto lbl_HTTPGetFromWeb_Exit
 EndIf  
                     
 lRetVal = #PBL_OK

  
 lbl_HTTPGetFromWeb_Exit:
 
 If *tHTTP\fpCallEnd
    *tHTTP\fpCallEnd (*tHTTP, lRetVal, lReadUntilNow, lContentLen, (ElapsedMilliseconds() - lStartTime) / 1000) 
 EndIf
 
 If (*tHTTP\lDestination = #PBL_WRITE_TO_FILE)
    ; free up the memory buffer
    If (*tHTTP\DestBuffer)
        FreeMemory(*tHTTP\DestBuffer)
        *tHTTP\DestBuffer = 0 
    EndIf
    
    ; if an error is happened, the file should be still opened
    If (lRetVal <> #PBL_OK) And IsFile(nFileNum) 
        ; clean up the partial file
        CloseFile(nFileNum)        
        DeleteFile(*tHTTP\sFullFileName)            
    EndIf    
 EndIf
    
 ProcedureReturn lRetVal
 
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x64)
; CursorPosition = 843
; FirstLine = 304
; Folding = w
; EnableThread
; EnableAdmin
; CPU = 1
; CurrentDirectory = D:\Work\PureBasic\Projects\