;/------------------\
;| Game Fixer       |
;| Injector         |
;|                  |
;| Version 0.01     |
;| Date: 28.04.2012 |
;| PureBasic 4.60   |
;|                  |
;| by Thorium       |
;\------------------/

EnableExplicit

XIncludeFile "HTTPGetFromWeb2.pbi"

#AppTitle = "GameFixer"

#CoreFileName = "GameFixer Core.dll"

#SharedData_Name = "Game Fixer Shared Data"
#SharedData_Size = 1024

#SystemFolder = "System\"
#PluginFolder = "Plugins\"

#DefaultUpdateServer = "http://www.GameTreasure.de/GameFixer/Update/"

#WaitCoreTimeOut = 60000

#DQuoteMask = "/#DQ/"

Enumeration 1
  #Error_PluginMissing
  #Error_PluginWrongFormat
  #Error_PluginWrongConfig
EndEnumeration

Enumeration
  #Mode_Normal
  #Mode_DontAsk
  #Mode_Silent
EndEnumeration

Structure SharedData
  Ready.a
  Error.l
  PlugInsCnt.u
  GfPathLd.u
EndStructure

Structure SharedDataPlugin
  FileLd.a
  ConfigLd.u
EndStructure

Global *SharedData.SharedData
Global SelfPath.s
Global Mode.i
Global AutoConfig.i
Global UpdateMode.i
Global NetworkInitialized.i
Global Target.s
Global TargetParameters.s

Global NewList UpdateServer.s()

Global Dim InstalledPlugins.s(1)
Global InstalledPluginsCnt.i

Global Dim UsePlugins.s(1)
Global Dim UsePluginsConfig.s(1)
Global UsePluginsCnt.i

Procedure InitConsole()
  
  If Mode <> #Mode_Silent
    
    If OpenConsole() = 0
      MessageRequester(#AppTitle, "Error: Failed to open console!", #MB_ICONERROR)
      End
    EndIf
    
    PrintN("")
    PrintN("GameFixer v0.01")

  EndIf
  
EndProcedure

Procedure Write(Text.s)
  
  If Mode <> #Mode_Silent
    Print(Text)
  EndIf
    
EndProcedure

Procedure WriteN(Text.s)
  
  If Mode <> #Mode_Silent
    PrintN(Text)
  EndIf
    
EndProcedure

Procedure EndProgram()
  
  WriteN("")
  WriteN("Press enter to end program.")
  
  If Mode < #Mode_DontAsk
    
    Repeat
      Delay(20)
    Until Inkey() = Chr(13)
      
  EndIf
    
  End
   
EndProcedure

Procedure AllocSharedMemory(MemoryName.s, SizeOfSharedData.i)
  
  Define hMapping.i
  
  hMapping = CreateFileMapping_(#INVALID_HANDLE_VALUE, 0, #PAGE_READWRITE, 0, SizeOfSharedData, MemoryName)
  hMapping = MapViewOfFile_(hMapping, #FILE_MAP_WRITE, 0, 0, 0)
  
  ProcedureReturn hMapping

EndProcedure

Procedure FreeSharedMemory(hMapping.i)
  
  UnmapViewOfFile_(hMapping)
  
EndProcedure

Procedure.s GetHttpErrorString(Error.i, *tHttp.T_PBL_HTTP_GET_FROM_WEB)
  
  Select Error
  
    Case #PBL_ERR_INTERNET_OPEN                
      ProcedureReturn "Error initializing the WININET.DLL API."
        
    Case #PBL_ERR_INTERNET_CONNECT
      ProcedureReturn "Error opening the session to the given site."
        
    Case #PBL_ERR_INTERNET_SET_PROXY
      ProcedureReturn "Error setting the options for the proxy server."
        
    Case #PBL_ERR_INTERNET_HTTP_OPEN
      ProcedureReturn "Error creating the HTTP request handle."
        
    Case #PBL_ERR_INTERNET_HTTP_SEND
      ProcedureReturn "Error sending the HTTP request to the server."
        
    Case #PBL_ERR_INTERNET_HTTP_QUERY_STATUS
      ProcedureReturn "Error retrieving the HTTP status code."
        
    Case #PBL_ERR_INTERNET_READ
      ProcedureReturn "Error reading data from the opened session."
        
    Case #PBL_ERR_INTERNET_CLOSE
      ProcedureReturn "Error closing the Internet handle."
    
    Case #PBL_ERR_INTERNET_BASIC_AUTH
      ProcedureReturn "Error setting the options for basic authentication."
    
    Case #PBL_ERR_HTTP_STATUS                      
      ProcedureReturn "HTTP status code = " + Str(*tHTTP\lErrorCodeEx)

    Case #PBL_ERR_FILE_CREATION                
      ProcedureReturn "Cannot create the file " + *tHTTP\sFullFileName
        
    Case #PBL_ERR_FILE_IO                      
      ProcedureReturn "Error writing to the file " + *tHTTP\sFullFileName
                
    Case #PBL_ERR_OUT_OF_MEMORY                        
      ProcedureReturn "Out of memory." 

    Case #PBL_ERR_ABORT_REQUESTED              
      ProcedureReturn "Download aborted."                    
        
    Default
      ProcedureReturn "Unexpected error = " + Str(Error)
      
  EndSelect
  
EndProcedure

Procedure.i DownloadUpdSvrLst(Server.s)
  
  Protected Ret.i
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  
  tHttp\lThreadID = 0 
  tHttp\sURL = Server + "UpdateServer.lst"
  tHttp\lDestination = #PBL_WRITE_TO_FILE
  tHttp\sFullFileName = SelfPath + "UpdateServer.lst"
  Ret = HTTPGetFromWeb(@tHttp)
  
  If Ret = #PBL_OK
    ProcedureReturn #True
  Else
    WriteN("Error: Can't download UpdateServer.lst")
    WriteN(GetHttpErrorString(Ret, tHttp))
    ProcedureReturn #False
  EndIf

EndProcedure

Procedure OnlineUpdate()
  
  Protected UpdSvrLstFile.i
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  Protected Ret.i
  
  ;initialize network lib
  If NetworkInitialized = #False
    If InitNetwork() = 0
      WriteN("Error: Can't initialize network!")
      EndProgram()
    EndIf
  EndIf
  
  ;load the list of update servers
  
  UpdSvrLstFile = ReadFile(#PB_Any, SelfPath + "UpdateServer.lst")
  
  If UpdSvrLstFile = 0
    WriteN("update server list file not found")
    WriteN("trying to get new one from " + #DOUBLEQUOTE$ + #DefaultUpdateServer + #DOUBLEQUOTE$ + "...")
    If DownloadUpdSvrLst(#DefaultUpdateServer) = #False
      WriteN("Online update failed!")
      EndProgram()
    EndIf    
  EndIf
  
  
  
  
  
  
  
  
  
  
  
EndProcedure

Procedure.i StartAndInject(GameFile.s, CommandLineParameters.s, Array PlugIns.s(1), Array PlugInConfigs.s(1))
  
  Protected lpStartupInfo.STARTUPINFO
  Protected lpProcessInfo.PROCESS_INFORMATION
  Protected FileNameLen.i
  Protected FileNameAddr.i
  Protected BytesWritten.i
  Protected hKernel32.i
  Protected LoadLibraryAAddr.i
  Protected hThread.i
  Protected *SharedDataPlugin.SharedDataPlugin
  Protected i.i
  Protected WaitCoreStartTime.i
  Protected FileName.s
  
  ;initialize shared data for game fixer core configuration
  
  *SharedData = AllocSharedMemory(#SharedData_Name, #SharedData_Size)
  
  *SharedData\Ready      = #False
  *SharedData\Error      = 0
  *SharedData\GfPathLd   = Len(SelfPath)
  *SharedData\PlugInsCnt = ArraySize(PlugIns())
  
  CopyMemory(@SelfPath, *SharedData + SizeOf(SharedData), *SharedData\GfPathLd)
  
  *SharedDataPlugin = *SharedData + SizeOf(SharedData) + *SharedData\GfPathLd
  
  For i = 1 To *SharedData\PlugInsCnt
    
    *SharedDataPlugin\FileLd   = Len(PlugIns(i))
    *SharedDataPlugin\ConfigLd = Len(PlugInConfigs(i))
    
    CopyMemory(@PlugIns(i), *SharedDataPlugin + SizeOf(SharedDataPlugin), *SharedDataPlugin\FileLd)
    CopyMemory(@PlugInConfigs(i), *SharedDataPlugin + SizeOf(SharedDataPlugin) + *SharedDataPlugin\FileLd, *SharedDataPlugin\ConfigLd)
    
    *SharedDataPlugin = *SharedDataPlugin + SizeOf(SharedDataPlugin) + *SharedDataPlugin\FileLd + *SharedDataPlugin\ConfigLd

  Next

  ;start up game process
  
  WriteN("")
  WriteN("Launching " + GameFile + "...")
  
  lpStartupInfo\cb = SizeOf(STARTUPINFO)
  
  If CreateProcess_(0, GameFile + " " + CommandLineParameters, 0, 0, #False, #CREATE_SUSPENDED | #DETACHED_PROCESS | #NORMAL_PRIORITY_CLASS, 0, GetPathPart(GameFile), lpStartupInfo, lpProcessInfo) = 0
    FreeSharedMemory(*SharedData)
    WriteN("")
    WriteN("Error: Failed to launch game!")
    ProcedureReturn #False
  EndIf
  
  ;inject game fixer core dll
  
  WriteN("Injecting GameFixer core into game process...")
  
  FileName = SelfPath + #SystemFolder + #CoreFileName
  FileNameLen = Len(FileName) + 1

  FileNameAddr = VirtualAllocEx_(lpProcessInfo\hProcess, 0, FileNameLen, #MEM_RESERVE | #MEM_COMMIT, #PAGE_READWRITE)
  If FileNameAddr = 0
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to access game process!")
    ProcedureReturn #False
  EndIf
  
  If WriteProcessMemory_(lpProcessInfo\hProcess, FileNameAddr, FileName, FileNameLen, @BytesWritten) = 0
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to access game process!")
    ProcedureReturn #False
  EndIf
  If BytesWritten <> FileNameLen
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to access game process!")
    ProcedureReturn #False
  EndIf

  hKernel32 = GetModuleHandle_("Kernel32.dll")
  If hKernel32 = 0
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to get Kernel32.dll!")
    ProcedureReturn #False
  EndIf
  LoadLibraryAAddr = GetProcAddress_(hKernel32, "LoadLibraryA")
  If LoadLibraryAAddr = 0
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to get LoadLibraryA!")
    ProcedureReturn #False
  EndIf
  
  hThread = CreateRemoteThread_(lpProcessInfo\hProcess, 0, 0, LoadLibraryAAddr, FileNameAddr, 0, 0)
  If hThread = 0
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")
    WriteN("Error: Failed to create thread in game process!")
    ProcedureReturn #False
  EndIf

  ;wait for game fixer core dll to initialize
  
  WaitForSingleObject_(hThread, #INFINITE)
  
  ;clean up
  
  VirtualFreeEx_(lpProcessInfo\hProcess, FileNameAddr, 0, #MEM_RELEASE)
  
  ;wait for game fixer core to initialize
  
  WriteN("Waiting for GameFixer core to load plugins and do the magic...")
  
  WaitCoreStartTime = ElapsedMilliseconds()
  
  Repeat
    
    Delay(100)
    If ElapsedMilliseconds() - WaitCoreStartTime >= #WaitCoreTimeOut
      FreeSharedMemory(*SharedData)
      TerminateProcess_(lpProcessInfo\hProcess, 0)
      WriteN("")
      WriteN("Error: GameFixer core does not respond!")
      ProcedureReturn #False
    EndIf
    
  Until *SharedData\Ready = #True Or *SharedData\Error <> 0
  
  If *SharedData\Error <> 0
    
    FreeSharedMemory(*SharedData)
    TerminateProcess_(lpProcessInfo\hProcess, 0)
    WriteN("")

    Select *SharedData\Error
        
      Case #Error_PluginMissing
        WriteN("Error: A plugin is missing!")
        
      Case #Error_PluginWrongFormat
        WriteN("Error: A plugin has the wrong format or version!")
        
      Case #Error_PluginWrongConfig
        WriteN("Error: Invalid configutation for a plugin!")
        
      Default
        WriteN("Error: Unkown error!")
        
    EndSelect
    
    ProcedureReturn #False
    
  EndIf
  
  WriteN("Running game...")
  
  FreeSharedMemory(*SharedData)
  ResumeThread_(lpProcessInfo\hThread)
  
  ProcedureReturn #True

EndProcedure

Procedure.s GetSelfPath()

  Protected ModulePath.s

  ModulePath = Space(1024)
  GetModuleFileName_(0, @ModulePath, 1024)
  ProcedureReturn GetPathPart(Trim(ModulePath))

EndProcedure

Procedure GetPluginList()
  
  Protected Dir.i
  
  InstalledPluginsCnt = 0
  
  Dir = ExamineDirectory(#PB_Any, SelfPath + #PluginFolder, "*.gfp")
  If Dir = 0
    ProcedureReturn
  EndIf
  
  While NextDirectoryEntry(Dir)
    
    If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
      InstalledPluginsCnt + 1
      ReDim InstalledPlugins.s(InstalledPluginsCnt)
      InstalledPlugins(InstalledPluginsCnt) = LCase(Left(DirectoryEntryName(Dir), Len(DirectoryEntryName(Dir)) - 4))
    EndIf
    
  Wend

  FinishDirectory(Dir)
  
EndProcedure

Procedure Init()
  
  Protected ParamCnt.i
  Protected i.i
  Protected i2.i
  Protected Param.s
  Protected ParamIsPlugin.i
  Protected ParamError.i
  
  SelfPath = GetSelfPath()
  GetPluginList()
  
  UsePluginsCnt = 0
  
  ParamCnt = CountProgramParameters() - 1
  
  If ParamCnt = -1
    
    ParamError = #True
    
  Else
    
    Target = ProgramParameter(0)
  
    For i = 1 To ParamCnt
      
      Param.s = LCase(ProgramParameter(i))
      
      Select Param
          
        Case "-dontask", "-da"
          Mode = #Mode_DontAsk
          
        Case "-silent", "-s"
          Mode = #Mode_Silent
          
        Case "-auto", "-a"
          AutoConfig = #True
          
        Case "-update", "-u"
          UpdateMode = #True
          
        Case "-params", "-p"
          i + 1
          TargetParameters = ReplaceString(ProgramParameter(i), #DQuoteMask, #DQUOTE$)
  
        Default
          
          For i2 = 1 To InstalledPluginsCnt
            
            If Param = "-" + InstalledPlugins(i2)
              
              UsePluginsCnt + 1
              ReDim UsePlugins.s(UsePluginsCnt)
              ReDim UsePluginsConfig.s(UsePluginsCnt)
              
              UsePlugins(UsePluginsCnt) = InstalledPlugins(i2) + ".gfp"
              UsePluginsConfig(UsePluginsCnt) = ProgramParameter(i + 1)
              If Left(UsePluginsConfig(UsePluginsCnt), 1) = "-"
                UsePluginsConfig(UsePluginsCnt) = ""
              Else
                i + 1
              EndIf
              
              ParamIsPlugin = #True
              
            EndIf
  
          Next
          
          If ParamIsPlugin = #False
            ParamError = #True
          Else
            ParamIsPlugin = #False
          EndIf
  
      EndSelect
      
    Next
    
  EndIf

  InitConsole()
  
  If ParamError = #True
    WriteN("")
    If ParamCnt = -1
      WriteN("Error: Missing game executable file name!")
    Else
      WriteN("Error: Unknown parameter!")
    EndIf
    EndProgram()
  EndIf
  
  If UsePluginsCnt = 0 And AutoConfig = #False
    WriteN("")
    WriteN("Error: No plugins specified!")
    Writen("Specify at least one plugin or turn on auto configuration.")
    EndProgram()
  EndIf
    
EndProcedure

Init()

If StartAndInject(Target, TargetParameters, UsePlugins(), UsePluginsConfig()) = #False
  WriteN("")
  WriteN("Error: Failed to fix game!")
Else
  WriteN("")
  WriteN("Game should be fixed.")
EndIf

EndProgram()

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Console
; CursorPosition = 82
; FirstLine = 51
; Folding = BA+
; EnableXP
; Executable = B:\GameFixer\GameFixer.exe
; IncludeVersionInfo
; VersionField0 = 0,1,0,0
; VersionField1 = 0,1,0,0
; VersionField3 = Game Fixer
; VersionField9 = Janos David Ommert