;/-------------------\
;| GameFixer Plugin  |
;| Memory Fix        |
;|                   |
;| Version 0.01      |
;| Date: 30.04.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

;Import all what we need from the GameFixer core.
Import "GameFixer Core.lib"
  UninstallStdProcHook(HookId.i)
  GetOrigStdProcAddr.i(HookId.i)
  InstallWin32ApiHook.i(DllName.s, ProcName.s, HookProcAddr.i)
EndImport

;We need this prototype to call GlobalMemoryStatus after we hooked it.
Prototype Proto_GlobalMemoryStatus(*lpBuffer.MEMORYSTATUS)
Global GlobalMemoryStatus.Proto_GlobalMemoryStatus

;Variable for the hook ID of the GlobalMemoryStatus hook.
Global GlobalMemoryStatusHookId.i

;Variables to hold the values we want to change inside the MEMORYSTATUS
;structure returned by GlobalMemoryStatus.
;-1 means dont change it.
Global MemoryLoad.i    = -1
Global TotalPhys.i     = -1
Global AvailPhys.i     = -1
Global TotalPageFile.i = -1
Global AvailPageFile.i = -1
Global TotalVirtual.i  = -1
Global AvailVirtual.i  = -1

;This procedure is called if the plugin DLL is first loaded.
ProcedureDLL AttachProcess(Instance.i)
  
  ;We do nothing here.
  ;It's not needed to include this procedure.
  
EndProcedure

;This procedure is called if the plugin DLL unloads.
ProcedureDLL DetachProcess(Instance.i)
  
  ;If our plugin unloads we need to uninstall the hook on GlobalMemoryStatus,
  ;or the game will crash if it calls it again.
  UninstallStdProcHook(GlobalMemoryStatusHookId)
  
EndProcedure

;We redirect all calls to GlobalMemoryStatus to this procedure.
Procedure GlobalMemoryStatusHook(*lpBuffer.MEMORYSTATUS)
  
  ;We call the original GlobalMemoryStatus procedure,
  ;to get the real values of the members in the MEMORYSTATUS structure.
  GlobalMemoryStatus(*lpBuffer)
  
  ;Change them if the user wants to.
  
  If MemoryLoad <> -1
    *lpBuffer\dwMemoryLoad = MemoryLoad
  EndIf
  
  If TotalPhys <> -1
    *lpBuffer\dwTotalPhys = TotalPhys
  EndIf
  
  If AvailPhys <> -1
    *lpBuffer\dwAvailPhys = AvailPhys
  EndIf
  
  If TotalPageFile <> -1
    *lpBuffer\dwTotalPageFile = TotalPageFile
  EndIf
  
  If AvailPageFile <> -1
    *lpBuffer\dwAvailPageFile = AvailPageFile
  EndIf
  
  If TotalVirtual <> -1
    *lpBuffer\dwTotalVirtual = TotalVirtual
  EndIf
  
  If AvailVirtual <> -1
    *lpBuffer\dwAvailVirtual = AvailVirtual
  EndIf
  
  ;The GlobalMemoryStatus function does not return anything,
  ;so we dont need to also.
  
EndProcedure

;This procedure is called by the GameFixer core to configure the plugin.
ProcedureDLL.i Config(Config.s)
  
  Protected i.i
  Protected Setting.s
  Protected Value.i
  
  ;We are parsing the config line in Config.s
  ;to know what members of the MEMORYSTATUS structure
  ;the user wants to change and to what values.
  
  For i = 1 To 100
    
    Setting = StringField(Config, i, " ")
    If Setting = ""
      Break
    EndIf
    
    Value = Val(StringField(Setting, 2, "="))

    Select LCase(StringField(Setting, 1, "="))
        
      Case "memoryload", "ml"
        MemoryLoad = Value
        
      Case "totalphys", "tp"
        TotalPhys = Value
        
      Case "availphys", "ap"
        AvailPhys = Value
        
      Case "totalpagefile", "tpf"
        TotalPageFile = Value
        
      Case "availpagefile", "apf"
        AvailPageFile = Value
        
      Case "totalvirtual", "tv"
        TotalVirtual = Value
        
      Case "availvirtual", "av"
        AvailVirtual = Value
        
    EndSelect

  Next
  
  ;Now we are installing a hook on GlobalMemoryStatus,
  ;so we can change the values of the MEMORYSTATUS
  ;every time the game calls GlobalMemoryStatus.
  ;The hook will redirect every call from GlobalMemoryStatus
  ;to our own procedure: GlobalMemoryStatusHook

  GlobalMemoryStatusHookId = InstallWin32ApiHook("Kernel32.dll", "GlobalMemoryStatus", @GlobalMemoryStatusHook())
  If GlobalMemoryStatusHookId = 0
    ;The hooking failed, so we tell the GameFixer core that our plugin failed by returning #False.
    ProcedureReturn #False
  EndIf
  
  ;We need to get a new address to call the original GlobalMemoryStatus procedure,
  ;without getting redirected to GlobalMemoryStatusHook.
  ;For easy calling later, we use a prototype here.
  GlobalMemoryStatus = GetOrigStdProcAddr(GlobalMemoryStatusHookId)
  
  ;We return #True to tell the GameFixer core that all wend fine.
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 165
; FirstLine = 8
; Folding = w
; EnableXP
; Executable = B:\GameFixer\Plugins\MemoryFix.dll
; IncludeVersionInfo
; VersionField0 = 0,1,0,0
; VersionField1 = 0,1,0,0
; VersionField3 = Memory Fix
; VersionField6 = Game Fixer Plugin
; VersionField9 = Janos David Ommert
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_DLL