;/-------------------\
;| GameFixer Plugin  |
;| Template          |
;|                   |
;| Version 0.01      |
;| Date: 30.04.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

;needed to use the disassembler
XIncludeFile "BeaEngine32.pbi"

;import of the functionalities exported by the GameFixer core
Import "GameFixer Core.lib"
  
  ;Disassembles a x86 instruction by using the BeaEngine.
  ;Go to http://www.beaengine.org/ to read about how to use it.
  ;There is a PureBasic example there.
  Dasm.i(*Instruction.DisasmStruc)
  
  ;Installs a standard procedure hook.
  ;Returns a hook ID you need later.
  ;Returns #False if it failed.
  ;Standard means no conversion of calling conventions is done.
  ;ProcAddr specifies the memory address of the procedure you want to hook.
  ;HookProcAddr specifies the memory address of the procedure you want to redirect to.
  ;To call the original procedure later you need a new address, you get by calling GetOrigStdProcAddr.
  ;Don't ever call ProcAddr after it is hooked or you will end up in a endless recursion.
  InstallStdProcHook.i(ProcAddr.i, HookProcAddr.i)
  
  ;Uninstalls a standard procedure hook.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  UninstallStdProcHook(HookId.i)
  
  ;Returns the address you have to call if you want to call the original procedure you had hooked before.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  GetOrigStdProcAddr.i(HookId.i)
  
  ;Installs a hook on a Win32 API function.
  ;Returns a hook ID you need later. It's valid to use with GetOrigStdProcAddr and UninstallStdProcHook.
  ;Returns #False if it failed.
  ;It's basicly the same as InstallStdProcHook.
  ;You just have To specify the library and function name instead of the procedure address.
  InstallWin32ApiHook.i(DllName.s, ProcName.s, HookProcAddr.i)
  
EndImport

;This procedure is called if the plugin DLL is first loaded.
;Do any initialization stuff here if needed,
;but dont call any exported functions from the GameFixer core here.
ProcedureDLL AttachProcess(Instance.i)

EndProcedure

;This procedure is called if the plugin DLL unloads.
;Do any cleanup here, like uninstalling hooks.
ProcedureDLL DetachProcess(Instance.i)
  
EndProcedure

;This procedure is called by the GameFixer core to configure the plugin.
;Config.s contains the config line for the plugin that was been specified
;by the command line parameters.
;Do anything you need here, like installing hooks.
;This procedure is called before the main thread of the game is resumed,
;so the game isnt running just yet and hasnt initialized anything.
;If you need the game in a initialized state, create a thread here and wait
;for it to initialize in the thread. The main thread of the game will not
;resume before all plugins are configured.
ProcedureDLL.i Config(Config.s)
  
  ;Return #True to tell the GameFixer core that your plugin is successfully
  ;configured.
  ;If something wend wrong return #False to tell the GameFixer core so.
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; CursorPosition = 59
; FirstLine = 21
; Folding = 5
; EnableXP