;/-------------------\
;| GameFixer Plugin  |
;| DDraw Hook Test   |
;|                   |
;| Version 0.01      |
;| Date: 28.06.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

;needed to use the disassembler
XIncludeFile "BeaEngine32.pbi"

;needed for the DirectDraw hooking system
XIncludeFile "GameFixer DirectDraw Hook.pbi"

;import of the functionalities exported by the GameFixer core
Import "GameFixer Core.lib"
  
  ;Disassembles a x86 instruction by using the BeaEngine.
  ;Go to http://www.beaengine.org/ to read about how to use it.
  ;There is a PureBasic example there.
  Dasm.i(*Instruction.DisasmStruc)
  
  ;Installs a standard procedure hook.
  ;Returns a hook ID you need later.
  ;Returns #False if it failed.
  ;Standard means no conversion of calling conventions is done.
  ;ProcAddr specifies the memory address of the procedure you want to hook.
  ;HookProcAddr specifies the memory address of the procedure you want to redirect to.
  ;To call the original procedure later you need a new address, you get by calling GetOrigStdProcAddr.
  ;Don't ever call ProcAddr after it is hooked or you will end up in a endless recursion.
  InstallStdProcHook.i(ProcAddr.i, HookProcAddr.i)
  
  ;Uninstalls a standard procedure hook.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  UninstallStdProcHook(HookId.i)
  
  ;Returns the address you have to call if you want to call the original procedure you had hooked before.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  GetOrigStdProcAddr.i(HookId.i)
  
  ;Installs a hook on a Win32 API function.
  ;Returns a hook ID you need later. It's valid to use with GetOrigStdProcAddr and UninstallStdProcHook.
  ;Returns #False if it failed.
  ;It's basicly the same as InstallStdProcHook.
  ;You just have To specify the library and function name instead of the procedure address.
  InstallWin32ApiHook.i(DllName.s, ProcName.s, HookProcAddr.i)
  
  
  
  
  
  ;new exports
  InstallDDrawHook.i(*Callback)
  UninstallDDrawHook(DDrawHookId.i)
  InstallDDrawProcHook(*Object, MethodeIndex.i, HookProcAddr.i)

EndImport


Prototype Proto_Flip(*this, *lpDDSurfaceTargetOverride, dwFlags.i)
Global Flip.Proto_Flip

Global DDrawHookId.i
Global FlipHookId.i

Global MaxFps.i = 30
Global FpsTimerDuration
Global StartTime.i


Procedure.s GetDDrawObjectName(ObjectType)
  
  Select ObjectType
      
    Case #GF_DD_IDirectDraw
      ProcedureReturn "IDirectDraw"
      
    Case #GF_DD_IDirectDraw2
      ProcedureReturn "IDirectDraw2"
      
    Case #GF_DD_IDirectDraw4
      ProcedureReturn "IDirectDraw4"
      
    Case #GF_DD_IDirectDraw7
      ProcedureReturn "IDirectDraw7"
      
    Case #GF_DD_IDirectDrawPalette
      ProcedureReturn "IDirectDrawPalette"
      
    Case #GF_DD_IDirectDrawClipper
      ProcedureReturn "IDirectDrawClipper"
      
    Case #GF_DD_IDirectDrawSurface
      ProcedureReturn "IDirectDrawSurface"
      
    Case #GF_DD_IDirectDrawSurface2
      ProcedureReturn "IDirectDrawSurface2"
      
    Case #GF_DD_IDirectDrawSurface3
      ProcedureReturn "IDirectDrawSurface3"
      
    Case #GF_DD_IDirectDrawSurface4
      ProcedureReturn "IDirectDrawSurface4"
      
    Case #GF_DD_IDirectDrawSurface7
      ProcedureReturn "IDirectDrawSurface7"
      
    Case #GF_DD_IDirectDrawColorControl
      ProcedureReturn "IDirectDrawColorControl"
      
    Case #GF_DD_IDirectDrawGammaControl
      ProcedureReturn "IDirectDrawGammaControl"
      
    Default
      ProcedureReturn "unknown"
      
  EndSelect
  
EndProcedure

Procedure.i DDraw_Flip_Hook(*this, *lpDDSurfaceTargetOverride, dwFlags.i)
  
  Protected Ret.i
  Protected TimeGab.i
  
  TimeGab = FpsTimerDuration - (timeGetTime_() - StartTime)

  If TimeGab > 1
    Delay(TimeGab)
  EndIf
  
  Ret = Flip(*this, *lpDDSurfaceTargetOverride, dwFlags)
  
  StartTime = timeGetTime_()
  
  ProcedureReturn Ret
  
EndProcedure




Procedure DDrawCallback(*Object, Action.i, ObjectType.i)
  
  Select Action
    
    Case #GF_DD_ObjectCreate
      
      Select ObjectType
        Case #GF_DD_IDirectDrawSurface, #GF_DD_IDirectDrawSurface2, #GF_DD_IDirectDrawSurface3, #GF_DD_IDirectDrawSurface4, #GF_DD_IDirectDrawSurface7
          If FlipHookId = 0
            FlipHookId   = InstallDDrawProcHook(*Object, #GF_DD_IDDS_Flip, @DDraw_Flip_Hook())
            Flip = GetOrigStdProcAddr(FlipHookId)
            StartTime = timeGetTime_()
          EndIf
      EndSelect
    
  EndSelect
  
EndProcedure



;This procedure is called if the plugin DLL is first loaded.
;Do any initialization stuff here if needed,
;but dont call any exported functions from the GameFixer core here.
ProcedureDLL AttachProcess(Instance.i)

EndProcedure

;This procedure is called if the plugin DLL unloads.
;Do any cleanup here, like uninstalling hooks.
ProcedureDLL DetachProcess(Instance.i)
  
  UnInstallDDrawHook(DDrawHookId)
  CloseFile(1)
  
EndProcedure

;This procedure is called by the GameFixer core to configure the plugin.
;Config.s contains the config line for the plugin that was been specified
;by the command line parameters.
;Do anything you need here, like installing hooks.
;This procedure is called before the main thread of the game is resumed,
;so the game isnt running just yet and hasnt initialized anything.
;If you need the game in a initialized state, create a thread here and wait
;for it to initialize in the thread. The main thread of the game will not
;resume before all plugins are configured.
ProcedureDLL.i Config(Config.s)
  
  DDrawHookId = InstallDDrawHook(@DDrawCallback())
  
  If DDrawHookId = 0
    ProcedureReturn #False
  EndIf
  
  FpsTimerDuration = 1000 / MaxFps

  ;Return #True to tell the GameFixer core that your plugin is successfully
  ;configured.
  ;If something wend wrong return #False to tell the GameFixer core so.
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 135
; FirstLine = 63
; Folding = C-
; EnableXP
; Executable = B:\GameFixer\Plugins\DDTest.dll
; Warnings = Display