;/-------------------\
;| GameFixer Plugin  |
;| DDraw Hook Test   |
;|                   |
;| Version 0.02      |
;| Date: 30.06.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

XIncludeFile "GameFixer DirectDraw Hook.pbi"
XIncludeFile "ddraw.pbi"

;import of the functionalities exported by the GameFixer core
Import "GameFixer Core.lib"
  UninstallStdProcHook(HookId.i)
  GetOrigStdProcAddr.i(HookId.i)
  InstallDDrawHook.i(*Callback)
  UninstallDDrawHook(DDrawHookId.i)
  InstallDDrawProcHook(*Object, MethodeIndex.i, HookProcAddr.i)
  GetDDrawVersion.i()
EndImport

Global DDrawHookId.i

Procedure.s GetDDrawObjectName(ObjectType)
  
  Select ObjectType
      
    Case #GF_DD_IDirectDraw
      ProcedureReturn "IDirectDraw"
      
    Case #GF_DD_IDirectDraw2
      ProcedureReturn "IDirectDraw2"
      
    Case #GF_DD_IDirectDraw4
      ProcedureReturn "IDirectDraw4"
      
    Case #GF_DD_IDirectDraw7
      ProcedureReturn "IDirectDraw7"
      
    Case #GF_DD_IDirectDrawPalette
      ProcedureReturn "IDirectDrawPalette"
      
    Case #GF_DD_IDirectDrawClipper
      ProcedureReturn "IDirectDrawClipper"
      
    Case #GF_DD_IDirectDrawSurface
      ProcedureReturn "IDirectDrawSurface"
      
    Case #GF_DD_IDirectDrawSurface2
      ProcedureReturn "IDirectDrawSurface2"
      
    Case #GF_DD_IDirectDrawSurface3
      ProcedureReturn "IDirectDrawSurface3"
      
    Case #GF_DD_IDirectDrawSurface4
      ProcedureReturn "IDirectDrawSurface4"
      
    Case #GF_DD_IDirectDrawSurface7
      ProcedureReturn "IDirectDrawSurface7"
      
    Case #GF_DD_IDirectDrawColorControl
      ProcedureReturn "IDirectDrawColorControl"
      
    Case #GF_DD_IDirectDrawGammaControl
      ProcedureReturn "IDirectDrawGammaControl"
      
    Default
      ProcedureReturn "unknown"
      
  EndSelect
  
EndProcedure

Procedure Create_IDirectDrawSurface(*Surface.IDirectDrawSurface)
  
  Protected PixelFormat.DDPIXELFORMAT
  Protected SurfaceDesc.DDSURFACEDESC
  
  WriteStringN(1, "--------------------------------------")
  WriteStringN(1, "pixel format infos")
  
  PixelFormat\dwSize = SizeOf(DDPIXELFORMAT)
  
  If *Surface\GetPixelFormat(@PixelFormat) = #DD_OK
    
    If PixelFormat\dwFlags & #DDPF_ALPHA = #DDPF_ALPHA
      WriteStringN(1, "-alpha-only surface")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_ALPHAPIXELS = #DDPF_ALPHAPIXELS
      WriteStringN(1, "-alpha-channel information in the pixel format")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_COMPRESSED = #DDPF_COMPRESSED
      WriteStringN(1, "-surface compresses data")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_FOURCC = #DDPF_FOURCC
      WriteStringN(1, "-FourCC code is valid")
      WriteStringN(1, "-FourCC code: " + Hex(PixelFormat\dwFourCC))
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_PALETTEINDEXED1 = #DDPF_PALETTEINDEXED1
      WriteStringN(1, "-1bit color indexed")
    EndIf

    If PixelFormat\dwFlags & #DDPF_PALETTEINDEXED2 = #DDPF_PALETTEINDEXED2
      WriteStringN(1, "-2bit color indexed")
    EndIf

    If PixelFormat\dwFlags & #DDPF_PALETTEINDEXED4 = #DDPF_PALETTEINDEXED4
      WriteStringN(1, "-4bit color indexed")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_PALETTEINDEXED8 = #DDPF_PALETTEINDEXED8
      WriteStringN(1, "-8bit color indexed")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_PALETTEINDEXEDTO8 = #DDPF_PALETTEINDEXEDTO8
      WriteStringN(1, "-1, 2, or 4bit color indexed to an 8bit palette")
    EndIf

    If PixelFormat\dwFlags & #DDPF_RGB = #DDPF_RGB
      WriteStringN(1, "-RGB data in the pixel format structure is valid")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_RGBTOYUV = #DDPF_RGBTOYUV
      WriteStringN(1, "-accept RGB Data and translate it during the write operation to YUV data")
    EndIf
    
    If PixelFormat\dwFlags & #DDPF_YUV = #DDPF_YUV
      WriteStringN(1, "-YUV data in the pixel format structure is valid")
    EndIf

    If PixelFormat\dwFlags & #DDPF_ZBUFFER = #DDPF_ZBUFFER
      WriteStringN(1, "-z-buffer-only surface")
    EndIf
    
    WriteStringN(1, "-bit count: " + Str(PixelFormat\dwRGBBitCount))
    WriteStringN(1, "-r/y bit mask: " + Hex(PixelFormat\dwRBitMask))
    WriteStringN(1, "-g/u bit mask: " + Hex(PixelFormat\dwGBitMask))
    WriteStringN(1, "-b/v bit mask: " + Hex(PixelFormat\dwBBitMask))
    WriteStringN(1, "-alpha bit mask: " + Hex(PixelFormat\dwRGBAlphaBitMask))

  Else
    
    WriteStringN(1, "ERROR: Can't get pixel format info!")
    
  EndIf
  
  WriteStringN(1, "--------------------------------------")
  WriteStringN(1, "surface infos")
  
  SurfaceDesc\dwSize = SizeOf(DDSURFACEDESC)
  
  If *Surface\GetSurfaceDesc(@SurfaceDesc) = #DD_OK
    
    WriteStringN(1, "-height: " + Str(SurfaceDesc\dwHeight))
    WriteStringN(1, "-width: " + Str(SurfaceDesc\dwWidth))
    WriteStringN(1, "-pitch: " + Str(SurfaceDesc\lPitch))
    
  Else
    
    WriteStringN(1, "ERROR: Can't get surface info!")
    
  EndIf

  WriteStringN(1, "--------------------------------------")
  
  FlushFileBuffers(1)
  
EndProcedure

Procedure DDrawCallback(*Object, Action.i, ObjectType.i)
  
  Select Action
    
    Case #GF_DD_ObjectCreate
      WriteStringN(1, Hex(*Object) + " " + GetDDrawObjectName(ObjectType) + " create")
      FlushFileBuffers(1)
      
      Select ObjectType
          
        Case #GF_DD_IDirectDrawSurface
          Create_IDirectDrawSurface(*Object)
          
      EndSelect

    Case #GF_DD_ObjectRelease
      WriteStringN(1, Hex(*Object) + " " + GetDDrawObjectName(ObjectType) + " release")
      FlushFileBuffers(1)
      
  EndSelect
  
EndProcedure

;This procedure is called if the plugin DLL is first loaded.
;Do any initialization stuff here if needed,
;but dont call any exported functions from the GameFixer core here.
ProcedureDLL AttachProcess(Instance.i)

EndProcedure

;This procedure is called if the plugin DLL unloads.
;Do any cleanup here, like uninstalling hooks.
ProcedureDLL DetachProcess(Instance.i)
  
  WriteStringN(1, "DirectDraw Version: " + Str(GetDDrawVersion()))
  FlushFileBuffers(1)
  
  UnInstallDDrawHook(DDrawHookId)
  CloseFile(1)
  
EndProcedure

;This procedure is called by the GameFixer core to configure the plugin.
;Config.s contains the config line for the plugin that was been specified
;by the command line parameters.
;Do anything you need here, like installing hooks.
;This procedure is called before the main thread of the game is resumed,
;so the game isnt running just yet and hasnt initialized anything.
;If you need the game in a initialized state, create a thread here and wait
;for it to initialize in the thread. The main thread of the game will not
;resume before all plugins are configured.
ProcedureDLL.i Config(Config.s)
  
  DDrawHookId = InstallDDrawHook(@DDrawCallback())
  
  If DDrawHookId = 0
    ProcedureReturn #False
  EndIf
  
  CreateFile(1, "DDraw.log")
  
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 164
; FirstLine = 100
; Folding = C-
; EnableXP
; Executable = B:\GameFixer\Plugins\DDTest.dll
; Warnings = Display