;/-------------------\
;| GameFixer Plugin  |
;| Template          |
;|                   |
;| Version 0.03      |
;| Date: 29.06.2012  |
;| PureBasic 4.60    |
;|                   |
;| by Thorium        |
;\-------------------/

EnableExplicit

;needed to use the disassembler
XIncludeFile "BeaEngine32.pbi"

;needed for the DirectDraw hooking
XIncludeFile "GameFixer DirectDraw Hook.pbi"

;import of the functionalities exported by the GameFixer core
Import "GameFixer Core.lib"
  
  ;****************
  ;present in v0.01
  ;****************
  
  ;Disassembles a x86 instruction by using the BeaEngine.
  ;Go to http://www.beaengine.org/ to read about how to use it.
  ;There is a PureBasic example there.
  Dasm.i(*Instruction.DisasmStruc)
  
  ;Installs a standard procedure hook.
  ;Returns a hook ID you need later.
  ;Returns #False if it failed.
  ;Standard means no conversion of calling conventions is done.
  ;ProcAddr specifies the memory address of the procedure you want to hook.
  ;HookProcAddr specifies the memory address of the procedure you want to redirect to.
  ;To call the original procedure later you need a new address, you get by calling GetOrigStdProcAddr.
  ;Don't ever call ProcAddr after it is hooked or you will end up in a endless recursion.
  InstallStdProcHook.i(ProcAddr.i, HookProcAddr.i)
  
  ;Uninstalls a standard procedure hook.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  UninstallStdProcHook(HookId.i)
  
  ;Returns the address you have to call if you want to call the original procedure you had hooked before.
  ;HookId is the hook ID you got from InstallStdProcHook or InstallWin32ApiHook.
  GetOrigStdProcAddr.i(HookId.i)
  
  ;Installs a hook on a Win32 API function.
  ;Returns a hook ID you need later. It's valid to use with GetOrigStdProcAddr and UninstallStdProcHook.
  ;Returns #False if it failed.
  ;It's basicly the same as InstallStdProcHook.
  ;You just have To specify the library and function name instead of the procedure address.
  InstallWin32ApiHook.i(DllName.s, ProcName.s, HookProcAddr.i)
  
  ;************
  ;new in v0.02
  ;************
  
  ;Installs a hook on DirectDraw.
  ;Returns a hook ID you need to uninstall the hook.
  ;Returns #False if it failed.
  ;*Callback is a pointer to a DDrawCallback procedure.
  ;The hook calls back on creation or release of any DirectDraw releated object.
  ;See DDrawCallback procedure for details.
  InstallDDrawHook.i(*Callback)
  
  ;Uninstalls a DirectDraw hook.
  ;DDrawHookId is the hook ID you got from InstallDDrawHook.
  UninstallDDrawHook(DDrawHookId.i)
  
  ;Installs a hook on a DirectDraw procedure.
  ;Returns a hook ID you need later. It's valid to use with GetOrigStdProcAddr and UninstallStdProcHook.
  ;Returns #False if it failed.
  ;This is not just valid for the methode of the specified object but for all objects that use this procedure.
  ;*Object is a pointer to a valid DirectDraw object. You can get the pointers from DDrawCallback.
  ;MethodeIndex is the index of the methode in the interface of the object. You can use #GF_DD_ constants here.
  ;HookProcAddr specifies the memory address of the procedure you want to redirect to.
  InstallDDrawProcHook.i(*Object, MethodeIndex.i, HookProcAddr.i)
  
  ;************
  ;new in v0.03
  ;************
  
  ;Returns the DirectDraw version the game is using.
  ;The version can only be obtained by watching the object creation,
  ;so the return value can change at a later time.
  GetDDrawVersion.i()
  
EndImport

;This procedure is called back by a DirectDraw hook.
;*Object is a pointer to a valid DirectDraw related object.
;Action tells if the object was been created or released. Uses #GF_DD_ constants.
;ObjectType tells the type of the object. Uses #GF_DD_ constants.
Procedure DDrawCallback(*Object, Action.i, ObjectType.i)
  
  Select Action
    
    Case #GF_DD_ObjectCreate
      ;A new DirectDraw releated object was been created.
      
      Select ObjectType
          
        Case #GF_DD_IDirectDraw
          ;It's a IDirectDraw object.
          
        Case #GF_DD_IDirectDrawSurface
          ;It's a IDirectDrawSurface object.

      EndSelect
      
    Case #GF_DD_ObjectRelease
      ;A object was been released.
      
      Select ObjectType
          
        Case #GF_DD_IDirectDraw
          ;It's a IDirectDraw object.
          
        Case #GF_DD_IDirectDrawSurface
          ;It's a IDirectDrawSurface object.

      EndSelect
      
  EndSelect
  
EndProcedure

;This procedure is called if the plugin DLL is first loaded.
;Do any initialization stuff here if needed,
;but dont call any exported functions from the GameFixer core here.
ProcedureDLL AttachProcess(Instance.i)

EndProcedure

;This procedure is called if the plugin DLL unloads.
;Do any cleanup here, like uninstalling hooks.
ProcedureDLL DetachProcess(Instance.i)
  
EndProcedure

;This procedure is called by the GameFixer core to configure the plugin.
;Config.s contains the config line for the plugin that was been specified
;by the command line parameters.
;Do anything you need here, like installing hooks.
;This procedure is called before the main thread of the game is resumed,
;so the game isnt running just yet and hasnt initialized anything.
;If you need the game in a initialized state, create a thread here and wait
;for it to initialize in the thread. The main thread of the game will not
;resume before all plugins are configured.
ProcedureDLL.i Config(Config.s)

  ;Return #True to tell the GameFixer core that your plugin is successfully
  ;configured.
  ;If something wend wrong return #False to tell the GameFixer core so.
  ProcedureReturn #True
  
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 5
; Folding = w
; EnableXP
; Executable = B:\GameFixer\Plugins\Test.dll
; Warnings = Display