;/---------------------------\
;| Thorium Ressource Manager |
;|                           |
;| Version 0.04              |
;| Date: 01.10.2013          |
;|                           |
;| PureBasic 5.20            |
;\---------------------------/

;callback procedure format: ProcedureName(ResIndex.i, Addr.i, Size.i, Error.i)
;Errorcodes: 1 = Could not load ressource from HDD.
;            2 = Could not decompress ressource.

;ThorReCo (Thorium Ressource Container) format structure
;------
;header
;------
;8 byte signature = ThorReCo
;1 byte format version = 2
;1 byte flags
;2 byte alignment
;2 byte compressed size of directory    (4 byte on huge file)
;2 byte uncompressed size of directory  (4 byte on huge file)
;4 byte offset of directory             (8 byte on huge file)
;1 byte compression of directory
;---------
;directory
;---------
;2 byte count of ressources             (4 byte on huge file)
;x byte ressource entries
;---------------
;ressource entry
;---------------
;1 byte lengh descriptor for ressource name
;x byte ressource name
;2 byte version of ressource
;1 byte compression
;1 byte count of language entries
;x byte language entries
;--------------
;language entry
;--------------
;1 byte language
;4 byte compressed size of ressource    (8 byte on huge file)
;4 byte uncompressed size of ressource  (8 byte on huge file)
;4 byte offset of ressource             (8 byte on huge file)
;
;------------
;header flags
;------------
;bit0 = ressource versioning
;bit1 = multilingual
;bit2 = big file (file > 2GB)
;bit3 = huge file (file > 4GB)
;bit4 = uses zlib compression
;bit5 = uses lzma compression
;bit6 = unused
;bit7 = unused
;
;-----------------------
;compression enumeration
;-----------------------
;00 = none (uncompressed)
;01 = zlib
;02 = lzma

EnableExplicit

CompilerSelect #PB_Compiler_OS
    
  CompilerCase #PB_OS_Windows
    ImportC "zlib.lib"
      compress2(*dest, *destLen, *source, sourceLen.i, level.i)
      uncompress(*dest, *destLen, *source, sourceLen.i)
    EndImport
    XIncludeFile "LZMA.pbi"

  CompilerCase #PB_OS_Linux
    ImportC #PB_Compiler_Home + "purelibraries/linux/libraries/zlib.a"
      compress2(*dest, *destLen, *source, sourceLen.i, level.i)
      uncompress(*dest, *destLen, *source, sourceLen.i)
    EndImport

  CompilerCase #PB_OS_MacOS
    ImportC "/usr/lib/libz.dylib"
      compress2(*dest, *destLen, *source, sourceLen.i, level.i)
      uncompress(*dest, *destLen, *source, sourceLen.i)
    EndImport

CompilerEndSelect

#Debug_Build = #False
  
#Rmgr_Archiv_Signature = $6F436552726F6854

#Rmgr_ArchivFlag_ResVersioning = %00000001
#Rmgr_ArchivFlag_MultiLingual  = %00000010
#Rmgr_ArchivFlag_BigFile       = %00000100
#Rmgr_ArchivFlag_HugeFile      = %00001000
#Rmgr_ArchivFlag_UsesZLib      = %00010000
#Rmgr_ArchivFlag_UsesLZMA      = %00100000

#Rmgr_Comp_None    = 0
#Rmgr_Comp_ZLib    = 1
#Rmgr_Comp_LZMA    = 2

Enumeration 1
  #Rmgr_Error_NoAccessFile              ;kein Zugriff auf Datei
  #Rmgr_Error_WrongOrDmgArchiv          ;Archiv ist kein ThReCo oder beschdigt
  #Rmgr_Error_UnsupArchivFormat         ;nicht untersttztes Archivformat
  #Rmgr_Error_UnsupArchivFormatVersion  ;nicht untersttze Version des Archivformats
  #Rmgr_Error_UnsupCompInArchiv         ;nicht untersttzte Kompression wird im Archiv genutzt
  #Rmgr_Error_ArchivIsBig               ;Archiv ist grer als 2GB und dies wird nicht untersttzt
  #Rmgr_Error_ArchivIsHuge              ;Archiv ist grer als 4GB und dies wird nicht untersttzt
  #Rmgr_Error_UnsupDirComp              ;Directory des Archivs ist mit nicht untersttzter Kompression komprimiert
  #Rmgr_Error_AllocMem                  ;Speicher konnte nicht alloziert werden
  #Rmgr_Error_ReadFile                  ;konnte Datei nicht auslesen
  #Rmgr_Error_Decomp                    ;konnte nicht dekomprimieren
  #Rmgr_Error_NoAccessDir               ;kein Zugriff auf Verzeichnis
  #Rmgr_Error_OutOfIndex                ;ausserhalb des gltigen Bereichs
  #Rmgr_Error_ResNotFound               ;Ressource konnte nicht gefunden werden
EndEnumeration

Structure Rmgr_LoadStackElement
  ResIndex.i
EndStructure

Structure Rmgr_DecompStackElement
  ResIndex.i
  Addr.i
EndStructure

Structure Rmgr_TrcHeaderSmall
  Signature.q
  FormatVersion.a
  Flags.a
  Alignment.u
  DirCompSize.u
  DirUncompSize.u
  DirOffset.l
  DirComp.a
EndStructure

Structure Rmgr_TrcHeaderHuge
  Signature.q
  FormatVersion.a
  Flags.a
  Alignment.u
  DirCompSize.l
  DirUncompSize.l
  DirOffset.q
  DirComp.a
EndStructure  

Structure Rmgr_Ressource
  Name.s
  Version.u
  Language.a
  FileHandle.i
  FileOffset.i
  Compression.a
  CompressedSize.i
  UncompressedSize.i
EndStructure

Structure Rmgr_LangEntry
  Language.a
  CompressedSize.l
  UncompressedSize.l
  Offset.l
EndStructure

Global NewList Rmgr_LoadStack.Rmgr_LoadStackElement()
Global NewList Rmgr_DecompStack.Rmgr_DecompStackElement()

Global Rmgr_LoadThreadHandle.i
Global Rmgr_LoadThreadRunning.i
Global Rmgr_DecompThreadRunningCnt.i

Global Rmgr_LoadStackMutex.i
Global Rmgr_DecompStackMutex.i

Global Rmgr_LoadedCallbackAddr.i

Global Rmgr_Path.s
Global Rmgr_Language.i

Global NewMap Rmgr_RessourcesMap.i()
Global Dim Rmgr_Ressources.Rmgr_Ressource(0)
Global Rmgr_RessourcesCnt.i

Global Rmgr_LastError.i

Global Rmgr_EnumResIndex.i

Global Rmgr_CoreCnt.i
Global Rmgr_UseCoreCnt.i

CompilerIf #Debug_Build = #True

Global LogFilePath.s
Global LogFile.i
Global WriteLogMutex.i

Procedure InitLog()

  LogFilePath = "B:\Projekte\PB Projekte\Thorium Ressource Manager\Version 0.0x\Debug.log"

  LogFile = CreateFile(#PB_Any, LogFilePath)

  If LogFile <> 0
    WriteStringN(LogFile, "Thorium Ressource Manager Log File")
    WriteStringN(LogFile, "Version: 0.03")
    WriteStringN(LogFile, "")
    FlushFileBuffers(LogFile)
  Else
    MessageRequester("", "Error: Can't create log file!", 16)
    End
  EndIf
  
  WriteLogMutex = CreateMutex()

EndProcedure

Procedure EndLog()

  WriteStringN(LogFile, "program correct terminated")
  CloseFile(LogFile)

EndProcedure

Procedure WriteLog(Text.s)

  LockMutex(WriteLogMutex)
  WriteStringN(LogFile, Text)
  FlushFileBuffers(LogFile)
  UnlockMutex(WriteLogMutex)

EndProcedure

Procedure CrashCatcher()

  WriteLog("/----------------------------------")
  WriteLog("|CRASH")
  WriteLog("|----------------------------------")
  writelog("|error code : " + Hex(ErrorCode() & $FFFFFFFF))
  WriteLog("|module     : " + ErrorFile())
  WriteLog("|line number: " + Str(ErrorLine()))
  WriteLog("|description: " + ErrorMessage())
  WriteLog("\----------------------------------")

  MessageRequester("", "A crash is ocurred. Please take a look at the logfile: " + LogFilePath, 16)

  ExitProcess_(0)

EndProcedure

Procedure DebuggerInit()

  InitLog()
  OnErrorCall(@CrashCatcher())

EndProcedure

CompilerEndIf

;decompresses a memory buffer that is compressed with ZLib
Procedure.i Rmgr_DecompZlib(*Source, *Destination, CompSize.i, UncompSize.i)
  
  Protected UncompressedSize.i
  
  UncompressedSize = UncompSize
  
  If uncompress(*Destination, @UncompressedSize, *Source, CompSize)
    ProcedureReturn #False
  EndIf
  
  If UncompressedSize <> UncompSize
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

;decompresses a memory buffer that is compressed with LZMA
Procedure.i Rmgr_DecompLzma(*Source, *Destination, CompSize.i, UncompSize.i)
  
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x86 Or #PB_Compiler_Processor = #PB_Processor_x64

    Protected SourceSize.i
    Protected *Props
    Protected UncompressedSize.i
    
    UncompressedSize = UncompSize
    
    SourceSize = CompSize - SizeOf(T_LZMA_HEADER)
    *Props = AllocateMemory(#LZMA_PROPS_SIZE)
    If *Props = 0
      ProcedureReturn #False
    EndIf
    
    CopyMemory(*Source, *Props, #LZMA_PROPS_SIZE)
    
    If LzmaUncompress(*Destination, @UncompressedSize, *Source + SizeOf(T_LZMA_HEADER), @SourceSize, *Props, #LZMA_PROPS_SIZE)  
      FreeMemory(*Props)
      ProcedureReturn #False
    EndIf
  
    FreeMemory(*Props)
    
    ProcedureReturn #True
    
  CompilerElse
    
    ProcedureReturn #False
    
  CompilerEndIf
  
EndProcedure

;enumerates the indexed ressources
;call it until it returns #False
;returns length of ressource name in bytes
ProcedureDLL.i Rmgr_EnumRes(*ResName)
  
  If Rmgr_EnumResIndex = Rmgr_RessourcesCnt
    Rmgr_EnumResIndex = 0
    ProcedureReturn #False
  EndIf
  
  Rmgr_EnumResIndex + 1
  PokeS(*ResName, Rmgr_Ressources(Rmgr_EnumResIndex)\Name)
  
  ProcedureReturn Len(Rmgr_Ressources(Rmgr_EnumResIndex)\Name)
  
EndProcedure

;clears the ressource index
ProcedureDLL Rmgr_ClearIndex()
  
  Protected i.i
  
  For i = 1 To Rmgr_RessourcesCnt
    If IsFile(Rmgr_Ressources(i)\FileHandle) <> #False
      CloseFile(Rmgr_Ressources(i)\FileHandle)
    EndIf
  Next
  
  ReDim Rmgr_Ressources.Rmgr_Ressource(0)
  Rmgr_RessourcesCnt = 0
  ClearMap(Rmgr_RessourcesMap())  
  
EndProcedure

;return value is the ressource index of the requested ressource
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_FindRes(ResName.s)

  ProcedureReturn Rmgr_RessourcesMap(ResName)

EndProcedure

;return value is the ressource index of the requested ressource
;if procedure fails, it returns 0
Procedure.i Rmgr_FindResArray(ResName.s)

  Protected i.i
  Protected Index.i
  
  For i = 1 To Rmgr_RessourcesCnt
    If ResName = Rmgr_Ressources(i)\Name
      Index = i
      Break
    EndIf
  Next

  ProcedureReturn Index

EndProcedure

;indexes all ressources stored in the archive
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_IndexArchiv(FileName.s)

  Protected File.i
  Protected ResCnt
  Protected i.i
  Protected i2.i
  Protected ResIndex.i
  Protected ResVersioning.i
  Protected MultiLingual.i
  Protected DirCompAddr.i
  Protected DirAddr.i
  Protected Pos.i
  Protected ResNameLen.i
  Protected ResVer.i
  Protected ResLang.i
  Protected ResComp.i
  Protected ResLangCnt.i
  Protected LangIndex.i
  Protected ResName.s
  
  Protected Dim LangEntries.Rmgr_LangEntry(0)
  Protected *TrcHeaderSmall.Rmgr_TrcHeaderSmall
  Protected *TrcHeaderHuge.Rmgr_TrcHeaderHuge
  
  CompilerIf #Debug_Build = #True
    WriteLog("indexing archiv...")
    WriteLog("file name: " + FileName)
  CompilerEndIf
  
  File = ReadFile(#PB_Any, FileName)
  
  If File = 0
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not access file")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_NoAccessFile
    ProcedureReturn #False
  EndIf
  
  If Lof(File) < SizeOf(Rmgr_TrcHeaderSmall)
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: wrong or damaged file format")
    CompilerEndIf
    CloseFile(File)
    Rmgr_LastError = #Rmgr_Error_WrongOrDmgArchiv
    ProcedureReturn #False
  EndIf
  
  *TrcHeaderSmall = AllocateMemory(SizeOf(Rmgr_TrcHeaderHuge))
  If *TrcHeaderSmall = 0
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not allocate memory")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_AllocMem
    CloseFile(File)
    ProcedureReturn #False
  EndIf
  
  ReadData(File, *TrcHeaderSmall, SizeOf(Rmgr_TrcHeaderSmall))

  If *TrcHeaderSmall\Signature <> #Rmgr_Archiv_Signature
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: wrong file signature")
    CompilerEndIf
    CloseFile(File)
    FreeMemory(*TrcHeaderSmall)
    Rmgr_LastError = #Rmgr_Error_UnsupArchivFormat
    ProcedureReturn #False
  EndIf

  CompilerIf #Debug_Build = #True
    WriteLog("file format version: " + Str(*TrcHeaderSmall\FormatVersion))
  CompilerEndIf
  
  If *TrcHeaderSmall\FormatVersion <> 2
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: unsupported file format version")
    CompilerEndIf
    CloseFile(File)
    FreeMemory(*TrcHeaderSmall)
    Rmgr_LastError = #Rmgr_Error_UnsupArchivFormatVersion
    ProcedureReturn #False
  EndIf
  
  CompilerIf #Debug_Build = #True
    Protected Text.s
    
    Text = "used compressions:"
    If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_UsesZLib = #Rmgr_ArchivFlag_UsesZLib
      Text = Text + " ZLib"
    EndIf
    If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_UsesLZMA = #Rmgr_ArchivFlag_UsesLZMA
      Text = Text + " LZMA"
    EndIf
    WriteLog(Text)
  CompilerEndIf
  
  If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_ResVersioning = #Rmgr_ArchivFlag_ResVersioning
    ResVersioning = #True
    CompilerIf #Debug_Build = #True
      WriteLog("archiv uses ressource versioning")
    CompilerEndIf
  EndIf
  
  If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_Multilingual = #Rmgr_ArchivFlag_Multilingual
    MultiLingual = 1
    CompilerIf #Debug_Build = #True
      WriteLog("archiv is multilingual")
    CompilerEndIf
  EndIf

  If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_BigFile = #Rmgr_ArchivFlag_BigFile
    CompilerIf #Debug_Build = #True
      WriteLog("archiv is a big file (>2GB)")
      WriteLog("ERROR: unsupported archiv size")
    CompilerEndIf
    CloseFile(File)
    FreeMemory(*TrcHeaderSmall)
    Rmgr_LastError = #Rmgr_Error_ArchivIsBig
    ProcedureReturn #False
  EndIf

  If *TrcHeaderSmall\Flags & #Rmgr_ArchivFlag_HugeFile = #Rmgr_ArchivFlag_HugeFile
    CompilerIf #Debug_Build = #True
      WriteLog("archiv is a huge file (>4GB)")
      WriteLog("ERROR: unsupported archiv size")
    CompilerEndIf
    CloseFile(File)
    FreeMemory(*TrcHeaderSmall)
    Rmgr_LastError = #Rmgr_Error_ArchivIsHuge
    ProcedureReturn #False
  EndIf
  
  CompilerIf #Debug_Build = #True
    WriteLog("ressource alignment: " + Str(*TrcHeaderSmall\Alignment))
  CompilerEndIf
  
  CompilerIf #Debug_Build = #True
    Select *TrcHeaderSmall\DirComp
      Case #Rmgr_Comp_None
        WriteLog("directory is not compressed")
      Case #Rmgr_Comp_ZLib
        WriteLog("directory is compressed with ZLib")
      Case #Rmgr_Comp_LZMA
        WriteLog("directory is compressed with LZMA")
    EndSelect
  CompilerEndIf

  CompilerIf #Debug_Build = #True
    WriteLog("compressed size of directory: " + Str(*TrcHeaderSmall\DirCompSize))
    WriteLog("uncompressed size of directory: " + Str(*TrcHeaderSmall\DirUncompSize))
  CompilerEndIf

  CompilerIf #Debug_Build = #True
    WriteLog("offset of directory: " + Str(*TrcHeaderSmall\DirOffset))
  CompilerEndIf

  DirAddr = AllocateMemory(*TrcHeaderSmall\DirUncompSize)
  If DirAddr = 0
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not allocate memory")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_AllocMem
    CloseFile(File)
    FreeMemory(*TrcHeaderSmall)
    ProcedureReturn #False
  EndIf

  If *TrcHeaderSmall\DirComp <> #Rmgr_Comp_None
  
    CompilerIf #Debug_Build = #True
      WriteLog("decompressing directory...")
      WriteLog("reading compressed directory from archiv...")
    CompilerEndIf
    
    DirCompAddr = AllocateMemory(*TrcHeaderSmall\DirCompSize)
    If DirCompAddr = 0
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not allocate memory")
      CompilerEndIf
      FreeMemory(DirAddr)
      Rmgr_LastError = #Rmgr_Error_AllocMem
      CloseFile(File)
      FreeMemory(*TrcHeaderSmall)
      ProcedureReturn #False
    EndIf
    
    FileSeek(File, *TrcHeaderSmall\DirOffset)
    If ReadData(File, DirCompAddr, *TrcHeaderSmall\DirCompSize) <> *TrcHeaderSmall\DirCompSize
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not read data from file")
      CompilerEndIf
      FreeMemory(DirAddr)
      FreeMemory(DirCompAddr)
      FreeMemory(*TrcHeaderSmall)
      CloseFile(File)
      Rmgr_LastError = #Rmgr_Error_ReadFile
      ProcedureReturn #False
    EndIf
    
    CompilerIf #Debug_Build = #True
      WriteLog("decompressing memory...")
    CompilerEndIf
    
    Select *TrcHeaderSmall\DirComp
        
      Case #Rmgr_Comp_ZLib
        If Rmgr_DecompZlib(DirCompAddr, DirAddr, *TrcHeaderSmall\DirCompSize, *TrcHeaderSmall\DirUncompSize) = #False
          CompilerIf #Debug_Build = #True
            WriteLog("ERROR: could not decompress memory")
          CompilerEndIf
          FreeMemory(DirAddr)
          FreeMemory(DirCompAddr)
          FreeMemory(*TrcHeaderSmall)
          CloseFile(File)
          Rmgr_LastError = #Rmgr_Error_Decomp
          ProcedureReturn #False
        EndIf
        
      Case #Rmgr_Comp_LZMA
        If Rmgr_DecompLzma(DirCompAddr, DirAddr, *TrcHeaderSmall\DirCompSize, *TrcHeaderSmall\DirUncompSize) = #False
          CompilerIf #Debug_Build = #True
            WriteLog("ERROR: could not decompress memory")
          CompilerEndIf
          FreeMemory(DirAddr)
          FreeMemory(DirCompAddr)
          FreeMemory(*TrcHeaderSmall)
          CloseFile(File)
          Rmgr_LastError = #Rmgr_Error_Decomp
          ProcedureReturn #False
        EndIf
        
      Default
          CompilerIf #Debug_Build = #True
            WriteLog("ERROR: unknown compression on directory")
          CompilerEndIf
          FreeMemory(DirAddr)
          FreeMemory(DirCompAddr)
          FreeMemory(*TrcHeaderSmall)
          CloseFile(File)
          Rmgr_LastError = #Rmgr_Error_UnsupDirComp
          ProcedureReturn #False
          
    EndSelect
    
    FreeMemory(DirCompAddr)
    
    CompilerIf #Debug_Build = #True
      WriteLog("decompression of directory done")
    CompilerEndIf

  Else
  
    CompilerIf #Debug_Build = #True
      WriteLog("reading uncompressed directory from archiv...")
    CompilerEndIf
  
    FileSeek(File, *TrcHeaderSmall\DirOffset)
    If ReadData(File, DirAddr, *TrcHeaderSmall\DirUncompSize) <> *TrcHeaderSmall\DirUncompSize
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not read data from file")
      CompilerEndIf
      FreeMemory(DirAddr)
      FreeMemory(*TrcHeaderSmall)
      CloseFile(File)
      Rmgr_LastError = #Rmgr_Error_ReadFile
      ProcedureReturn #False
    EndIf

  EndIf
  
  CompilerIf #Debug_Build = #True
    WriteLog("indexing ressources of directory...")
  CompilerEndIf

  Pos = DirAddr
  ResCnt = PeekU(Pos)
  Pos + 2

  CompilerIf #Debug_Build = #True
    WriteLog("count of ressources in archiv: " + Str(ResCnt))
  CompilerEndIf

  For i = 1 To ResCnt
  
    CompilerIf #Debug_Build = #True
      WriteLog("indexing ressource " + Str(i) + "...")
    CompilerEndIf
    
    ResIndex = 0

    ResNameLen = PeekA(Pos)
    Pos + 1
    ResName = Space(ResNameLen)
    CopyMemory(Pos, @ResName, ResNameLen)
    Pos + ResNameLen
    ResName = ResName
    
    CompilerIf #Debug_Build = #True
      WriteLog("ressource name: " + ResName)
    CompilerEndIf

    ResVer = PeekU(Pos)
    Pos + 2
    
    CompilerIf #Debug_Build = #True
      WriteLog("ressource version: " + Str(ResVer))
    CompilerEndIf

    ResComp = PeekA(Pos)
    Pos + 1

    ResLangCnt = PeekA(Pos)
    Pos + 1
    
    CompilerIf #Debug_Build = #True
      WriteLog("count of language versions: " + Str(ResLangCnt))
    CompilerEndIf

    ReDim LangEntries.Rmgr_LangEntry(ResLangCnt)
    
    For i2 = 1 To ResLangCnt
      CopyMemory(Pos, @LangEntries(i2)\Language, SizeOf(Rmgr_LangEntry))
      Pos + SizeOf(Rmgr_LangEntry)
    Next

    ResIndex = Rmgr_FindResArray(ResName)
    LangIndex = 0
    
    If ResIndex <> 0
      
      CompilerIf #Debug_Build = #True
        WriteLog("ressource allready indexed")
      CompilerEndIf
      
      If ResVer > Rmgr_Ressources(ResIndex)\Version
      
        CompilerIf #Debug_Build = #True
          WriteLog("but is a newer version")
        CompilerEndIf
        
        For i2 = 1 To ResLangCnt
          If Rmgr_Language = LangEntries(i2)\Language
            LangIndex = i2
            CompilerIf #Debug_Build = #True
              WriteLog("matching language version found")
            CompilerEndIf
          EndIf
        Next

        If LangIndex = 0
          If Rmgr_Language <> Rmgr_Ressources(ResIndex)\Language
            LangIndex = 1
          CompilerIf #Debug_Build = #True
            WriteLog("matching language version not found, but allready indexed ressource is also not matching global language")
          CompilerEndIf
          EndIf
        EndIf
      
      Else
        
        If Rmgr_Language <> Rmgr_Ressources(ResIndex)\Language
          
          CompilerIf #Debug_Build = #True
            WriteLog("but allready indexed ressource is also not matching global language")
          CompilerEndIf
        
          For i2 = 1 To ResLangCnt
            If Rmgr_Language = LangEntries(i2)\Language
              LangIndex = i2
              CompilerIf #Debug_Build = #True
                WriteLog("matching language version found")
              CompilerEndIf
            EndIf
          Next
          
        EndIf
      
      EndIf
      
      If LangIndex = 0
        ResIndex = 0
        CompilerIf #Debug_Build = #True
          WriteLog("ressource will not be updated")
        CompilerEndIf
      Else
        CompilerIf #Debug_Build = #True
          WriteLog("ressource will be updated")
        CompilerEndIf
      EndIf
      
    Else
    
      For i2 = 1 To ResLangCnt
        If Rmgr_Language = LangEntries(i2)\Language
          LangIndex = i2
          CompilerIf #Debug_Build = #True
            WriteLog("matching language version found")
          CompilerEndIf
        EndIf
      Next
    
      If LangIndex = 0
        CompilerIf #Debug_Build = #True
          WriteLog("matching language version not found")
        CompilerEndIf
        LangIndex = 1
      EndIf

    EndIf
    
    If LangIndex <> 0
    
      If ResIndex = 0
        Rmgr_RessourcesCnt + 1
        ReDim Rmgr_Ressources.Rmgr_Ressource(Rmgr_RessourcesCnt)
        ResIndex = Rmgr_RessourcesCnt
        Rmgr_RessourcesMap(ResName) = ResIndex
      EndIf
    
      Rmgr_Ressources(ResIndex)\Name = ResName
      Rmgr_Ressources(ResIndex)\Version = ResVer
      Rmgr_Ressources(ResIndex)\Language = LangEntries(LangIndex)\Language
      Rmgr_Ressources(ResIndex)\FileHandle = File
      Rmgr_Ressources(ResIndex)\FileOffset = LangEntries(LangIndex)\Offset
      Rmgr_Ressources(ResIndex)\CompressedSize = LangEntries(LangIndex)\CompressedSize
      Rmgr_Ressources(ResIndex)\UncompressedSize = LangEntries(LangIndex)\UncompressedSize
      Rmgr_Ressources(ResIndex)\Compression = ResComp
      
      CompilerIf #Debug_Build = #True
        
        Select Rmgr_Ressources(ResIndex)\Compression
          Case #Rmgr_Comp_None
            WriteLog("compression: none (uncompressed)")
          Case #Rmgr_Comp_ZLib
            WriteLog("compression: ZLib")
          Case #Rmgr_Comp_LZMA
            WriteLog("compression: LZMA")
          Default
            WriteLog("ERROR: unknown compression")
        EndSelect
  
        WriteLog("language: " + Str(Rmgr_Ressources(ResIndex)\Language))
        WriteLog("file offset: 0x" + Hex(Rmgr_Ressources(ResIndex)\FileOffset))
        WriteLog("compressed size: " + Str(Rmgr_Ressources(ResIndex)\CompressedSize) + "byte")
        WriteLog("uncompressed size: " + Str(Rmgr_Ressources(ResIndex)\UncompressedSize) + "byte")
        WriteLog("ressource indexed as " + Str(ResIndex))
        
      CompilerEndIf
    
    EndIf

  Next
  
  FreeMemory(*TrcHeaderSmall)
  
  CompilerIf #Debug_Build = #True
    WriteLog("indexing archiv done")
  CompilerEndIf
  
  ProcedureReturn #True

EndProcedure

;indexes all ressources stored in archives or files for fast access
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_IndexRes()

  Protected Dir.i
  Protected ResIndex.i
  Protected FileName.s
  
  CompilerIf #Debug_Build = #True
    WriteLog("indexing ressources...")
  CompilerEndIf
  
  Rmgr_ClearIndex()
  
  ;indexing files in archives
  
  Dir = ExamineDirectory(#PB_Any, Rmgr_Path, "*.trc")
  
  If Dir = 0
    
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not access ressource directory")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_NoAccessDir
    
    ProcedureReturn #False
  
  EndIf
  
  While NextDirectoryEntry(Dir) <> 0
  
    If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
    
      If Rmgr_IndexArchiv(Rmgr_Path + DirectoryEntryName(Dir)) = 0
        CompilerIf #Debug_Build = #True
          WriteLog("indexing of ressources aborted")
        CompilerEndIf
        ProcedureReturn #False
      EndIf
    
    EndIf

  Wend
  
  FinishDirectory(Dir)

  ;indexing single files

  CompilerIf #Debug_Build = #True
    WriteLog("indexing ressources from single files...")
  CompilerEndIf

  Dir = ExamineDirectory(#PB_Any, Rmgr_Path, "*")

  If Dir = 0
    
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not access ressource directory")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_NoAccessDir
    
    ProcedureReturn #False
  
  EndIf
  
  While NextDirectoryEntry(Dir) <> 0
  
    If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
    
      FileName = LCase(DirectoryEntryName(Dir))
      If Right(FileName, 4) <> ".trc"
        
        ResIndex = 0
        FileName = FileName
        
        CompilerIf #Debug_Build = #True
          WriteLog("indexing ressource " + FileName + "...")
        CompilerEndIf
        
        ResIndex = Rmgr_FindResArray(FileName)
  
        CompilerIf #Debug_Build = #True
          If ResIndex <> 0
            WriteLog("ressource allready indexed, will be updated")
          EndIf
        CompilerEndIf
        
        If ResIndex = 0
          Rmgr_RessourcesCnt + 1
          ReDim Rmgr_Ressources.Rmgr_Ressource(Rmgr_RessourcesCnt)
          ResIndex = Rmgr_RessourcesCnt
          Rmgr_RessourcesMap(FileName) = ResIndex
        EndIf
        
        Rmgr_Ressources(ResIndex)\Name = FileName
        Rmgr_Ressources(ResIndex)\Compression = #Rmgr_Comp_None
        Rmgr_Ressources(ResIndex)\FileOffset = 0
        Rmgr_Ressources(ResIndex)\CompressedSize = FileSize(Rmgr_Path + FileName)
        Rmgr_Ressources(ResIndex)\UncompressedSize = Rmgr_Ressources(ResIndex)\CompressedSize
        Rmgr_Ressources(ResIndex)\FileHandle = 0

        CompilerIf #Debug_Build = #True
          WriteLog("size: " + Str(Rmgr_Ressources(ResIndex)\CompressedSize) + "byte")
          WriteLog("ressource indexed as " + Str(ResIndex))
        CompilerEndIf
    
      EndIf
    
    EndIf

  Wend
  
  FinishDirectory(Dir)

  ProcedureReturn #True

EndProcedure

;loads a ressource from HDD
;return value is the address of the ressource in memory
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_LoadResByIndex(Index.i)

  Protected ResAddr.i
  Protected BytesRead.i
  Protected File.i

  CompilerIf #Debug_Build = #True
    WriteLog("loading ressource from HDD...")
    WriteLog("ressource index: " + Str(Index))
  CompilerEndIf
  
  If Index > Rmgr_RessourcesCnt
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: ressource index is higher than count of indexed ressources")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_OutOfIndex
    
    ProcedureReturn #False
  EndIf
  
  CompilerIf #Debug_Build = #True
    WriteLog("ressource name: " + Rmgr_Ressources(Index)\Name)
  CompilerEndIf
  
  If Rmgr_Ressources(Index)\FileHandle <> 0
  
    CompilerIf #Debug_Build = #True
      WriteLog("loading ressource from archiv...")
    CompilerEndIf
  
    ResAddr = AllocateMemory(Rmgr_Ressources(Index)\CompressedSize)
    
    If ResAddr = 0
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not allocate memory")
      CompilerEndIf
      Rmgr_LastError = #Rmgr_Error_AllocMem
      ProcedureReturn #False
    EndIf
    
    FileSeek(Rmgr_Ressources(Index)\FileHandle, Rmgr_Ressources(Index)\FileOffset)
    BytesRead = ReadData(Rmgr_Ressources(Index)\FileHandle, ResAddr, Rmgr_Ressources(Index)\CompressedSize)
    
    If BytesRead <> Rmgr_Ressources(Index)\CompressedSize
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not read data from file")
      CompilerEndIf
      FreeMemory(ResAddr)
      Rmgr_LastError = #Rmgr_Error_ReadFile
      ProcedureReturn #False
    EndIf
    
    CompilerIf #Debug_Build = #True
      WriteLog("loading of ressource from HDD done")
    CompilerEndIf
    
    ProcedureReturn ResAddr

  Else
    
    CompilerIf #Debug_Build = #True
      WriteLog("loading ressource from single file...")
    CompilerEndIf
    
    ResAddr = AllocateMemory(Rmgr_Ressources(Index)\CompressedSize)
    
    If ResAddr = 0
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not allocate memory")
      CompilerEndIf
      Rmgr_LastError = #Rmgr_Error_AllocMem
      ProcedureReturn #False
    EndIf
    
    File = ReadFile(#PB_Any, Rmgr_Path + Rmgr_Ressources(Index)\Name)
    
    If File = 0
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not access file")
      CompilerEndIf
      FreeMemory(ResAddr)
      Rmgr_LastError = #Rmgr_Error_NoAccessFile
      ProcedureReturn #False
    EndIf
    
    BytesRead = ReadData(File, ResAddr, Rmgr_Ressources(Index)\CompressedSize)
    
    CloseFile(File)
    
    If BytesRead <> Rmgr_Ressources(Index)\CompressedSize
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: could not read data from file")
      CompilerEndIf
      FreeMemory(ResAddr)
      Rmgr_LastError = #Rmgr_Error_ReadFile
      ProcedureReturn #False
    EndIf
    
    CompilerIf #Debug_Build = #True
      WriteLog("loading of ressource from HDD done")
    CompilerEndIf
  
    ProcedureReturn ResAddr
  
  EndIf

EndProcedure

;decompresses a ressource in memory
;return value is the address of the decompressed ressource in memory
;if procedure fails, it returns 0
Procedure.i Rmgr_DecompRes(CompResAddr.i, CompSize.i, DecompSize.i, Compression.i)

  Protected DecompAddr.i
  
  If Compression = #Rmgr_Comp_None
    CompilerIf #Debug_Build = #True
      WriteLog("type of compression: none (uncompressed)")
    CompilerEndIf
  
    ProcedureReturn CompResAddr
  EndIf

  CompilerIf #Debug_Build = #True
    WriteLog("allocating memory for decompressed ressource...")
  CompilerEndIf

  DecompAddr = AllocateMemory(DecompSize)
  If DecompAddr = 0
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: could not allocate memory")
      WriteLog("decompression of ressource aborted")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_AllocMem
    ProcedureReturn #False
  EndIf

  CompilerIf #Debug_Build = #True
    Select Compression
      Case #Rmgr_Comp_ZLib
        WriteLog("type of compression: ZLib")
      Case #Rmgr_Comp_LZMA
        WriteLog("type of compression: LZMA")
      Default
        CompilerIf #Debug_Build = #True
          WriteLog("type of compression: unknown")
          WriteLog("ERROR: unknown compression")
        CompilerEndIf
    EndSelect
  CompilerEndIf

  Select Compression
      
    Case #Rmgr_Comp_ZLib
      If Rmgr_DecompZlib(CompResAddr, DecompAddr, CompSize.i, DecompSize.i) = #False
        CompilerIf #Debug_Build = #True
          WriteLog("ERROR: could not decompress memory")
          WriteLog("decompression of ressource aborted")
        CompilerEndIf
        Rmgr_LastError = #Rmgr_Error_Decomp
        ProcedureReturn #False
      EndIf
      
    Case #Rmgr_Comp_LZMA
      If Rmgr_DecompLzma(CompResAddr, DecompAddr, CompSize.i, DecompSize.i) = #False
        CompilerIf #Debug_Build = #True
          WriteLog("ERROR: could not decompress memory")
          WriteLog("decompression of ressource aborted")
        CompilerEndIf
        Rmgr_LastError = #Rmgr_Error_Decomp
        ProcedureReturn #False
      EndIf
      
    Default
      CompilerIf #Debug_Build = #True
        WriteLog("ERROR: unknown compression")
      CompilerEndIf
      Rmgr_LastError = #Rmgr_Error_Decomp
      ProcedureReturn #False
      
  EndSelect
  
  FreeMemory(CompResAddr)

  CompilerIf #Debug_Build = #True
    WriteLog("decompression of memory done")
  CompilerEndIf

  ProcedureReturn DecompAddr

EndProcedure

;This is the thread that manages the asynchronous decompression of the ressources.
Procedure Rmgr_DecompResThread(Dummy.i)

  Protected ResIndex.i
  Protected CompAddr.i
  Protected DecompAddr.i
  Protected Error.i
  Protected ThreadEnd.i

  Repeat

    LockMutex(Rmgr_DecompStackMutex)
    
    If FirstElement(Rmgr_DecompStack()) <> 0

      CompilerIf #Debug_Build = #True
        WriteLog("new ressource found on stack by decompression thread")
        WriteLog("decompressing ressource...")
        WriteLog("popping ressource from stack...")
      CompilerEndIf

      ResIndex = Rmgr_DecompStack()\ResIndex
      CompAddr = Rmgr_DecompStack()\Addr
      DeleteElement(Rmgr_DecompStack())
      
      UnlockMutex(Rmgr_DecompStackMutex)

      CompilerIf #Debug_Build = #True
        WriteLog("ressource index: " + Str(ResIndex))
      CompilerEndIf

      DecompAddr = Rmgr_DecompRes(CompAddr, Rmgr_Ressources(ResIndex)\CompressedSize, Rmgr_Ressources(ResIndex)\UncompressedSize, Rmgr_Ressources(ResIndex)\Compression)
      If DecompAddr = 0
        Error = 2
      EndIf
      
      CompilerIf #Debug_Build = #True
        WriteLog("executing callback...")
      CompilerEndIf
      
      CallFunctionFast(Rmgr_LoadedCallbackAddr, ResIndex, DecompAddr, Rmgr_Ressources(ResIndex)\UncompressedSize, Error)
      
      CompilerIf #Debug_Build = #True
        WriteLog("decompression of ressource done")
      CompilerEndIf

    Else
      
      Rmgr_DecompThreadRunningCnt = Rmgr_DecompThreadRunningCnt - 1
      UnlockMutex(Rmgr_DecompStackMutex)
      ThreadEnd = #True
      
    EndIf
  
  Until ThreadEnd = #True
  
EndProcedure

;This is the thread that manages the asynchronous loading of ressources from the HDD.
Procedure Rmgr_LoadResThread(Dummy.i)

  Protected ResIndex.i
  Protected RetVal.i
  Protected ThreadEnd.i

  Repeat

    LockMutex(Rmgr_LoadStackMutex)

    If FirstElement(Rmgr_LoadStack()) <> 0
      
      CompilerIf #Debug_Build = #True
        WriteLog("new ressource found on stack by loading thread")
        WriteLog("loading the ressource...")
        WriteLog("popping ressource from stack...")
      CompilerEndIf
      
      ResIndex = Rmgr_LoadStack()\ResIndex
      DeleteElement(Rmgr_LoadStack())
      
      UnlockMutex(Rmgr_LoadStackMutex)

      CompilerIf #Debug_Build = #True
        WriteLog("ressource index: " + Str(ResIndex))
      CompilerEndIf
      
      RetVal = Rmgr_LoadResByIndex(ResIndex)
      
      If RetVal = 0
        CallFunctionFast(Rmgr_LoadedCallbackAddr, ResIndex, 0, 0, 1)
        
        CompilerIf #Debug_Build = #True
          WriteLog("loading of ressource aborted")
        CompilerEndIf
        
      Else
        
        CompilerIf #Debug_Build = #True
          WriteLog("pushing ressource to decompression stack...")
        CompilerEndIf
        
        LockMutex(Rmgr_DecompStackMutex)
        
          LastElement(Rmgr_DecompStack())
          AddElement(Rmgr_DecompStack())
          Rmgr_DecompStack()\ResIndex = ResIndex
          Rmgr_DecompStack()\Addr = RetVal
          
          
          If Rmgr_DecompThreadRunningCnt < Rmgr_UseCoreCnt
            Rmgr_DecompThreadRunningCnt = Rmgr_DecompThreadRunningCnt + 1
            CreateThread(@Rmgr_DecompResThread(), 0)
          EndIf
        
        UnlockMutex(Rmgr_DecompStackMutex)
        
      EndIf
      
      CompilerIf #Debug_Build = #True
        WriteLog("loading of ressource done")
      CompilerEndIf

    Else
    
      Rmgr_LoadThreadRunning = #False
      UnlockMutex(Rmgr_LoadStackMutex)
      ThreadEnd = #True
      
    EndIf

  Until ThreadEnd = #True

EndProcedure

;initializes the ressource manager
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_InitManager(RessourcePath.s, Language.i, CallbackAddr.i, UseCoreCount.i)

  Protected hThread.i

  CompilerIf #Debug_Build = #True
    DebuggerInit()
    WriteLog("initializing ressource manager...")
  CompilerEndIf

  Rmgr_Language = Language

  Rmgr_LoadStackMutex = CreateMutex()
  Rmgr_DecompStackMutex = CreateMutex()
  
  Rmgr_LoadedCallbackAddr = CallbackAddr
  
  Rmgr_CoreCnt = CountCPUs(#PB_System_ProcessCPUs)
  
  If UseCoreCount = 0
    Rmgr_UseCoreCnt = Rmgr_CoreCnt
  Else
    Rmgr_UseCoreCnt = UseCoreCount
  EndIf
  
  If RessourcePath <> ""
  
    If Right(RessourcePath, 1) <> "\"
      Rmgr_Path = RessourcePath + "\"
    Else
      Rmgr_Path = RessourcePath
    EndIf
    
    CompilerIf #Debug_Build = #True
      WriteLog("ressource path: " + Rmgr_Path)
    CompilerEndIf
    
    If Rmgr_IndexRes() = #False
      CompilerIf #Debug_Build = #True
        WriteLog("initialization of ressource manager aborted")
      CompilerEndIf
      ProcedureReturn #False
    EndIf
    
  EndIf

  CompilerIf #Debug_Build = #True
    WriteLog("ressource manager initialization done")
  CompilerEndIf

  ProcedureReturn #True

EndProcedure

;loads a ressource parallel
;return value is the ressource index of the requested ressource
;the ressource index will be passed to the callback procedure
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_LoadRes(ResName.s)

  Protected ResIndex.i
  
  CompilerIf #Debug_Build = #True
    WriteLog("initilizing asynchronously loading of ressource: " + ResName)
  CompilerEndIf
  
  ResIndex = Rmgr_RessourcesMap(ResName)
  If ResIndex = 0
  
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: ressource not found")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_ResNotFound
  
    ProcedureReturn #False
  EndIf

  CompilerIf #Debug_Build = #True
    WriteLog("ressource index: " + Str(ResIndex))
    WriteLog("pushing ressource to loading stack...")
  CompilerEndIf

  LockMutex(Rmgr_LoadStackMutex)
  
    LastElement(Rmgr_LoadStack())
    AddElement(Rmgr_LoadStack())
    Rmgr_LoadStack()\ResIndex = ResIndex 
  
    If Rmgr_LoadThreadRunning = #False
      Rmgr_LoadThreadRunning = #True
      Rmgr_LoadThreadHandle = CreateThread(@Rmgr_LoadResThread(), 0)  
    EndIf
  
  UnlockMutex(Rmgr_LoadStackMutex)

  CompilerIf #Debug_Build = #True
    WriteLog("initilizing of asynchronously loading of ressource done")
  CompilerEndIf

  ProcedureReturn ResIndex

EndProcedure

;loads a ressource sequential
ProcedureDLL.i Rmgr_LoadResSeq(ResName.s, *Ressource.Integer, *Size.Integer)
  
  Protected ResIndex.i
  Protected CompAddr.i
  Protected DecompAddr.i
  
  CompilerIf #Debug_Build = #True
    WriteLog("loading ressource sequential: " + ResName)
  CompilerEndIf
  
  ResIndex = Rmgr_RessourcesMap(ResName)
  If ResIndex = 0
  
    CompilerIf #Debug_Build = #True
      WriteLog("ERROR: ressource not found")
    CompilerEndIf
    Rmgr_LastError = #Rmgr_Error_ResNotFound
  
    ProcedureReturn #False
  EndIf

  CompilerIf #Debug_Build = #True
    WriteLog("ressource index: " + Str(ResIndex))
  CompilerEndIf

  CompAddr = Rmgr_LoadResByIndex(ResIndex)
  
  If CompAddr = 0
    ProcedureReturn #False
  EndIf
  
  DecompAddr = Rmgr_DecompRes(CompAddr, Rmgr_Ressources(ResIndex)\CompressedSize, Rmgr_Ressources(ResIndex)\UncompressedSize, Rmgr_Ressources(ResIndex)\Compression)
  
  If DecompAddr = 0
    ProcedureReturn #False
  EndIf
  
  *Ressource\i = DecompAddr
  *Size\i = Rmgr_Ressources(ResIndex)\UncompressedSize

  CompilerIf #Debug_Build = #True
    WriteLog("sequential loading of ressource done")
  CompilerEndIf

  ProcedureReturn ResIndex

EndProcedure

;frees the memory of a loaded ressource
ProcedureDLL Rmgr_FreeRes(*Res)
  
  FreeMemory(*Res)
  
EndProcedure

;changes the language of ressources, needs to reindex all ressources
;if procedure fails, it returns 0
ProcedureDLL.i Rmgr_ChangeLang(Language.i)

  CompilerIf #Debug_Build = #True
    WriteLog("changing global language to " + Str(Language) + "...")
  CompilerEndIf

  Rmgr_Language = Language

  If Rmgr_IndexRes() = 0
    ProcedureReturn #False
  EndIf
  
  CompilerIf #Debug_Build = #True
    WriteLog("changing global language done")
  CompilerEndIf
  
  ProcedureReturn #True

EndProcedure

;returns the code of the last occurred error
ProcedureDLL.i Rmgr_GetLastError()

  ProcedureReturn Rmgr_LastError

EndProcedure

; IDE Options = PureBasic 5.20 LTS (Windows - x64)
; ExecutableFormat = Shared Dll
; CursorPosition = 4
; Folding = AAAg
; EnableThread
; EnableOnError
; Executable = D:\Thorium Ressource Manager\ThoriumRessourceManager64.dll
; DisableDebugger
; IncludeVersionInfo
; VersionField0 = 0,01,0,0
; VersionField1 = 0,01,0,0
; VersionField2 = Thorium
; VersionField3 = Thorium Ressource Manager
; VersionField4 = 0.01
; VersionField5 = 0.01
; VersionField6 = ressource manager for games
; VersionField7 = Thorium Ressource Manager
; VersionField8 = ThoriumRessourceManager.dll