;/------------------\
;| Game Fixer       |
;| Updater          |
;|                  |
;| Version 0.02     |
;| Date: 01.10.2013 |
;| PureBasic 5.20   |
;|                  |
;| by Thorium       |
;\------------------/

;--------------------
;v0.02
;-fixed: If the internal default server is on the server list and all server connections fail it will no longer try to connect twice to the internal default server.
;-fixed: Crash if UpdateServer.lst was not found.
;-changed: Internal default server URL changed to: http://www.GameFixer.org/Update/
;-changed: Compiler changed to PureBasic 5.20.
;-changed: Does not ask for admin rights any longer if it has write rights to the GameFixer folder. *WIP*
;-added: GUI *WIP*
;-added: command line parameter to switch to console mode: -console or -c

;--------------------
;v0.01
;-initial release

EnableExplicit

XIncludeFile "ListIconProgress.pbi"
XIncludeFile "HTTPGetFromWeb2.pbi"
XIncludeFile "Thorium Ressource Manager.pbi"

#AppTitle = "GameFixer Updater"

#SystemFolder = "System\"
#PluginFolder = "Plugins\"
#UpdateFolder = "Update\"

#InjectorFileName    = "GameFixer.exe"
#UpdaterFileName     = "Updater.exe"
#TempUpdaterFileName = "Updater.tmp"

#DefaultUpdateServer = "http://www.GameFixer.org/Update/"

#UpdateInfoFileName      = "UpdateInfo.txt"
#VersionInfoFileName     = "VersionInfo.txt"
#ServerListFileName      = "UpdateServer.lst"
#UpdaterUpdateFileName   = "Updater.gfu"

#UserAgent = "GameFixer Updater"

#Status_Ready      = "Ready"
#Status_Connecting = "Connecting to update server... "
#Status_LoadList   = "Downloading server list..."
#Status_Checking   = "Checking for new versions..."
#Status_LoadInfo   = "Downloading update infos..."
#Status_Updating   = "Updating..."

Enumeration
  #Mode_Normal
  #Mode_DontAsk
  #Mode_Silent
EndEnumeration

Enumeration
  #GFVI_FileVersion      = $0001
  #GFVI_FileDescription  = $0002
  #GFVI_LegalCopyright   = $0004
  #GFVI_InternalName     = $0008
  #GFVI_OriginalFilename = $0010
  #GFVI_ProductName      = $0020
  #GFVI_ProductVersion   = $0040
  #GFVI_CompanyName      = $0080
  #GFVI_LegalTrademarks  = $0100
  #GFVI_SpecialBuild     = $0200
  #GFVI_PrivateBuild     = $0400
  #GFVI_Comments         = $0800
  #GFVI_Language         = $1000
  #GFVI_All              = $1FFF
EndEnumeration

Enumeration
  #Statusbar_Status
  #Statusbar_Progress
EndEnumeration

Enumeration
  #State_Init
  #State_Connecting
  #State_ConnectingFailed
  #State_Connected
  #State_DownloadingServerList
  #State_DownloadingServerListFailed
  #State_ServerListDownloaded
  #State_GettingInfo
  #State_GettingInfoFailed
  #State_GotInfo
  #State_StartUpdateing
  #State_Updateing
  #State_UpdateingFailed
  #State_Updated
  #State_WaitingForUser
EndEnumeration

Structure UpdateInfo
  UpdateName.s
  FileName.s
  Version.i
EndStructure

Prototype.i Proto_GetFileVersionInfoSize(FileName.s, *nUnused)
Prototype.i Proto_VerQueryValue(*sBuffer, *Unknown, *lpBuffer, *nVerSize)
Prototype.i Proto_GetFileVersionInfo(*FileName, Unknown.i, nBufferSize.i, *sBuffer)

Global GetFileVersionInfoSize.Proto_GetFileVersionInfoSize
Global GetFileVersionInfo.Proto_GetFileVersionInfo
Global VerQueryValue.Proto_VerQueryValue

Global SelfPath.s
Global Mode.i
Global ForceUpdate.i
Global OnlyApply.i
Global OnlyDownload.i
Global SelfReplace.i
Global Server.s
Global SingleServer.i
Global DefaultServerTried.i
Global UseConsole.i

Global UpdaterVersion.i
Global GameFixerVersion.i

Global Dim InstalledPlugins.s(1)
Global Dim InstalledPluginsVersion.i(1)
Global InstalledPluginsCnt.i

Global NewList UpdateServer.s()
Global UpdateServerCnt.i

Global NewList UpdateDownloads.UpdateInfo()
Global UpdateDownloadsCnt.i

Global MainWnd.i
Global StatusBar.i
Global UpdateList.i
Global InfoEdit.i
Global UpdateButton.i
Global CancelButton.i

Global Exit.i
Global UpdateState.i

Global OutputMutex.i
Global StateOutputChanged.i
Global StateOutput.s
Global DownloadProgressChanged.i
Global DownloadProgress.i
Global OldDownloadProgress.i
Global DoUpdateUpdater.i

Global UpdateFilesCnt.i
Global Dim UpdateFiles.UpdateInfo(0)

Procedure InitConsole()
  
  If Mode <> #Mode_Silent
    
    If OpenConsole() = 0
      MessageRequester(#AppTitle, "Error: Failed to open console!", #MB_ICONERROR)
      End
    EndIf
    
    PrintN("")
    PrintN("GameFixer Updater v0.01")

  EndIf
  
EndProcedure

Procedure ErrorBox(Text.s)
  
  If Mode < #Mode_DontAsk
    MessageRequester(#AppTitle, Text, #MB_ICONERROR)
  EndIf
  
  End
  
EndProcedure

Procedure Write(Text.s)
  
  If Mode <> #Mode_Silent  And UseConsole = #True
    Print(Text)
  EndIf
    
EndProcedure

Procedure WriteN(Text.s)
  
  If Mode <> #Mode_Silent And UseConsole = #True
    PrintN(Text)
  EndIf
  
EndProcedure

Procedure ShowError(Text.s)
  
  If Mode <> #Mode_Silent
    If UseConsole = #True
      WriteN(Text)
    Else
      ErrorBox(Text)
    EndIf
  EndIf
  
EndProcedure

Procedure ShowText(Text.s)
  
  If Mode <> #Mode_Silent
    If UseConsole = #True
      WriteN(Text)
    Else
      MessageRequester(#AppTitle, Text, #MB_ICONINFORMATION)
    EndIf
  EndIf
  
EndProcedure

Procedure.i YnQuestion(Text.s)
  
  Protected PressedKey.s
  
  If Mode < #Mode_DontAsk
    
    WriteN(Text)
    
    Repeat
      
      Delay(20)
      PressedKey = Inkey()
      
      Select PressedKey
        Case "y", "Y"
          ProcedureReturn #True
        Case "n", "N"
          ProcedureReturn #False
      EndSelect
      
    ForEver

  EndIf
    
EndProcedure

Procedure EndProgram()
  
  If Mode <> #Mode_Silent And UseConsole = #True
    
    WriteN("")
    WriteN("Press enter to end program.")
    
    If Mode < #Mode_DontAsk
      
      Repeat
        Delay(20)
      Until Inkey() = Chr(13)
        
    EndIf
    
  EndIf
    
  End
  
EndProcedure

Procedure CreateMainWnd()
  
  Protected StatusHeight.i
  Protected WndWidth.i
  Protected WndHeight.i
  Protected EditorHeight.i
  Protected ListHeight.i
  
  MainWnd = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, 600, 400, #AppTitle, #PB_Window_SystemMenu | #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_SizeGadget | #PB_Window_TitleBar | #PB_Window_ScreenCentered)
  If MainWnd = #False
    ErrorBox("Error: Failed to create window.")
  EndIf
  
  WindowBounds(MainWnd, 150, 150, #PB_Ignore, #PB_Ignore)
  
  WndWidth  = WindowWidth(MainWnd)
  WndHeight = WindowHeight(MainWnd)
  
  StatusBar = CreateStatusBar(#PB_Any, WindowID(MainWnd))
  
  AddStatusBarField(#PB_Ignore)
  StatusBarText(StatusBar, #Statusbar_Status, #Status_Ready)

  AddStatusBarField(#PB_Ignore)
  StatusBarProgress(StatusBar, #Statusbar_Progress, 0)
  
  StatusHeight = StatusBarHeight(StatusBar)
  
  ListHeight = (WndHeight - StatusHeight - 40) * 0.25
  UpdateList = ListIconGadget(#PB_Any, 0, 0, WndWidth, ListHeight, "Update", WndWidth, #PB_ListIcon_CheckBoxes | #PB_ListIcon_GridLines | #PB_ListIcon_FullRowSelect | #PB_ListIcon_AlwaysShowSelection)
  AddGadgetColumn(UpdateList, 1, "New Version", WndWidth * 0.125 - 3)
  AddGadgetColumn(UpdateList, 2, "Your Version", WndWidth * 0.125 - 3)
  AddGadgetColumn(UpdateList, 3, "Download Progress", WndWidth * 0.25)
  AddGadgetColumn(UpdateList, 4, "Install Progress", WndWidth * 0.25)
  ListIconProgress::Add(UpdateList, 3, 0, 100, #PB_ProgressBar_Smooth)
  ListIconProgress::Add(UpdateList, 4, 0, 100, #PB_ProgressBar_Smooth)
  
  EditorHeight = (WndHeight - StatusHeight - 40) * 0.25
  InfoEdit = EditorGadget(#PB_Any, 0, WndHeight - EditorHeight - StatusHeight - 35, WndWidth, EditorHeight, #PB_Editor_ReadOnly)
  
  UpdateButton = ButtonGadget(#PB_Any, WndWidth / 2 - 75, WndHeight - 25 - StatusHeight - 5, 70, 25, "&Update")
  CancelButton = ButtonGadget(#PB_Any, WndWidth / 2 + 5, WndHeight - 25 - StatusHeight - 5, 70, 25, "&Cancel")
  
EndProcedure

Procedure MainWndResize()
  
  Protected StatusHeight.i
  Protected WndWidth.i
  Protected WndHeight.i
  Protected EditorHeight.i
  Protected ListHeight.i
  Protected ToSmall.i
  
  WndWidth  = WindowWidth(MainWnd)
  WndHeight = WindowHeight(MainWnd)
  
  If WndWidth < 150
    WndWidth = 150
    ToSmall = #True
  EndIf
  
  If WndHeight < 150
    WndHeight = 150
    ToSmall = #True
  EndIf
  
  If ToSmall = #True
    ResizeWindow(MainWnd, #PB_Ignore, #PB_Ignore, WndWidth, WndHeight)
  EndIf

  StatusHeight = StatusBarHeight(StatusBar)
  ListHeight = (WndHeight - StatusHeight - 40) * 0.75
  EditorHeight = (WndHeight - StatusHeight - 40) * 0.25
  
  ResizeGadget(UpdateList, 0, 0, WndWidth, ListHeight)
  SetGadgetItemAttribute(UpdateList, 0, #PB_ListIcon_ColumnWidth, WndWidth * 0.25, 0)
  SetGadgetItemAttribute(UpdateList, 0, #PB_ListIcon_ColumnWidth, WndWidth * 0.125 - 3, 1)
  SetGadgetItemAttribute(UpdateList, 0, #PB_ListIcon_ColumnWidth, WndWidth * 0.125 - 3, 2)
  SetGadgetItemAttribute(UpdateList, 0, #PB_ListIcon_ColumnWidth, WndWidth * 0.25, 3)
  SetGadgetItemAttribute(UpdateList, 0, #PB_ListIcon_ColumnWidth, WndWidth * 0.25, 4)
  
  ResizeGadget(InfoEdit, 0, WndHeight - EditorHeight - StatusHeight - 35, WndWidth, EditorHeight)
  ResizeGadget(UpdateButton, WndWidth / 2 - 75, WndHeight - 25 - StatusHeight - 5, 70, 25)
  ResizeGadget(CancelButton, WndWidth / 2 + 5, WndHeight - 25 - StatusHeight - 5, 70, 25)

EndProcedure

Procedure InitGui()
  
  CreateMainWnd()
  
EndProcedure

Procedure.i GetFileVersion(FileName.s)
  
  Protected *sBuffer
  Protected nBufferSize.i
  Protected lpBuffer.i
  Protected nVerSize.i
  Protected nUnused.i
  Protected FileVersion.i
  Protected Minor.s
  
  nBufferSize = GetFileVersionInfoSize(FileName, @nUnused)
  
  If nBufferSize > 0
    
    *sBuffer = AllocateMemory(nBufferSize)
    GetFileVersionInfo(@FileName, 0, nBufferSize, *sBuffer)
    VerQueryValue(*sBuffer, @"\", @lpBuffer, @nVerSize)
    
    Minor = Str(PeekW(lpBuffer + 8))
    If Len(Minor) = 1
      Minor = "0" + Minor
    EndIf
    
    FileVersion = Val(Str(PeekW(lpBuffer + 10)) + Minor)
    
  EndIf
  
  ProcedureReturn FileVersion
  
EndProcedure

Procedure.s CommandField(String.s, Index.i, StripQuotes.i = 0)
  
  Protected l.i, field.i, inside.i, p_start.i, p.i
  ;
  ; *** returns space seperated fields, keeps double quoted sections whole
  ;
  ; in:       string.s                - string to parse
  ;           index.i                 - field to return
  ;           stripquotes.i = #true   - include encompassing quotes in return value
  ; retval:   .s                      - string matching the index number
  ;
  ; a 'command line string' contains multiple sections, of which some are enclosed in parenthesis
  ; this routine splits up and spits out :-) space seperated sections, where spaces in doublequoted
  ; parts are ignored
  ;
  l = Len(string)
  field = 0
  inside = #False
  p_start = 1
  p = 0
  While p < l And field <> index
    p = p+1
    Select Asc(Mid(string,p,1))
    Case ' '                        ; it's a space
      If Not inside                 ; but we're not inside so it counts as a seperator
        If p = p_start              ; we just started so it must be two spaces following eachother
          p_start = p+1             ; next char should be a non-space, otherwise we end up here again
        Else
          field = field+1           ; so we just passed a valid field
          If field <> index         ; if it wasn't the field we were looking for
            p_start = p+1           ; then we'll set the start pointer to the start of the next field
          EndIf
        EndIf
      EndIf
    Case '"'                        ; toggle for doublequoted sections
      inside = 1-inside
    EndSelect
  Wend
  If field <> index And p = l       ; reached the end of the string
    field = field+1
    p = p+1
  EndIf
  If field = index                                               ; we had a match
    If stripquotes <> 0 And Mid(string,p_start,1)=#DQUOTE$       ; optional stripping of doublequotes
      ProcedureReturn Trim(Mid(string,p_start+1,p-p_start-2))
    Else
      ProcedureReturn Mid(string,p_start,p-p_start)              ; no stripping
    EndIf
  Else
    ProcedureReturn ""
  EndIf
  
EndProcedure

Procedure.s GetHttpErrorString(Error.i, *tHttp.T_PBL_HTTP_GET_FROM_WEB)
  
  Select Error
  
    Case #PBL_ERR_INTERNET_OPEN                
      ProcedureReturn "Error initializing the WININET.DLL API."
        
    Case #PBL_ERR_INTERNET_CONNECT
      ProcedureReturn "Error opening the session to the given site."
        
    Case #PBL_ERR_INTERNET_SET_PROXY
      ProcedureReturn "Error setting the options for the proxy server."
        
    Case #PBL_ERR_INTERNET_HTTP_OPEN
      ProcedureReturn "Error creating the HTTP request handle."
        
    Case #PBL_ERR_INTERNET_HTTP_SEND
      ProcedureReturn "Error sending the HTTP request to the server."
        
    Case #PBL_ERR_INTERNET_HTTP_QUERY_STATUS
      ProcedureReturn "Error retrieving the HTTP status code."
        
    Case #PBL_ERR_INTERNET_READ
      ProcedureReturn "Error reading data from the opened session."
        
    Case #PBL_ERR_INTERNET_CLOSE
      ProcedureReturn "Error closing the Internet handle."
    
    Case #PBL_ERR_INTERNET_BASIC_AUTH
      ProcedureReturn "Error setting the options for basic authentication."
    
    Case #PBL_ERR_HTTP_STATUS                      
      ProcedureReturn "HTTP status code = " + Str(*tHTTP\lErrorCodeEx)

    Case #PBL_ERR_FILE_CREATION                
      ProcedureReturn "Cannot create the file " + *tHTTP\sFullFileName
        
    Case #PBL_ERR_FILE_IO                      
      ProcedureReturn "Error writing to the file " + *tHTTP\sFullFileName
                
    Case #PBL_ERR_OUT_OF_MEMORY                        
      ProcedureReturn "Out of memory." 

    Case #PBL_ERR_ABORT_REQUESTED              
      ProcedureReturn "Download aborted."                    
        
    Default
      ProcedureReturn "Unexpected error = " + Str(Error)
      
  EndSelect
  
EndProcedure

Procedure.s GetSelfPath()

  Protected ModulePath.s

  ModulePath = Space(1024)
  GetModuleFileName_(0, @ModulePath, 1024)
  ProcedureReturn GetPathPart(Trim(ModulePath))

EndProcedure

Procedure.s GetVersionString(Version.i)
  
  Protected VersionStr.s
  
  VersionStr = Str(Version)
  Select Len(VersionStr)
    Case 1
      VersionStr = "0.0" + VersionStr
    Case 2
      VersionStr = "0." + VersionStr
    Default
      InsertString(VersionStr, ".", Len(VersionStr) - 1)
  EndSelect
  
  ProcedureReturn VersionStr
  
EndProcedure

Procedure.i CreateDirs(Path.s)
  
  Protected i.i
  Protected PathPartCnt.i
  Protected WorkPath.s
  
  If Right(Path, 1) <> "\"
    Path = Path + "\"
  EndIf
  
  If Path = "\"
    ProcedureReturn #True
  EndIf

  PathPartCnt = CountString(Path, "\")
  WorkPath = SelfPath
  
  For i = 1 To PathPartCnt
    WorkPath = WorkPath + StringField(Path, i, "\") + "\"
    If FileSize(WorkPath) <> -2
      WriteN("Creating directory... : " + WorkPath)
      If CreateDirectory(WorkPath) = #False
        ProcedureReturn #False
      EndIf
    EndIf
  Next
  
  ProcedureReturn #True
      
EndProcedure

Procedure.s BuildUpdaterParams(AddSelfReplace.i = #False)
  
  Protected Params.s
  
  If Mode = #Mode_DontAsk
    Params = Params + "-da "
  ElseIf Mode = #Mode_Silent
    Params = Params + "-s "
  EndIf
  
  If ForceUpdate = #True
    Params = Params + "-f "
  EndIf
  
  If OnlyApply = #True
    Params = Params + "-oa "
  EndIf
  
  If OnlyDownload = #True
    Params = Params + "-od "
  EndIf
  
  If AddSelfReplace = #True
    Params = Params + "-sr "
  EndIf
  
  If SingleServer = #True
    Params = Params + "-ss "
  EndIf
  
  If Server <> ""
    Params = Params + "-svr " + Server + " "
  EndIf
  
  If Len(Params) > 0
    Params = Left(Params, Len(Params) - 1)
  EndIf
  
  ProcedureReturn Params
  
EndProcedure

Procedure GetServerList()
  
  Protected ListFile.i
  Protected Server.s
  
  UpdateServerCnt = 0
  ClearList(UpdateServer())
  ListFile = ReadFile(#PB_Any, SelfPath + #SystemFolder + #ServerListFileName)
  
  If ListFile <> 0
    While Eof(ListFile) = #False
      
      AddElement(UpdateServer())
      Server = ReadString(ListFile)
      If Right(Server, 1) <> "/"
        Server = Server + "/"
      EndIf
      UpdateServer() = Server
      UpdateServerCnt + 1
      
    Wend
    
    CloseFile(ListFile)
  EndIf

EndProcedure

Procedure GetInstalledPlugins()
  
  Protected Dir.i
  
  InstalledPluginsCnt = 0
  
  Dir = ExamineDirectory(#PB_Any, SelfPath + #PluginFolder, "*.gfp")
  If Dir = 0
    ProcedureReturn
  EndIf
  
  While NextDirectoryEntry(Dir)
    
    If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
      InstalledPluginsCnt + 1
      ReDim InstalledPlugins.s(InstalledPluginsCnt)
      InstalledPlugins(InstalledPluginsCnt) = DirectoryEntryName(Dir)
    EndIf
    
  Wend

  FinishDirectory(Dir)
  
EndProcedure

Procedure GetInstalledVersions()
  
  Protected VersionDll.i
  Protected i.i
  
  VersionDll = OpenLibrary(#PB_Any, "Version.dll")
  GetFileVersionInfoSize = GetFunction(VersionDll, "GetFileVersionInfoSizeA")
  GetFileVersionInfo = GetFunction(VersionDll, "GetFileVersionInfoA")
  VerQueryValue = GetFunction(VersionDll, "VerQueryValueA")
  
  UpdaterVersion = GetFileVersion(SelfPath + #SystemFolder + #UpdaterFileName)
  GameFixerVersion = GetFileVersion(SelfPath + #InjectorFileName)
  
  ReDim InstalledPluginsVersion.i(InstalledPluginsCnt)
  For i = 1 To InstalledPluginsCnt
    InstalledPluginsVersion(i) = GetFileVersion(SelfPath + #PluginFolder + InstalledPlugins(i))    
  Next

EndProcedure

Procedure.s SelectServer()
  
  Protected ServerIndex.i
  Protected Server.s
  
  If UpdateServerCnt = 0
    ProcedureReturn ""
  EndIf
  
  ServerIndex = Random(UpdateServerCnt - 1)
  SelectElement(UpdateServer(), ServerIndex)
  Server = UpdateServer()
  If LCase(Server) = LCase(#DefaultUpdateServer)
    DefaultServerTried = #True
  EndIf
  DeleteElement(UpdateServer())
  UpdateServerCnt - 1
  
  ProcedureReturn Server
    
EndProcedure

Procedure.i DecompressRes(ResName.s, DestFileName.s)
  
  Protected ResAddr.i
  Protected ResSize.i
  Protected File.i

  If Rmgr_LoadResSeq(ResName, @ResAddr, @ResSize) = #False
    WriteN("Error: Failed to decompress file!")
    ProcedureReturn #False
  EndIf
    
  File = CreateFile(#PB_Any, DestFileName)
  If File = #False
    WriteN("Error: Failed to write file to disk!")
    ProcedureReturn #False
  EndIf
  
  WriteData(File, ResAddr, ResSize)
  CloseFile(File)
  
  ProcedureReturn #True
  
EndProcedure

Procedure UpdateUpdater(Url.s)
  
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  Protected Ret.i
  
  WriteN("")
  WriteN("Downloading Updater...")
  
  tHttp\lThreadID     = 0
  tHttp\sURL          = Url
  tHttp\lDestination  = #PBL_WRITE_TO_FILE
  tHttp\lChunkSize    = 65536
  tHttp\sUserAgent    = #UserAgent
  tHttp\sFullFileName = SelfPath + #UpdateFolder + "Updater.gfu"
  tHttp\DestBuffer    = 0
  tHttp\fpCallBack    = 0
  tHttp\fpCallStart   = 0
  tHttp\fpCallEnd     = 0
  
  Ret = HTTPGetFromWeb(@tHttp)
  
  If Ret <> #PBL_OK
    WriteN("Error: " + GetHttpErrorString(Ret, @tHttp))
  Else
  
    WriteN("Decompressing Updater...")

    If Rmgr_IndexArchiv(SelfPath + #UpdateFolder + "Updater.gfu") = #False
      WriteN("Error: Failed to read update file!")
    Else
      
      If DecompressRes(#SystemFolder + #UpdaterFileName, SelfPath + #UpdateFolder + #TempUpdaterFileName) = #True
        
        Rmgr_ClearIndex()
        DeleteFile(SelfPath + #UpdateFolder + "Updater.gfu")
        WriteN("Running new Updater...")
        
        If RunProgram(SelfPath + #UpdateFolder + #TempUpdaterFileName, BuildUpdaterParams(#True), SelfPath) = #False
          WriteN("Error: Failed to run new Updater.")
        Else
          End
        EndIf

      EndIf
    EndIf
  EndIf
  
EndProcedure

Procedure DeleteUpdates()
  
  Protected Dir.i
  
  Dir = ExamineDirectory(#PB_Any, SelfPath + #UpdateFolder, "*.gfu")
  If Dir <> 0
    While NextDirectoryEntry(Dir)
      If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
        DeleteFile(SelfPath + #UpdateFolder + DirectoryEntryName(Dir))
      EndIf
    Wend
  EndIf

EndProcedure

; Procedure DownloadUpdates_Old()
;   
;   Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
;   Protected Ret.i
;   
;   WriteN("")
;   WriteN("Downloading updates...")
;   
;   DeleteUpdates()
;   
;   ForEach UpdateDownloads()
;   
;     WriteN("Downloading... : " + UpdateDownloads()\UpdateName)
;     
;     tHttp\lThreadID     = 0
;     tHttp\sURL          = Server + UpdateDownloads()\FileName
;     tHttp\lDestination  = #PBL_WRITE_TO_FILE
;     tHttp\lChunkSize    = 65536
;     tHttp\sUserAgent    = #UserAgent
;     tHttp\sFullFileName = SelfPath + #UpdateFolder + UpdateDownloads()\FileName
;     tHttp\DestBuffer    = 0
;     tHttp\fpCallBack    = 0
;     tHttp\fpCallStart   = 0
;     tHttp\fpCallEnd     = 0
;     
;     Ret = HTTPGetFromWeb(@tHttp)
;     
;     If Ret <> #PBL_OK
;       WriteN("Error: " + GetHttpErrorString(Ret, @tHttp))
;       ProcedureReturn #False
;     EndIf
; 
;   Next
; 
;   ProcedureReturn #True
;   
; EndProcedure
; 
; Procedure ApplyUpdates_Old()
;   
;   Protected Dir.i
;   Protected UpdateFileName.s
;   Protected Dim ResName.a(255)
;   Protected ResNameLen.i
;   Protected Res.s
;   
;   WriteN("")
;   WriteN("Applying updates...")
;   
;   Dir = ExamineDirectory(#PB_Any, SelfPath + #UpdateFolder, "*.gfu")
;   If Dir = 0
;     ProcedureReturn #False
;   EndIf
;   
;   While NextDirectoryEntry(Dir)
;     If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
;       
;       UpdateFileName = DirectoryEntryName(Dir)
;       If Rmgr_IndexArchiv(SelfPath + #UpdateFolder + UpdateFileName) = #False
;         WriteN("Error: Failed to read update file: " + UpdateFileName)
;       Else
;         
;         Repeat
;           
;           ResNameLen = Rmgr_EnumRes(@ResName(0))
;           If ResNameLen > 0
;             Res = PeekS(@ResName(0), ResNameLen)
;             
;             If CreateDirs(GetPathPart(Res)) = #False
;               FinishDirectory(Dir)
;               ProcedureReturn #False
;             EndIf
;             
;             WriteN("Decompressing... : " + Res)
;             
;             If DecompressRes(Res, SelfPath + Res) = #False
;               FinishDirectory(Dir)
;               ProcedureReturn #False
;             EndIf
; 
;           EndIf
;         
;         Until ResNameLen = 0
;         
;         Rmgr_ClearIndex()
;         DeleteFile(SelfPath + #UpdateFolder + UpdateFileName)
; 
;       EndIf
;   
;     EndIf
;   Wend
;   
;   FinishDirectory(Dir)
;   
;   ProcedureReturn #True
;   
; EndProcedure
; 
; Procedure OnlineUpdate_Old()
;   
;   Protected UseDefaultServer.i
;   Protected UpdateOk.i
;   Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
;   Protected Ret.i
;   Protected UpdateInfo.s
;   Protected NextServer.i
;   Protected i.i
;   Protected i2.i
;   Protected Dim UpdateFiles.UpdateInfo(1)
;   Protected UpdateFilesCnt.i
;   Protected UpdateInfoLine.s
;   Protected UpdaterUpdateFailed.i
;   Protected NewVersionsOnServer.i
;   Protected PluginVersion.i
;   Protected VersionInfo.s
;   
;   If Server = ""
;     Server = SelectServer()
;     If Server = ""
;       Server = #DefaultUpdateServer
;       UseDefaultServer = #True
;     EndIf
;   EndIf
;   
;   Repeat
;     
;     UpdaterUpdateFailed = #False
; 
;     WriteN("")
;     WriteN("Connecting to update server... : " + Server)
;     
;     tHttp\lThreadID     = 0
;     tHttp\sURL          = Server + #UpdateInfoFileName
;     tHttp\lDestination  = #PBL_WRITE_TO_MEMORY
;     tHttp\lChunkSize    = 0
;     tHttp\sUserAgent    = #UserAgent
;     tHttp\sFullFileName = ""
;     tHttp\DestBuffer    = 0
;     tHttp\fpCallBack    = 0
;     tHttp\fpCallStart   = 0
;     tHttp\fpCallEnd     = 0
;     Ret = HTTPGetFromWeb(@tHttp)
;     
;     If Ret <> #PBL_OK
;       WriteN("Error: " + GetHttpErrorString(Ret, @tHttp))
;     Else
;       
;       UpdateInfo = LCase(PeekS(tHttp\DestBuffer, tHttp\lTotBytesRead))
;       FreeMemory(tHttp\DestBuffer)
;       
;       Select UpdateInfo
;           
;         Case "ok"
;           NextServer = #False
;           
;         Case "noupdates"
;           WriteN("Server does not offer updates anymore.")
;           NextServer = #True
;           
;         Case "maintenance"
;           WriteN("Server is currently under maintenance.")
;           NextServer = #True
;           
;         Default
;           WriteN("Server is in unknown state.")
;           NextServer = #True
;           
;       EndSelect
;       
;       If NextServer = #False
;         
;         WriteN("Downloading server list...")
;         
;         tHttp\lThreadID     = 0
;         tHttp\sURL          = Server + #ServerListFileName
;         tHttp\lDestination  = #PBL_WRITE_TO_FILE
;         tHttp\lChunkSize    = 0
;         tHttp\sUserAgent    = #UserAgent
;         tHttp\sFullFileName = SelfPath + #SystemFolder + #ServerListFileName
;         tHttp\DestBuffer    = 0
;         tHttp\fpCallBack    = 0
;         tHttp\fpCallStart   = 0
;         tHttp\fpCallEnd     = 0
;         Ret = HTTPGetFromWeb(@tHttp)
;         
;         If Ret <> #PBL_OK          
;           WriteN("Error: " + GetHttpErrorString(Ret, @tHttp))
;         Else
; 
;           WriteN("Checking for new versions...")
;           
;           tHttp\lThreadID     = 0
;           tHttp\sURL          = Server + #VersionInfoFileName
;           tHttp\lDestination  = #PBL_WRITE_TO_MEMORY
;           tHttp\lChunkSize    = 0
;           tHttp\sUserAgent    = #UserAgent
;           tHttp\sFullFileName = ""
;           tHttp\DestBuffer    = 0
;           tHttp\fpCallBack    = 0
;           tHttp\fpCallStart   = 0
;           tHttp\fpCallEnd     = 0
;           Ret = HTTPGetFromWeb(@tHttp)
;           
;           If Ret <> #PBL_OK
;             WriteN("Error: " + GetHttpErrorString(Ret, @tHttp))
;           Else
;             
;             VersionInfo = PeekS(tHttp\DestBuffer, tHttp\lTotBytesRead)
;             FreeMemory(tHttp\DestBuffer)
;             
;             UpdateFilesCnt = CountString(VersionInfo, #CRLF$)
;             
;             If UpdateFilesCnt = 0
;               
;               WriteN("Error: Corrupted version infos on server!")
;               
;             Else
;             
;               ReDim UpdateFiles.UpdateInfo(UpdateFilesCnt)
;               
;               For i = 1 To UpdateFilesCnt
;                 UpdateInfoLine = StringField(VersionInfo, i, #CRLF$)
;                 UpdateFiles(i)\UpdateName = CommandField(UpdateInfoLine, 1)
;                 If Left(UpdateFiles(i)\UpdateName, 1) = #LF$
;                   UpdateFiles(i)\UpdateName = Right(UpdateFiles(i)\UpdateName, Len(UpdateFiles(i)\UpdateName) - 1)
;                 EndIf
;                 UpdateFiles(i)\FileName   = CommandField(UpdateInfoLine, 2)
;                 UpdateFiles(i)\Version    = Val(CommandField(UpdateInfoLine, 3))
;               Next
;               
;               VersionInfo = ""
;               
;               For i = 1 To UpdateFilesCnt
;                 If LCase(UpdateFiles(i)\UpdateName) = "updater"
;                   If UpdateFiles(i)\Version > UpdaterVersion
;                     
;                     WriteN("")
;                     WriteN("New Updater version on server.")
;                     WriteN("Installed version: " + GetVersionString(UpdaterVersion))
;                     WriteN("New version      : " + GetVersionString(UpdateFiles(i)\Version))
;                     WriteN("")
;                     
;                     If YnQuestion("Do you want to update? (y/n)") = #False
;                       EndProgram()
;                     Else
;                       UpdateUpdater(Server + UpdateFiles(i)\FileName)
;                       UpdaterUpdateFailed = #True
;                     EndIf
;                     
;                   EndIf
;                 EndIf
;               Next
;               
;               If UpdaterUpdateFailed = #False
;                 
;                 WriteN("")
;                 WriteN("Old Version | New Version | Name")
;                 WriteN("")
;                 
;                 ClearList(UpdateDownloads())
;                 UpdateDownloadsCnt = 0
;                 
;                 For i = 1 To UpdateFilesCnt
;                   
;                   If LCase(UpdateFiles(i)\UpdateName) = "updater"
;                     
;                     ;do nothing
;                     
;                   ElseIf LCase(UpdateFiles(i)\UpdateName) = "gamefixer"
;                     
;                     WriteN(GetVersionString(GameFixerVersion) + Space(10) + GetVersionString(UpdateFiles(i)\Version) + Space(10) + UpdateFiles(i)\UpdateName)
;                     
;                     If GameFixerVersion < UpdateFiles(i)\Version Or ForceUpdate = #True
;                       AddElement(UpdateDownloads())
;                       UpdateDownloads()\FileName   = UpdateFiles(i)\FileName
;                       UpdateDownloads()\UpdateName = UpdateFiles(i)\UpdateName
;                       UpdateDownloadsCnt + 1
;                     EndIf
;                     
;                   Else
;                     
;                     PluginVersion = 0
;                     For i2 = 1 To InstalledPluginsCnt
;                       If LCase(InstalledPlugins(i2)) = LCase(UpdateFiles(i)\UpdateName) + ".gfp"
;                         PluginVersion = InstalledPluginsVersion(i2)
;                         Break                        
;                       EndIf
;                     Next
;                     
;                     If PluginVersion = 0
;                       Write("None")
;                     Else
;                       Write(GetVersionString(PluginVersion))
;                     EndIf
;                     WriteN(Space(10) + GetVersionString(UpdateFiles(i)\Version) + Space(10) + UpdateFiles(i)\UpdateName)
; 
;                     If PluginVersion < UpdateFiles(i)\Version Or ForceUpdate = #True
;                       AddElement(UpdateDownloads())
;                       UpdateDownloads()\FileName   = UpdateFiles(i)\FileName
;                       UpdateDownloads()\UpdateName = UpdateFiles(i)\UpdateName
;                       UpdateDownloadsCnt + 1
;                     EndIf
;                     
;                   EndIf
;                   
;                 Next
;                 
;                 WriteN("")
;                 WriteN(Str(UpdateDownloadsCnt) + " updates available.")
;                 
;                 If UpdateDownloadsCnt > 0
;                   If YnQuestion("Do you want to update? (y/n)") = #False
;                     EndProgram()
;                   Else
;                     If DownloadUpdates() = #True
;                       If OnlyDownload = #True
;                         UpdateOk = #True
;                       ElseIf ApplyUpdates() = #True
;                         UpdateOk = #True
;                       EndIf
;                     EndIf
;                   EndIf
;                 Else
;                   UpdateOk = #True
;                 EndIf
; 
;               EndIf
;             EndIf
;           EndIf
;         EndIf
;       EndIf
;     EndIf
; 
;     If UpdateOk = #False And UseDefaultServer = #True
;       WriteN("")
;       WriteN("Tried all servers of the update server list.")
;       WriteN("Update failed!")
;       EndProgram()
;     EndIf
;     
;     Server = SelectServer()
;     If Server = ""
;       If DefaultServerTried = #True
;         WriteN("")
;         WriteN("Tried all servers of the update server list.")
;         WriteN("Update failed!")
;         EndProgram()
;       Else
;         Server = #DefaultUpdateServer
;         UseDefaultServer = #True
;       EndIf
;     EndIf
;     
;   Until UpdateOk = #True
; 
; EndProcedure

Procedure.i DownloadCallback(*tHTTP, BytesReceived.i, Size.i, ElapsedTime.i)
  
  Protected DownReceived.f
  Protected DownSize.f
  
  DownReceived = BytesReceived
  DownSize     = Size
  
  OldDownloadProgress = DownloadProgress
  DownloadProgress = DownReceived / DownSize * 100
  
  If DownloadProgress <> OldDownloadProgress
    DownloadProgressChanged = #True
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

Procedure ConnectServer(Dummy.i)
  
  Protected UseDefaultServer.i
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  Protected Ret.i
  Protected NextServer.i
  Protected ConnectOk.i
  Protected UpdateInfo.s
  
  If Server = ""
    Server = SelectServer()
    If Server = ""
      Server = #DefaultUpdateServer
      UseDefaultServer = #True
    EndIf
  EndIf
  
  Repeat
    
    tHttp\lThreadID     = 0
    tHttp\sURL          = Server + #UpdateInfoFileName
    tHttp\lDestination  = #PBL_WRITE_TO_MEMORY
    tHttp\lChunkSize    = 0
    tHttp\sUserAgent    = #UserAgent
    tHttp\sFullFileName = ""
    tHttp\DestBuffer    = 0
    tHttp\fpCallBack    = 0
    tHttp\fpCallStart   = 0
    tHttp\fpCallEnd     = 0
    Ret = HTTPGetFromWeb(@tHttp)
    
    If Ret <> #PBL_OK
      
      LockMutex(OutputMutex)
      StateOutput = "Failed to connect to " + Server + " Error: " + GetHttpErrorString(Ret, @tHttp)
      StateOutputChanged = #True
      UnlockMutex(OutputMutex)
      NextServer = #True
      
    Else
      
      UpdateInfo = LCase(PeekS(tHttp\DestBuffer, tHttp\lTotBytesRead))
      FreeMemory(tHttp\DestBuffer)
      
      Select UpdateInfo
        
        Case "ok"
          NextServer = #False
          
        Case "noupdates"
          LockMutex(OutputMutex)
          StateOutput = "Server does not offer updates anymore: " + Server
          StateOutputChanged = #True
          UnlockMutex(OutputMutex)
          NextServer = #True
          
        Case "maintenance"
          LockMutex(OutputMutex)
          StateOutput = "Server is currently under maintenance: " + Server
          StateOutputChanged = #True
          UnlockMutex(OutputMutex)
          NextServer = #True
          
        Default
          LockMutex(OutputMutex)
          StateOutput = "Server is in unknown state: " + Server
          StateOutputChanged = #True
          UnlockMutex(OutputMutex)
          NextServer = #True
          
      EndSelect
      
    EndIf
      
    If NextServer = #False
      
      ConnectOk = #True
      LockMutex(OutputMutex)
      UpdateState = #State_Connected
      UnlockMutex(OutputMutex)
      
    Else
    
      If UseDefaultServer = #True
        LockMutex(OutputMutex)
        UpdateState = #State_ConnectingFailed
        UnlockMutex(OutputMutex)
      Else
        Server = SelectServer()
        If Server = ""
          If DefaultServerTried = #True
            LockMutex(OutputMutex)
            UpdateState = #State_ConnectingFailed
            UnlockMutex(OutputMutex)
          Else
            Server = #DefaultUpdateServer
            UseDefaultServer = #True
          EndIf
        EndIf
      EndIf
      
    EndIf
    
  Until ConnectOk = #True
  
EndProcedure

Procedure DownloadServerList(Dummy.i)
  
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  Protected Ret.i

  tHttp\lThreadID     = 0
  tHttp\sURL          = Server + #ServerListFileName
  tHttp\lDestination  = #PBL_WRITE_TO_FILE
  tHttp\lChunkSize    = 4096
  tHttp\sUserAgent    = #UserAgent
  tHttp\sFullFileName = "D:\" + #ServerListFileName ;SelfPath + #SystemFolder + #ServerListFileName
  tHttp\DestBuffer    = 0
  tHttp\fpCallBack    = @DownloadCallback()
  tHttp\fpCallStart   = 0
  tHttp\fpCallEnd     = 0
  Ret = HTTPGetFromWeb(@tHttp)
  
  If Ret <> #PBL_OK
    LockMutex(OutputMutex)
    UpdateState = #State_DownloadingServerListFailed
    UnlockMutex(OutputMutex)
  Else
    LockMutex(OutputMutex)
    UpdateState = #State_ServerListDownloaded
    UnlockMutex(OutputMutex)
  EndIf

EndProcedure

Procedure GetUpdateInfos(Dummy.i)
  
  Protected i.i
  Protected tHttp.T_PBL_HTTP_GET_FROM_WEB
  Protected Ret.i
  Protected VersionInfo.s
  Protected UpdateInfoLine.s

  tHttp\lThreadID     = 0
  tHttp\sURL          = Server + #VersionInfoFileName
  tHttp\lDestination  = #PBL_WRITE_TO_MEMORY
  tHttp\lChunkSize    = 4096
  tHttp\sUserAgent    = #UserAgent
  tHttp\sFullFileName = SelfPath + #SystemFolder + #ServerListFileName
  tHttp\DestBuffer    = 0
  tHttp\fpCallBack    = @DownloadCallback()
  tHttp\fpCallStart   = 0
  tHttp\fpCallEnd     = 0
  Ret = HTTPGetFromWeb(@tHttp)

  If Ret <> #PBL_OK
    
    LockMutex(OutputMutex)
    UpdateState = #State_GettingInfoFailed
    UnlockMutex(OutputMutex)
    
  Else

    VersionInfo = PeekS(tHttp\DestBuffer, tHttp\lTotBytesRead)
    FreeMemory(tHttp\DestBuffer)
    
    UpdateFilesCnt = CountString(VersionInfo, #CRLF$)
    
    If UpdateFilesCnt = 0
      
      LockMutex(OutputMutex)
      UpdateState = #State_GettingInfoFailed
      UnlockMutex(OutputMutex)
      
    Else
    
      ReDim UpdateFiles.UpdateInfo(UpdateFilesCnt)
      
      For i = 1 To UpdateFilesCnt
        UpdateInfoLine = StringField(VersionInfo, i, #CRLF$)
        UpdateFiles(i)\UpdateName = CommandField(UpdateInfoLine, 1)
        If Left(UpdateFiles(i)\UpdateName, 1) = #LF$
          UpdateFiles(i)\UpdateName = Right(UpdateFiles(i)\UpdateName, Len(UpdateFiles(i)\UpdateName) - 1)
        EndIf
        UpdateFiles(i)\FileName = CommandField(UpdateInfoLine, 2)
        UpdateFiles(i)\Version = Val(CommandField(UpdateInfoLine, 3))
      Next
      
      LockMutex(OutputMutex)
      UpdateState = #State_GotInfo
      UnlockMutex(OutputMutex)
      
    EndIf
    
  EndIf

EndProcedure

Procedure AddToUpdateList(Index.i)
  
  Protected i.i
  Protected ListPos.i
  Protected PluginVersion.i
  
  UpdateDownloadsCnt = UpdateDownloadsCnt + 1
  AddElement(UpdateDownloads())
  UpdateDownloads()\FileName   = UpdateFiles(Index)\FileName
  UpdateDownloads()\UpdateName = UpdateFiles(Index)\UpdateName
  UpdateDownloads()\Version    = UpdateFiles(Index)\Version
  
  If Mode <> #Mode_Silent
    Select LCase(UpdateFiles(Index)\UpdateName)
        
      Case "updater"
        If UseConsole = #True
  
        Else
          AddGadgetItem(UpdateList, -1, UpdateFiles(Index)\UpdateName)
          ListPos = CountGadgetItems(UpdateList) - 1
          SetGadgetItemText(UpdateList, ListPos, GetVersionString(UpdateFiles(Index)\Version), 1)
          SetGadgetItemText(UpdateList, ListPos, GetVersionString(UpdaterVersion), 2)
          SetGadgetItemState(UpdateList, ListPos, #PB_ListIcon_Checked)
        EndIf
        
      Case "gamefixer"
        If UseConsole = #True
  
        Else
          AddGadgetItem(UpdateList, -1, UpdateFiles(Index)\UpdateName)
          ListPos = CountGadgetItems(UpdateList) - 1
          SetGadgetItemText(UpdateList, ListPos, GetVersionString(UpdateFiles(Index)\Version), 1)
          SetGadgetItemText(UpdateList, ListPos, GetVersionString(GameFixerVersion), 2)
          SetGadgetItemState(UpdateList, ListPos, #PB_ListIcon_Checked)
        EndIf
        
      Default
        If UseConsole = #True
  
        Else
          AddGadgetItem(UpdateList, -1, UpdateFiles(Index)\UpdateName)
          ListPos = CountGadgetItems(UpdateList) - 1
          SetGadgetItemText(UpdateList, ListPos, GetVersionString(UpdateFiles(Index)\Version), 1)
          PluginVersion = 0
          For i = 1 To InstalledPluginsCnt
            If LCase(InstalledPlugins(i)) = LCase(UpdateFiles(Index)\UpdateName) + ".gfp"
              PluginVersion = InstalledPluginsVersion(i)
              Break
            EndIf
          Next
          If PluginVersion = 0
            SetGadgetItemText(UpdateList, ListPos, "none", 2)
          Else
            SetGadgetItemText(UpdateList, ListPos, GetVersionString(PluginVersion), 2)
          EndIf
        EndIf
        SetGadgetItemState(UpdateList, ListPos, #PB_ListIcon_Checked)

    EndSelect
  EndIf
  
EndProcedure

Procedure CheckUpdates()
  
  Protected i.i
  Protected i2.i
  Protected PluginVersion.i
  
  ClearList(UpdateDownloads())
  UpdateDownloadsCnt = 0
  
  ;check if there is a update for the updater
  
  For i = 1 To UpdateFilesCnt
    If LCase(UpdateFiles(i)\UpdateName) = "updater"
      If UpdateFiles(i)\Version > UpdaterVersion
        AddToUpdateList(i)
        DoUpdateUpdater = #True
        ShowText("There is a new version of the updater." + Chr(10) + "You need to update the updater first.")
      EndIf
    EndIf
  Next
  
  If DoUpdateUpdater = #False
    
    For i = 1 To UpdateFilesCnt
      
      If LCase(UpdateFiles(i)\UpdateName) = "updater"
        
        ;do nothing
        
      ElseIf LCase(UpdateFiles(i)\UpdateName) = "gamefixer"
        
        If GameFixerVersion < UpdateFiles(i)\Version Or ForceUpdate = #True
          AddToUpdateList(i)
        EndIf
        
      Else
        
        PluginVersion = 0
        For i2 = 1 To InstalledPluginsCnt
          If LCase(InstalledPlugins(i2)) = LCase(UpdateFiles(i)\UpdateName) + ".gfp"
            PluginVersion = InstalledPluginsVersion(i2)
            Break                        
          EndIf
        Next
        If PluginVersion < UpdateFiles(i)\Version Or ForceUpdate = #True
          AddToUpdateList(i)
        EndIf
        
      EndIf
      
    Next
    
    If UpdateDownloadsCnt = 0
      ShowText("No updates available.")
      EndProgram()
    EndIf
    
  EndIf
  
EndProcedure

Procedure UpdateLoop()
  
  Select UpdateState
      
    Case #State_Init
      UpdateState = #State_Connecting
      If Mode <> #Mode_Silent
        StatusBarText(StatusBar, #Statusbar_Status, #Status_Connecting)
        CreateThread(@ConnectServer(), 0)
      EndIf
      
    Case #State_Connecting
      If Mode <> #Mode_Silent
        If StateOutputChanged = #True
          LockMutex(OutputMutex)
          StateOutputChanged = #False
          AddGadgetItem(InfoEdit, -1, StateOutput)
          UnlockMutex(OutputMutex)
        EndIf
      EndIf
      
    Case #State_ConnectingFailed
      ShowError("Tried all servers of the update server list." + Chr(10) + "Update failed!")
      EndProgram()
      
    Case #State_Connected
      UpdateState = #State_DownloadingServerList
      If Mode <> #Mode_Silent
        StatusBarText(StatusBar, #Statusbar_Status, #Status_LoadList)
        CreateThread(@DownloadServerList(), 0)
      EndIf
      
    Case #State_DownloadingServerList
      If Mode <> #Mode_Silent
        If DownloadProgressChanged = #True
          DownloadProgressChanged = #False
          StatusBarProgress(StatusBar, #Statusbar_Progress, DownloadProgress / 2)
        EndIf
      EndIf
      
    Case #State_DownloadingServerListFailed
      ShowError("Error on downloading update server list." + Chr(10) + "Update failed!")
      EndProgram()
      
    Case #State_ServerListDownloaded
      UpdateState = #State_GettingInfo
      If Mode <> #Mode_Silent
        StatusBarText(StatusBar, #Statusbar_Status, #Status_LoadInfo)
        StatusBarProgress(StatusBar, #Statusbar_Progress, 50)
        CreateThread(@GetUpdateInfos(), 0)
      EndIf
      
    Case #State_GettingInfo
      If Mode <> #Mode_Silent
        If DownloadProgressChanged = #True
          DownloadProgressChanged = #False
          StatusBarProgress(StatusBar, #Statusbar_Progress, DownloadProgress / 2 + 50)
        EndIf
      EndIf
      
    Case #State_GettingInfoFailed
      ShowError("Error on downloading update infos." + Chr(10) + "Update failed!")
      EndProgram()
      
    Case #State_GotInfo
      StatusBarProgress(StatusBar, #Statusbar_Progress, 100)
      CheckUpdates()
      Select Mode
        Case #Mode_Silent
          UpdateState = #State_StartUpdateing
        Case #Mode_DontAsk
          StatusBarText(StatusBar, #Statusbar_Status, #Status_Ready)
          StatusBarProgress(StatusBar, #Statusbar_Progress, 0)
          UpdateState = #State_StartUpdateing
        Case #Mode_Normal
          StatusBarText(StatusBar, #Statusbar_Status, #Status_Ready)
          StatusBarProgress(StatusBar, #Statusbar_Progress, 0)
          UpdateState = #State_WaitingForUser
      EndSelect
      
    Case #State_StartUpdateing
      

  EndSelect
  
EndProcedure

Procedure MainWndEventHandler(Event.i)
  
  Select Event
  
    Case #PB_Event_CloseWindow
      Exit = #True
      
    Case #PB_Event_SizeWindow
      MainWndResize()

  EndSelect
  
EndProcedure

Procedure MainEventLoop()
  
  Protected Event.i
  Protected EventWnd.i

  Repeat  
    
    Event = WindowEvent()

    If Event = #False
      
      Delay(20)
      UpdateLoop()

    Else
    
      EventWnd = EventWindow()
    
      Select EventWnd
      
        Case MainWnd
          MainWndEventHandler(Event)
          
      EndSelect

    EndIf
    
  Until Exit = #True
  
EndProcedure

Procedure Main()
  
  Protected ParamCnt.i
  Protected i.i
  
  ;Get the command line parameters.
  
  ParamCnt = CountProgramParameters() - 1
  
  For i = 0 To ParamCnt
  
    Select LCase(ProgramParameter(i))
        
      Case "-dontask", "-da"
        Mode = #Mode_DontAsk
        
      Case "-silent", "-s"
        Mode = #Mode_Silent
    
      Case "-force", "-f"
        ForceUpdate = #True
        
      Case "-onlyapply", "-oa"
        OnlyApply = #True
        
      Case "-onlydownload", "-od"
        OnlyDownload = #True
        
      Case "-selfreplace", "-sr"
        SelfReplace = #True
        
      Case "-singleserver", "-ss"
        SingleServer = #True
        
      Case "-server", "-svr"
        i + 1
        Server = ProgramParameter(i)
        
      Case "-console", "-c"
        UseConsole = #True

    EndSelect
    
  Next
  
  ;Get the path the updater (and GameFixer) is located in.
  SelfPath = GetSelfPath()
  
  If SelfReplace = #False
    
    SelfPath = Left(SelfPath, Len(SelfPath) - Len(#SystemFolder))
    
    ;Delete the temporary updater executable, thats there if the updater got updated.
    If FileSize(SelfPath + #UpdateFolder + #TempUpdaterFileName) <> -1
      Delay(1000)
      DeleteFile(SelfPath + #UpdateFolder + #TempUpdaterFileName)
    EndIf
    
  Else
    
    Delay(1000)
    SelfPath = Left(SelfPath, Len(SelfPath) - Len(#UpdateFolder))
    
    If DeleteFile(SelfPath + #SystemFolder + #UpdaterFileName) = #False
      ErrorBox("Error: Failed to delete " + #UpdaterFileName + ".")
    EndIf
    If CopyFile(SelfPath + #UpdateFolder + #TempUpdaterFileName, SelfPath + #SystemFolder + #UpdaterFileName) = #False
      ErrorBox("Error: Failed to copy " + #TempUpdaterFileName + ".")
    EndIf
    
    If RunProgram(SelfPath + #SystemFolder + #UpdaterFileName, BuildUpdaterParams(), SelfPath + #SystemFolder) = #False
      ErrorBox("Error: Failed to run " + #UpdaterFileName + ".")
    EndIf
    
    End
    
  EndIf
  
  If Mode <> #Mode_Silent
    If UseConsole = #True
      InitConsole()
    Else
      InitGui()
    EndIf
  EndIf
  
  Rmgr_InitManager("", 0, 0, 0)
  GetServerList()
  GetInstalledPlugins()
  GetInstalledVersions()
  
  OutputMutex = CreateMutex()
  
  If OnlyApply = #True
    
    If UseConsole = #True
      ;ApplyUpdates()
    EndIf
    
  Else
    
    If InitNetwork() = #False
      ShowError("Error: Failed to initialize network!")
      EndProgram()
    EndIf
    
    If Mode <> #Mode_Silent And UseConsole = #False
      MainEventLoop()
    Else
      ;OnlineUpdate()
    EndIf
    
  EndIf
  
  WriteN("")
  WriteN("Update finished.")
  EndProgram()
  
EndProcedure

Main()
End

; IDE Options = PureBasic 5.20 LTS (Windows - x64)
; CursorPosition = 287
; FirstLine = 174
; Folding = AEAAAg
; EnableThread
; EnableXP
; EnableUser
; Executable = B:\GameFixer\System\Updater.exe
; Compiler = PureBasic 5.20 LTS (Windows - x86)
; IncludeVersionInfo
; VersionField0 = 0,2,0,0
; VersionField1 = 0,2,0,0
; VersionField3 = GameFixer Updater
; VersionField4 = 0.02
; VersionField5 = 0.02
; VersionField9 = Janos David Ommert
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_APP